@@ -10,7 +10,7 @@
 #include <time.h>
 #include "gettext.h"
 #include "datatrak.h"
-#include "dive.h"
+#include "subsurface-string.h"
 #include "units.h"
 #include "device.h"
 #include "file.h"
