@@ -2,6 +2,8 @@
 #ifndef DIVE_H
 #define DIVE_H
 
+// dive and dive computer related structures and helpers
+
 #include <stdlib.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -12,57 +14,6 @@
 #include <string.h>
 #include <sys/stat.h>
 #include "divesite.h"
-
-/* Windows has no MIN/MAX macros - so let's just roll our own */
-#define MIN(x, y) ({                \
-	__typeof__(x) _min1 = (x);          \
-	__typeof__(y) _min2 = (y);          \
-	(void) (&_min1 == &_min2);      \
-	_min1 < _min2 ? _min1 : _min2; })
-
-#define MAX(x, y) ({                \
-	__typeof__(x) _max1 = (x);          \
-	__typeof__(y) _max2 = (y);          \
-	(void) (&_max1 == &_max2);      \
-	_max1 > _max2 ? _max1 : _max2; })
-
-#define IS_FP_SAME(_a, _b) (fabs((_a) - (_b)) <= 0.000001 * MAX(fabs(_a), fabs(_b)))
-
-static inline bool same_string(const char *a, const char *b)
-{
-	return !strcmp(a ?: "", b ?: "");
-}
-
-static inline bool same_string_caseinsensitive(const char *a, const char *b)
-{
-	return !strcasecmp(a ?: "", b ?: "");
-}
-
-static inline bool empty_string(const char *s)
-{
-	return !s || !*s;
-}
-
-static inline bool includes_string_caseinsensitive(const char *haystack, const char *needle)
-{
-	if (!needle)
-		return 1; /* every string includes the NULL string */
-	if (!haystack)
-		return 0; /* nothing is included in the NULL string */
-	int len = strlen(needle);
-	while (*haystack) {
-		if (strncasecmp(haystack, needle, len))
-			return 1;
-		haystack++;
-	}
-	return 0;
-}
-
-static inline char *copy_string(const char *s)
-{
-	return (s && *s) ? strdup(s) : NULL;
-}
-
 #include <libxml/tree.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
@@ -76,10 +27,11 @@ extern "C" {
 
 extern int last_xml_version;
 
-enum dive_comp_type {OC, CCR, PSCR, FREEDIVE, NUM_DC_TYPE};	// Flags (Open-circuit and Closed-circuit-rebreather) for setting dive computer type
+enum divemode_t {OC, CCR, PSCR, FREEDIVE, NUM_DIVEMODE, UNDEF_COMP_TYPE};	// Flags (Open-circuit and Closed-circuit-rebreather) for setting dive computer type
 enum cylinderuse {OC_GAS, DILUENT, OXYGEN, NOT_USED, NUM_GAS_USE}; // The different uses for cylinders
 
 extern const char *cylinderuse_text[];
+extern const char *divemode_text_ui[];
 extern const char *divemode_text[];
 
 struct gasmix {
@@ -133,14 +85,13 @@ struct event {
 	int flags, value;
 	/* .. and this is our "extended" data for some event types */
 	union {
+		enum divemode_t divemode; // for divemode change events
 		/*
-		 * Currently only for gas switch events.
-		 *
 		 * NOTE! The index may be -1, which means "unknown". In that
 		 * case, the get_cylinder_index() function will give the best
 		 * match with the cylinders in the dive based on gasmix.
 		 */
-		struct {
+		struct { // for gas switch events
 			int index;
 			struct gasmix mix;
 		} gas;
@@ -182,7 +133,7 @@ struct gas_pressures {
 	double o2, n2, he;
 };
 
-extern void fill_pressures(struct gas_pressures *pressures, const double amb_pressure, const struct gasmix *mix, double po2, enum dive_comp_type dctype);
+extern void fill_pressures(struct gas_pressures *pressures, const double amb_pressure, const struct gasmix *mix, double po2, enum divemode_t dctype);
 
 extern void sanitize_gasmix(struct gasmix *mix);
 extern int gasmix_distance(const struct gasmix *a, const struct gasmix *b);
@@ -201,7 +152,7 @@ static inline int interpolate(int a, int b, int part, int whole)
 	/* It is doubtful that we actually need floating point for this, but whatever */
 	if (whole) {
 		double x = (double)a * (whole - part) + (double)b * part;
-		return lrint(x / whole);
+		return (int)lrint(x / whole);
 	}
 	return (a+b)/2;
 }
@@ -230,7 +181,7 @@ struct sample                         // BASE TYPE BYTES  UNITS    RANGE
 	volume_t sac;                     //            4  ml/min                          predefined SAC
 	bool in_deco;                     // bool       1    y/n      y/n                  this sample is part of deco
 	bool manually_entered;            // bool       1    y/n      y/n                  this sample was entered by the user,
-	                                  //                                               not calculated when planning a dive
+					  //                                               not calculated when planning a dive
 };	                                  // Total size of structure: 57 bytes, excluding padding at end
 
 struct divetag {
@@ -264,10 +215,13 @@ struct tag_entry *taglist_added(struct tag_entry *original_list, struct tag_entr
 void dump_taglist(const char *intro, struct tag_entry *tl);
 
 /*
- * Writes all divetags in tag_list to buffer, limited by the buffer's (len)gth.
- * Returns the characters written
+ * Writes all divetags form tag_list into internally allocated buffer
+ * Function returns pointer to allocated buffer
+ * Buffer contains comma separated list of tags names or null terminated string
+ *
+ * NOTE! The returned buffer must be freed once used.
  */
-int taglist_get_tagstring(struct tag_entry *tag_list, char *buffer, int len);
+char *taglist_get_tagstring(struct tag_entry *tag_list);
 
 /* cleans up a list: removes empty tags and duplicates */
 void taglist_cleanup(struct tag_entry **tag_list);
@@ -306,7 +260,7 @@ struct divecomputer {
 	depth_t maxdepth, meandepth;
 	temperature_t airtemp, watertemp;
 	pressure_t surface_pressure;
-	enum dive_comp_type divemode;	// dive computer type: OC(default) or CCR
+	enum divemode_t divemode;	// dive computer type: OC(default) or CCR
 	uint8_t no_o2sensors;		// rebreathers: number of O2 sensors used
 	int salinity; 			// kg per 10000 l
 	const char *model, *serial, *fw_version;
@@ -321,6 +275,7 @@ struct divecomputer {
 #define MAX_CYLINDERS (20)
 #define MAX_WEIGHTSYSTEMS (6)
 #define MAX_TANK_INFO (100)
+#define MAX_WS_INFO (100)
 #define W_IDX_PRIMARY 0
 #define W_IDX_SECONDARY 1
 
@@ -409,6 +364,10 @@ struct dive_components {
 	unsigned int weights : 1;
 };
 
+extern enum divemode_t get_current_divemode(struct divecomputer *dc, int time, struct event **evp, enum divemode_t *divemode);
+extern struct event *get_next_divemodechange(struct event **evd, bool update_pointer);
+extern enum divemode_t get_divemode_at_time(struct divecomputer *dc, int dtime, struct event **ev_dmc);
+
 /* picture list and methods related to dive picture handling */
 struct picture {
 	char *filename;
@@ -426,7 +385,6 @@ struct picture {
 	for (struct picture *picture = (_divestruct).picture_list; picture; picture = picture->next)
 
 extern struct picture *alloc_picture();
-extern struct picture *clone_picture(struct picture *src);
 extern bool dive_check_picture_time(struct dive *d, int shift_time, timestamp_t timestamp);
 extern void dive_create_picture(struct dive *d, const char *filename, int shift_time, bool match_all);
 extern void dive_add_picture(struct dive *d, struct picture *newpic);
@@ -496,7 +454,7 @@ static inline int rel_mbar_to_depth(int mbar, struct dive *dive)
 	if (dive->dc.salinity)
 		specific_weight = dive->dc.salinity / 10000.0 * 0.981;
 	/* whole mbar gives us cm precision */
-	cm = lrint(mbar / specific_weight);
+	cm = (int)lrint(mbar / specific_weight);
 	return cm * 10;
 }
 
@@ -515,7 +473,7 @@ static inline depth_t gas_mod(struct gasmix *mix, pressure_t po2_limit, struct d
 	depth_t rounded_depth;
 
 	double depth = (double) mbar_to_depth(po2_limit.mbar * 1000 / get_o2(mix), dive);
-	rounded_depth.mm = lrint(depth / roundto) * roundto;
+	rounded_depth.mm = (int)lrint(depth / roundto) * roundto;
 	return rounded_depth;
 }
 
@@ -525,8 +483,8 @@ static inline depth_t gas_mnd(struct gasmix *mix, depth_t end, struct dive *dive
 	pressure_t ppo2n2;
 	ppo2n2.mbar = depth_to_mbar(end.mm, dive);
 
-	int maxambient = lrint(ppo2n2.mbar / (1 - get_he(mix) / 1000.0));
-	rounded_depth.mm = lrint(((double)mbar_to_depth(maxambient, dive)) / roundto) * roundto;
+	int maxambient = (int)lrint(ppo2n2.mbar / (1 - get_he(mix) / 1000.0));
+	rounded_depth.mm = (int)lrint(((double)mbar_to_depth(maxambient, dive)) / roundto) * roundto;
 	return rounded_depth;
 }
 
@@ -789,6 +747,8 @@ extern struct dive *clone_dive(struct dive *s);
 
 extern void clear_table(struct dive_table *table);
 
+extern void alloc_samples(struct divecomputer *dc, int num);
+extern void free_samples(struct divecomputer *dc);
 extern struct sample *prepare_sample(struct divecomputer *dc);
 extern void finish_sample(struct divecomputer *dc);
 extern void add_sample_pressure(struct sample *sample, int sensor, int mbar);
@@ -893,9 +853,10 @@ struct deco_state {
 	int ci_pointing_to_guiding_tissue;
 	double gf_low_pressure_this_dive;
 	int deco_time;
+	bool icd_warning;
 };
 
-extern void add_segment(struct deco_state *ds, double pressure, const struct gasmix *gasmix, int period_in_seconds, int setpoint, const struct dive *dive, int sac);
+extern void add_segment(struct deco_state *ds, double pressure, const struct gasmix *gasmix, int period_in_seconds, int setpoint, enum divemode_t divemode, int sac);
 extern void clear_deco(struct deco_state *ds, double surface_pressure);
 extern void dump_tissues(struct deco_state *ds);
 extern void set_gf(short gflow, short gfhigh);
@@ -916,6 +877,7 @@ struct divedatapoint {
 	int setpoint;
 	bool entered;
 	struct divedatapoint *next;
+	enum divemode_t divemode;
 };
 
 struct diveplan {
@@ -932,7 +894,7 @@ struct diveplan {
 	int surface_interval;
 };
 
-struct divedatapoint *plan_add_segment(struct diveplan *diveplan, int duration, int depth, int cylinderid, int po2, bool entered);
+struct divedatapoint *plan_add_segment(struct diveplan *diveplan, int duration, int depth, int cylinderid, int po2, bool entered, enum divemode_t divemode);
 struct divedatapoint *create_dp(int time_incr, int depth, int cylinderid, int po2);
 #if DEBUG_PLAN
 void dump_plan(struct diveplan *diveplan);
@@ -956,7 +918,7 @@ static inline struct gasmix *get_gasmix(struct dive *dive, struct divecomputer *
 	if (!gasmix) {
 		int cyl = explicit_first_cylinder(dive, dc);
 		gasmix = &dive->cylinder[cyl].gasmix;
-		ev = dc ? dc->events : NULL;
+		ev = dc ? get_next_event(dc->events, "gaschange") : NULL;
 	}
 	while (ev && ev->time.seconds < time) {
 		gasmix = get_gasmix_from_event(dive, ev);
@@ -983,7 +945,7 @@ struct ws_info_t {
 	const char *name;
 	int grams;
 };
-extern struct ws_info_t ws_info[100];
+extern struct ws_info_t ws_info[MAX_WS_INFO];
 
 extern bool cylinder_nodata(const cylinder_t *cyl);
 extern bool cylinder_none(void *_data);
@@ -996,25 +958,13 @@ extern void reset_cylinders(struct dive *dive, bool track_gas);
 extern void dump_cylinders(struct dive *dive, bool verbose);
 #endif
 
-/*
- * String handling.
- */
-#define STRTOD_NO_SIGN 0x01
-#define STRTOD_NO_DOT 0x02
-#define STRTOD_NO_COMMA 0x04
-#define STRTOD_NO_EXPONENT 0x08
-extern double strtod_flags(const char *str, const char **ptr, unsigned int flags);
-
-#define STRTOD_ASCII (STRTOD_NO_COMMA)
-
-#define ascii_strtod(str, ptr) strtod_flags(str, ptr, STRTOD_ASCII)
-
 extern void set_userid(const char *user_id);
 extern void set_informational_units(const char *units);
 extern void set_git_prefs(const char *prefs);
 
-extern const char *get_dive_date_c_string(timestamp_t when);
+extern char *get_dive_date_c_string(timestamp_t when);
 extern void update_setpoint_events(struct dive *dive, struct divecomputer *dc);
+
 #ifdef __cplusplus
 }
 #endif
