@@ -2,8 +2,7 @@
 #include "core/gpslocation.h"
 #include "qt-models/gpslistmodel.h"
 #include "core/pref.h"
-#include "core/dive.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include <time.h>
 #include <unistd.h>
 #include <QDebug>
@@ -60,7 +59,7 @@ void GpsLocation::setGpsTimeThreshold(int seconds)
 {
 	if (m_GpsSource) {
 		m_GpsSource->setUpdateInterval(seconds * 1000);
-		status(QString("Set GPS service update interval to %1").arg(m_GpsSource->updateInterval()));
+		status(QString("Set GPS service update interval to %1 s").arg(m_GpsSource->updateInterval() / 1000));
 	}
 }
 
@@ -131,14 +130,23 @@ void GpsLocation::serviceEnable(bool toggle)
 
 QString GpsLocation::currentPosition()
 {
+	qDebug() << "current position requested";
 	if (!hasLocationsSource())
-		return tr("Unknown GPS location");
-	if (m_trackers.count() && m_trackers.lastKey() + 300 >= QDateTime::currentMSecsSinceEpoch() / 1000) {
-		// we can simply use the last position that we tracked
-		gpsTracker gt = m_trackers.last();
-		QString gpsString = printGPSCoords(gt.latitude.udeg, gt.longitude.udeg);
-		qDebug() << "returning last position" << gpsString;
-		return gpsString;
+		return tr("Unknown GPS location (no GPS source)");
+	if (m_trackers.count()) {
+		QDateTime lastFixTime =	QDateTime().fromMSecsSinceEpoch((m_trackers.lastKey() - gettimezoneoffset(m_trackers.lastKey())) * 1000);
+		QDateTime now = QDateTime::currentDateTime();
+		int delta = lastFixTime.secsTo(now);
+		qDebug() << "lastFixTime" << lastFixTime.toString() << "now" << now.toString() << "delta" << delta;
+		if (delta < 300) {
+			// we can simply use the last position that we tracked
+			gpsTracker gt = m_trackers.last();
+			QString gpsString = printGPSCoords(gt.latitude.udeg, gt.longitude.udeg);
+			qDebug() << "returning last position" << gpsString;
+			return gpsString;
+		} else {
+			qDebug() << "last position saved was too old" << lastFixTime.toString();
+		}
 	}
 	qDebug() << "requesting new GPS position";
 	m_GpsSource->requestUpdate();
@@ -165,15 +173,18 @@ void GpsLocation::newPosition(QGeoPositionInfo pos)
 	int64_t delta = (int64_t)pos.timestamp().toTime_t() + gettimezoneoffset() - lastTime;
 	if (!nr || waitingForPosition || delta > prefs.time_threshold ||
 	    lastCoord.distanceTo(pos.coordinate()) > prefs.distance_threshold) {
-		QString msg("received new position %1 after delta %2 threshold %3");
-		status(qPrintable(msg.arg(pos.coordinate().toString()).arg(delta).arg(prefs.time_threshold)));
+		QString msg("received new position %1 after delta %2 threshold %3 (now %4 last %5)");
+		status(qPrintable(msg.arg(pos.coordinate().toString()).arg(delta).arg(prefs.time_threshold).arg(pos.timestamp().toString()).arg(QDateTime().fromMSecsSinceEpoch(lastTime * 1000).toString())));
 		waitingForPosition = false;
+		acquiredPosition();
 		gpsTracker gt;
 		gt.when = pos.timestamp().toTime_t();
 		gt.when += gettimezoneoffset(gt.when);
 		gt.latitude.udeg = lrint(pos.coordinate().latitude() * 1000000);
 		gt.longitude.udeg = lrint(pos.coordinate().longitude() * 1000000);
 		addFixToStorage(gt);
+		gpsTracker gtNew = m_trackers.last();
+		qDebug() << "newest fix is now at" << QDateTime().fromMSecsSinceEpoch(gtNew.when - gettimezoneoffset(gtNew.when) * 1000).toString();
 	}
 }
 
@@ -455,15 +466,13 @@ void GpsLocation::clearGpsData()
 }
 #endif
 
-void GpsLocation::postError(QNetworkReply::NetworkError error)
+void GpsLocation::postError(QNetworkReply::NetworkError)
 {
-	Q_UNUSED(error);
 	status(QString("error when sending a GPS fix: %1").arg(reply->errorString()));
 }
 
-void GpsLocation::getUseridError(QNetworkReply::NetworkError error)
+void GpsLocation::getUseridError(QNetworkReply::NetworkError)
 {
-	Q_UNUSED(error);
 	status(QString("error when retrieving Subsurface webservice user id: %1").arg(reply->errorString()));
 }
 
