@@ -1,7 +1,9 @@
 // SPDX-License-Identifier: GPL-2.0
 /* linux.c */
 /* implements Linux specific functions */
+#include "ssrf.h"
 #include "dive.h"
+#include "subsurface-string.h"
 #include "display.h"
 #include "membuffer.h"
 #include <string.h>
@@ -18,10 +20,15 @@ const char linux_system_divelist_default_font[] = "Sans";
 const char *system_divelist_default_font = linux_system_divelist_default_font;
 double system_divelist_default_font_size = -1.0;
 
+void subsurface_OS_pref_setup(void)
+{
+	// nothing
+}
+
 bool subsurface_ignore_font(const char *font)
 {
 	// there are no old default fonts to ignore
-	(void)font;
+	UNUSED(font);
 	return false;
 }
 
