@@ -1,9 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-#ifdef __clang__
-// Clang has a bug on zero-initialization of C structs.
-#pragma clang diagnostic ignored "-Wmissing-field-initializers"
-#endif
-
+#include "ssrf.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -20,6 +16,7 @@
 #include "gettext.h"
 
 #include "dive.h"
+#include "subsurface-string.h"
 #include "divelist.h"
 #include "device.h"
 #include "membuffer.h"
@@ -28,14 +25,6 @@
 
 const char *saved_git_id = NULL;
 
-struct picture_entry_list {
-	void *data;
-	int len;
-	const char *hash;
-	struct picture_entry_list *next;
-};
-struct picture_entry_list *pel = NULL;
-
 struct keyword_action {
 	const char *keyword;
 	void (*fn)(char *, struct membuffer *, void *);
@@ -45,23 +34,6 @@ struct keyword_action {
 extern degrees_t parse_degrees(char *buf, char **end);
 git_blob *git_tree_entry_blob(git_repository *repo, const git_tree_entry *entry);
 
-static void save_picture_from_git(struct picture *picture)
-{
-	struct picture_entry_list *pic_entry = pel;
-
-	while (pic_entry) {
-		char *hash = hashstring(picture->filename);
-		if (same_string(pic_entry->hash, hash)) {
-			savePictureLocal(picture, hash, pic_entry->data, pic_entry->len);
-			free(hash);
-			return;
-		}
-		free(hash);
-		pic_entry = pic_entry->next;
-	}
-	fprintf(stderr, "didn't find picture entry for %s\n", picture->filename);
-}
-
 static char *get_utf8(struct membuffer *b)
 {
 	int len = b->len;
@@ -161,9 +133,9 @@ static duration_t get_duration(const char *line)
 	return d;
 }
 
-static enum dive_comp_type get_dctype(const char *line)
+static enum divemode_t get_dctype(const char *line)
 {
-	for (enum dive_comp_type i = 0; i < NUM_DC_TYPE; i++) {
+	for (enum divemode_t i = 0; i < NUM_DIVEMODE; i++) {
 		if (strcmp(line, divemode_text[i]) == 0)
 			return i;
 	}
@@ -178,7 +150,7 @@ static int get_hex(const char *line)
 
 static void parse_dive_gps(char *line, struct membuffer *str, void *_dive)
 {
-	(void) str;
+	UNUSED(str);
 	uint32_t uuid;
 	degrees_t latitude = parse_degrees(line, &line);
 	degrees_t longitude = parse_degrees(line, &line);
@@ -205,7 +177,7 @@ static void parse_dive_gps(char *line, struct membuffer *str, void *_dive)
 
 static void parse_dive_location(char *line, struct membuffer *str, void *_dive)
 {
-	(void) line;
+	UNUSED(line);
 	uint32_t uuid;
 	char *name = get_utf8(str);
 	struct dive *dive = _dive;
@@ -229,19 +201,19 @@ static void parse_dive_location(char *line, struct membuffer *str, void *_dive)
 }
 
 static void parse_dive_divemaster(char *line, struct membuffer *str, void *_dive)
-{ (void) line; struct dive *dive = _dive; dive->divemaster = get_utf8(str); }
+{ UNUSED(line); struct dive *dive = _dive; dive->divemaster = get_utf8(str); }
 
 static void parse_dive_buddy(char *line, struct membuffer *str, void *_dive)
-{ (void) line; struct dive *dive = _dive; dive->buddy = get_utf8(str); }
+{ UNUSED(line); struct dive *dive = _dive; dive->buddy = get_utf8(str); }
 
 static void parse_dive_suit(char *line, struct membuffer *str, void *_dive)
-{ (void) line; struct dive *dive = _dive; dive->suit = get_utf8(str); }
+{ UNUSED(line); struct dive *dive = _dive; dive->suit = get_utf8(str); }
 
 static void parse_dive_notes(char *line, struct membuffer *str, void *_dive)
-{ (void) line; struct dive *dive = _dive; dive->notes = get_utf8(str); }
+{ UNUSED(line); struct dive *dive = _dive; dive->notes = get_utf8(str); }
 
 static void parse_dive_divesiteid(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->dive_site_uuid = get_hex(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->dive_site_uuid = get_hex(line); }
 
 /*
  * We can have multiple tags in the membuffer. They are separated by
@@ -249,7 +221,7 @@ static void parse_dive_divesiteid(char *line, struct membuffer *str, void *_dive
  */
 static void parse_dive_tags(char *line, struct membuffer *str, void *_dive)
 {
-	(void) line;
+	UNUSED(line);
 	struct dive *dive = _dive;
 	const char *tag;
 	int len = str->len;
@@ -272,40 +244,40 @@ static void parse_dive_tags(char *line, struct membuffer *str, void *_dive)
 }
 
 static void parse_dive_airtemp(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->airtemp = get_temperature(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->airtemp = get_temperature(line); }
 
 static void parse_dive_watertemp(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->watertemp = get_temperature(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->watertemp = get_temperature(line); }
 
 static void parse_dive_duration(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->duration = get_duration(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->duration = get_duration(line); }
 
 static void parse_dive_rating(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->rating = get_index(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->rating = get_index(line); }
 
 static void parse_dive_visibility(char *line, struct membuffer *str, void *_dive)
-{ (void) str; struct dive *dive = _dive; dive->visibility = get_index(line); }
+{ UNUSED(str); struct dive *dive = _dive; dive->visibility = get_index(line); }
 
 static void parse_dive_notrip(char *line, struct membuffer *str, void *_dive)
 {
-	(void) str;
-	(void) line;
+	UNUSED(str);
+	UNUSED(line);
 	struct dive *dive = _dive; dive->tripflag = NO_TRIP;
 }
 
 static void parse_site_description(char *line, struct membuffer *str, void *_ds)
-{ (void) line; struct dive_site *ds = _ds; ds->description = strdup(mb_cstring(str)); }
+{ UNUSED(line); struct dive_site *ds = _ds; ds->description = strdup(mb_cstring(str)); }
 
 static void parse_site_name(char *line, struct membuffer *str, void *_ds)
-{ (void) line; struct dive_site *ds = _ds; ds->name = strdup(mb_cstring(str)); }
+{ UNUSED(line); struct dive_site *ds = _ds; ds->name = strdup(mb_cstring(str)); }
 
 static void parse_site_notes(char *line, struct membuffer *str, void *_ds)
-{ (void) line; struct dive_site *ds = _ds; ds->notes = strdup(mb_cstring(str)); }
+{ UNUSED(line); struct dive_site *ds = _ds; ds->notes = strdup(mb_cstring(str)); }
 
 extern degrees_t parse_degrees(char *buf, char **end);
 static void parse_site_gps(char *line, struct membuffer *str, void *_ds)
 {
-	(void) str;
+	UNUSED(str);
 	struct dive_site *ds = _ds;
 
 	ds->latitude = parse_degrees(line, &line);
@@ -675,52 +647,61 @@ static void sample_parser(char *line, struct divecomputer *dc)
 }
 
 static void parse_dc_airtemp(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->airtemp = get_temperature(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->airtemp = get_temperature(line); }
 
 static void parse_dc_date(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; update_date(&dc->when, line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; update_date(&dc->when, line); }
 
 static void parse_dc_deviceid(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; set_dc_deviceid(dc, get_hex(line)); }
+{ UNUSED(str); struct divecomputer *dc = _dc; set_dc_deviceid(dc, get_hex(line)); }
 
 static void parse_dc_diveid(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->diveid = get_hex(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->diveid = get_hex(line); }
 
 static void parse_dc_duration(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->duration = get_duration(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->duration = get_duration(line); }
 
 static void parse_dc_dctype(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->divemode = get_dctype(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->divemode = get_dctype(line); }
 
 static void parse_dc_lastmanualtime(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->last_manual_time = get_duration(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->last_manual_time = get_duration(line); }
 
 static void parse_dc_maxdepth(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->maxdepth = get_depth(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->maxdepth = get_depth(line); }
 
 static void parse_dc_meandepth(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->meandepth = get_depth(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->meandepth = get_depth(line); }
 
 static void parse_dc_model(char *line, struct membuffer *str, void *_dc)
-{ (void) line; struct divecomputer *dc = _dc; dc->model = get_utf8(str); }
+{ UNUSED(line); struct divecomputer *dc = _dc; dc->model = get_utf8(str); }
 
 static void parse_dc_numberofoxygensensors(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->no_o2sensors = get_index(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->no_o2sensors = get_index(line); }
 
 static void parse_dc_surfacepressure(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->surface_pressure = get_pressure(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->surface_pressure = get_pressure(line); }
 
 static void parse_dc_salinity(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->salinity = get_salinity(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->salinity = get_salinity(line); }
 
 static void parse_dc_surfacetime(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->surfacetime = get_duration(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->surfacetime = get_duration(line); }
 
 static void parse_dc_time(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; update_time(&dc->when, line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; update_time(&dc->when, line); }
 
 static void parse_dc_watertemp(char *line, struct membuffer *str, void *_dc)
-{ (void) str; struct divecomputer *dc = _dc; dc->watertemp = get_temperature(line); }
+{ UNUSED(str); struct divecomputer *dc = _dc; dc->watertemp = get_temperature(line); }
+
+
+int get_divemode(const char *divemodestring) {
+	for (int i = 0; i < NUM_DIVEMODE; i++) {
+		if (!strcmp(divemodestring, divemode_text[i]))
+			return i;
+	}
+	return 0;
+}
 
 static void parse_event_keyvalue(void *_event, const char *key, const char *value)
 {
@@ -735,6 +716,8 @@ static void parse_event_keyvalue(void *_event, const char *key, const char *valu
 		event->value = val;
 	} else if (!strcmp(key, "name")) {
 		/* We get the name from the string handling */
+	} else if (!strcmp(key,"divemode")) {
+		event->value = get_divemode(value);
 	} else if (!strcmp(key, "cylinder")) {
 		/* NOTE! We add one here as a marker that "yes, we got a cylinder index" */
 		event->gas.index = 1+get_index(value);
@@ -820,37 +803,37 @@ static void parse_dc_event(char *line, struct membuffer *str, void *_dc)
 }
 
 static void parse_trip_date(char *line, struct membuffer *str, void *_trip)
-{ (void) str; dive_trip_t *trip = _trip; update_date(&trip->when, line); }
+{ UNUSED(str); dive_trip_t *trip = _trip; update_date(&trip->when, line); }
 
 static void parse_trip_time(char *line, struct membuffer *str, void *_trip)
-{ (void) str; dive_trip_t *trip = _trip; update_time(&trip->when, line); }
+{ UNUSED(str); dive_trip_t *trip = _trip; update_time(&trip->when, line); }
 
 static void parse_trip_location(char *line, struct membuffer *str, void *_trip)
-{ (void) line; dive_trip_t *trip = _trip; trip->location = get_utf8(str); }
+{ UNUSED(line); dive_trip_t *trip = _trip; trip->location = get_utf8(str); }
 
 static void parse_trip_notes(char *line, struct membuffer *str, void *_trip)
-{ (void) line; dive_trip_t *trip = _trip; trip->notes = get_utf8(str); }
+{ UNUSED(line); dive_trip_t *trip = _trip; trip->notes = get_utf8(str); }
 
 static void parse_settings_autogroup(char *line, struct membuffer *str, void *_unused)
 {
-	(void) line;
-	(void) str;
-	(void) _unused;
+	UNUSED(line);
+	UNUSED(str);
+	UNUSED(_unused);
 	set_autogroup(true);
 }
 
 static void parse_settings_units(char *line, struct membuffer *str, void *unused)
 {
-	(void) str;
-	(void) unused;
+	UNUSED(str);
+	UNUSED(unused);
 	if (line)
 		set_informational_units(line);
 }
 
 static void parse_settings_userid(char *line, struct membuffer *str, void *_unused)
 {
-	(void) str;
-	(void) _unused;
+	UNUSED(str);
+	UNUSED(_unused);
 	if (line) {
 		prefs.save_userid_local = true;
 		set_userid(line);
@@ -859,8 +842,8 @@ static void parse_settings_userid(char *line, struct membuffer *str, void *_unus
 
 static void parse_settings_prefs(char *line, struct membuffer *str, void *unused)
 {
-	(void) str;
-	(void) unused;
+	UNUSED(str);
+	UNUSED(unused);
 	if (line)
 		set_git_prefs(line);
 }
@@ -874,8 +857,8 @@ static void parse_settings_prefs(char *line, struct membuffer *str, void *unused
  */
 static void parse_settings_version(char *line, struct membuffer *str, void *_unused)
 {
-	(void) str;
-	(void) _unused;
+	UNUSED(str);
+	UNUSED(_unused);
 	int version = atoi(line);
 	report_datafile_version(version);
 	if (version > DATAFORMAT_VERSION)
@@ -885,9 +868,9 @@ static void parse_settings_version(char *line, struct membuffer *str, void *_unu
 /* The string in the membuffer is the version string of subsurface that saved things, just FYI */
 static void parse_settings_subsurface(char *line, struct membuffer *str, void *_unused)
 {
-	(void) line;
-	(void) str;
-	(void) _unused;
+	UNUSED(line);
+	UNUSED(str);
+	UNUSED(_unused);
 }
 
 struct divecomputerid {
@@ -935,7 +918,7 @@ static void parse_divecomputerid_keyvalue(void *_cid, const char *key, const cha
  */
 static void parse_settings_divecomputerid(char *line, struct membuffer *str, void *_unused)
 {
-	(void) _unused;
+	UNUSED(_unused);
 	struct divecomputerid id = { mb_cstring(str) };
 
 	id.cstr = id.model + strlen(id.model) + 1;
@@ -956,14 +939,14 @@ static void parse_settings_divecomputerid(char *line, struct membuffer *str, voi
 
 static void parse_picture_filename(char *line, struct membuffer *str, void *_pic)
 {
-	(void) line;
+	UNUSED(line);
 	struct picture *pic = _pic;
 	pic->filename = get_utf8(str);
 }
 
 static void parse_picture_gps(char *line, struct membuffer *str, void *_pic)
 {
-	(void) str;
+	UNUSED(str);
 	struct picture *pic = _pic;
 
 	pic->latitude = parse_degrees(line, &line);
@@ -972,7 +955,7 @@ static void parse_picture_gps(char *line, struct membuffer *str, void *_pic)
 
 static void parse_picture_hash(char *line, struct membuffer *str, void *_pic)
 {
-	(void) line;
+	UNUSED(line);
 	struct picture *pic = _pic;
 	char *hash = get_utf8(str);
 	register_hash(pic->filename, get_utf8(str));
@@ -1044,7 +1027,7 @@ static struct keyword_action settings_action[] = {
 
 static void settings_parser(char *line, struct membuffer *str, void *_unused)
 {
-	(void) _unused;
+	UNUSED(_unused);
 	match_action(line, str, NULL, settings_action, ARRAY_SIZE(settings_action));
 }
 
@@ -1205,18 +1188,6 @@ static void finish_active_dive(void)
 	struct dive *dive = active_dive;
 
 	if (dive) {
-		/* check if we need to save pictures */
-		FOR_EACH_PICTURE(dive) {
-			if (!picture_exists(picture))
-				save_picture_from_git(picture);
-		}
-		/* free any memory we allocated to track pictures */
-		while (pel) {
-			free(pel->data);
-			void *lastone = pel;
-			pel = pel->next;
-			free(lastone);
-		}
 		active_dive = NULL;
 		record_dive(dive);
 	}
@@ -1371,8 +1342,8 @@ static int dive_directory(const char *root, const git_tree_entry *entry, const c
 
 static int picture_directory(const char *root, const char *name)
 {
-	(void) root;
-	(void) name;
+	UNUSED(root);
+	UNUSED(name);
 	if (!active_dive)
 		return GIT_WALK_SKIP;
 	return GIT_WALK_OK;
@@ -1509,7 +1480,7 @@ static struct divecomputer *create_new_dc(struct dive *dive)
  */
 static int parse_divecomputer_entry(git_repository *repo, const git_tree_entry *entry, const char *suffix)
 {
-	(void) suffix;
+	UNUSED(suffix);
 	git_blob *blob = git_tree_entry_blob(repo, entry);
 
 	if (!blob)
@@ -1578,26 +1549,6 @@ static int parse_settings_entry(git_repository *repo, const git_tree_entry *entr
 	return 0;
 }
 
-static int parse_picture_file(git_repository *repo, const git_tree_entry *entry, const char *name)
-{
-	/* remember the picture data so we can handle it when all dive data has been loaded
-	 * the name of the git file is PIC-<hash> */
-	git_blob *blob = git_tree_entry_blob(repo, entry);
-	if (blob) {
-		const void*rawdata = git_blob_rawcontent(blob);
-		int len = git_blob_rawsize(blob);
-		struct picture_entry_list *new_pel = malloc(sizeof(struct picture_entry_list));
-		new_pel->next = pel;
-		pel = new_pel;
-		pel->data = malloc(len);
-		memcpy(pel->data, rawdata, len);
-		pel->len = len;
-		pel->hash = strdup(name + 4);
-		git_blob_free(blob);
-	}
-	return 0;
-}
-
 static int parse_picture_entry(git_repository *repo, const git_tree_entry *entry, const char *name)
 {
 	git_blob *blob;
@@ -1639,7 +1590,6 @@ static int walk_tree_file(const char *root, const git_tree_entry *entry, git_rep
 	if (verbose > 1)
 		fprintf(stderr, "git load handling file %s\n", name);
 	switch (*name) {
-	/* Picture file? They are saved as time offsets in the dive */
 	case '-': case '+':
 		if (dive)
 			return parse_picture_entry(repo, entry, name);
@@ -1660,10 +1610,6 @@ static int walk_tree_file(const char *root, const git_tree_entry *entry, git_rep
 		if (!strcmp(name, "00-Subsurface"))
 			return parse_settings_entry(repo, entry);
 		break;
-	case 'P':
-		if (dive && !strncmp(name, "PIC-", 4))
-			return parse_picture_file(repo, entry, name);
-		break;
 	}
 	report_error("Unknown file %s%s (%p %p)", root, name, dive, trip);
 	return GIT_WALK_SKIP;
