@@ -4,6 +4,7 @@
 #pragma clang diagnostic ignored "-Wmissing-field-initializers"
 #endif
 
+#include "ssrf.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -22,6 +23,7 @@
 #include "gettext.h"
 
 #include "dive.h"
+#include "subsurface-string.h"
 #include "parse.h"
 #include "divelist.h"
 #include "device.h"
@@ -403,8 +405,8 @@ static void gasmix(char *buffer, fraction_t *fraction)
 
 static void gasmix_nitrogen(char *buffer, struct gasmix *gasmix)
 {
-	(void) buffer;
-	(void) gasmix;
+	UNUSED(buffer);
+	UNUSED(gasmix);
 	/* Ignore n2 percentages. There's no value in them. */
 }
 
@@ -427,7 +429,7 @@ static void event_name(char *buffer, char *name)
 {
 	int size = trimspace(buffer);
 	if (size >= MAX_EVENT_NAME)
-		size = MAX_EVENT_NAME-1;
+		size = MAX_EVENT_NAME - 1;
 	memcpy(name, buffer, size);
 	name[size] = 0;
 }
@@ -436,10 +438,10 @@ static void event_name(char *buffer, char *name)
 const char *libdc_divemode_text[] = { "oc", "cc", "pscr", "freedive", "gauge"};
 
 /* Extract the dive computer type from the xml text buffer */
-static void get_dc_type(char *buffer, enum dive_comp_type *dct)
+static void get_dc_type(char *buffer, enum divemode_t *dct)
 {
 	if (trimspace(buffer)) {
-		for (enum dive_comp_type i = 0; i < NUM_DC_TYPE; i++) {
+		for (enum divemode_t i = 0; i < NUM_DIVEMODE; i++) {
 			if (strcmp(buffer, divemode_text[i]) == 0)
 				*dct = i;
 			else if (strcmp(buffer, libdc_divemode_text[i]) == 0)
@@ -448,6 +450,24 @@ static void get_dc_type(char *buffer, enum dive_comp_type *dct)
 	}
 }
 
+/* For divemode_text[] (defined in dive.h) determine the index of
+ * the string contained in the xml divemode attribute and passed
+ * in buffer, below. Typical xml input would be:
+ * <event name='modechange' divemode='OC' /> */
+static void event_divemode(char *buffer, int *value)
+{
+	int size = trimspace(buffer);
+	if (size >= MAX_EVENT_NAME)
+		size = MAX_EVENT_NAME - 1;
+	buffer[size] = 0x0;
+	for (int i = 0; i < NUM_DIVEMODE; i++) {
+		if (!strcmp(buffer,divemode_text[i])) {
+			*value = i;
+			break;
+		}
+	}
+}
+
 #define MATCH(pattern, fn, dest) ({ 			\
 	/* Silly type compatibility test */ 		\
 	if (0) (fn)("test", dest);			\
@@ -707,6 +727,8 @@ static void try_to_fill_event(const char *name, char *buf)
 		return;
 	if (MATCH("value", get_index, &cur_event.value))
 		return;
+	if (MATCH("divemode", event_divemode, &cur_event.value))
+		return;
 	if (MATCH("cylinder", get_index, &cur_event.gas.index)) {
 		/* We add one to indicate that we got an actual cylinder index value */
 		cur_event.gas.index++;
@@ -904,7 +926,7 @@ static void try_to_fill_sample(struct sample *sample, const char *name, char *bu
 
 static void try_to_fill_userid(const char *name, char *buf)
 {
-	(void) name;
+	UNUSED(name);
 	if (prefs.save_userid_local)
 		set_userid(buf);
 }
@@ -1585,7 +1607,7 @@ static const char *preprocess_divelog_de(const char *buffer)
 int parse_xml_buffer(const char *url, const char *buffer, int size,
 		      struct dive_table *table, const char **params)
 {
-	(void) size;
+	UNUSED(size);
 	xmlDoc *doc;
 	const char *res = preprocess_divelog_de(buffer);
 	int ret = 0;
