@@ -87,6 +87,7 @@ struct preferences {
 	bool gf_low_at_maxdepth;
 	bool show_ccr_setpoint;
 	bool show_ccr_sensors;
+	bool show_scr_ocpo2;
 	bool display_invalid_dives;
 	short unit_system;
 	struct units units;
@@ -153,6 +154,7 @@ struct preferences {
 	int distance_threshold;
 	bool git_local_only;
 	short cloud_timeout;
+	bool auto_recalculate_thumbnails;
 	locale_prefs_t locale; //: TODO: move the rest of locale based info here.
 	update_manager_prefs_t update_manager;
 	dive_computer_prefs_t dive_computer;
@@ -188,6 +190,7 @@ extern double system_divelist_default_font_size;
 extern const char *system_default_directory(void);
 extern const char *system_default_filename();
 extern bool subsurface_ignore_font(const char *font);
+extern void subsurface_OS_pref_setup();
 extern void copy_prefs(struct preferences *src, struct preferences *dest);
 
 #ifdef __cplusplus
