@@ -17,13 +17,10 @@
 #include "libdivecomputer.h"
 #include "core/qt-ble.h"
 #include "core/btdiscovery.h"
-
-#if defined(SSRF_CUSTOM_IO)
-
-#include <libdivecomputer/custom_io.h>
+#include "core/subsurface-string.h"
 
 #define BLE_TIMEOUT 12000 // 12 seconds seems like a very long time to wait
-#define DEBUG_THRESHOLD 20
+#define DEBUG_THRESHOLD 50
 static int debugCounter;
 
 #define IS_HW(_d) same_string((_d)->vendor, "Heinrichs Weikamp")
@@ -32,26 +29,32 @@ static int debugCounter;
 #define MAXIMAL_HW_CREDIT	255
 #define MINIMAL_HW_CREDIT	32
 
-extern "C" {
+#define WAITFOR(expression, ms) do {					\
+	Q_ASSERT(QCoreApplication::instance());				\
+	Q_ASSERT(QThread::currentThread());				\
+									\
+	if (expression)							\
+		break;							\
+	QElapsedTimer timer;						\
+	timer.start();							\
+									\
+	do {								\
+		QCoreApplication::processEvents(QEventLoop::AllEvents, ms); \
+		if (expression)						\
+			break;						\
+		QThread::msleep(10);					\
+	} while (timer.elapsed() < (ms));				\
+} while (0)
 
-void waitFor(int ms) {
-	Q_ASSERT(QCoreApplication::instance());
-	Q_ASSERT(QThread::currentThread());
-
-	QElapsedTimer timer;
-	timer.start();
-
-	do {
-		QCoreApplication::processEvents(QEventLoop::AllEvents, ms);
-		QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
-		QThread::msleep(10);
-	} while (timer.elapsed() < ms);
+static void waitFor(int ms)
+{
+	WAITFOR(false, ms);
 }
 
-void BLEObject::serviceStateChanged(QLowEnergyService::ServiceState s)
-{
-	Q_UNUSED(s)
+extern "C" {
 
+void BLEObject::serviceStateChanged(QLowEnergyService::ServiceState)
+{
 	QList<QLowEnergyCharacteristic> list;
 
 	auto service = qobject_cast<QLowEnergyService*>(sender());
@@ -93,10 +96,8 @@ void BLEObject::characteristicWritten(const QLowEnergyCharacteristic &c, const Q
 	}
 }
 
-void BLEObject::writeCompleted(const QLowEnergyDescriptor &d, const QByteArray &value)
+void BLEObject::writeCompleted(const QLowEnergyDescriptor&, const QByteArray&)
 {
-	Q_UNUSED(value)
-	Q_UNUSED(d)
 	qDebug() << "BLE write completed";
 }
 
@@ -150,10 +151,13 @@ BLEObject::~BLEObject()
 
 dc_status_t BLEObject::write(const void *data, size_t size, size_t *actual)
 {
-	Q_UNUSED(actual) // that seems like it might cause problems
+	if (actual) *actual = 0;
 
 	if (!receivedPackets.isEmpty()) {
 		qDebug() << ".. write HIT with still incoming packets in queue";
+		do {
+			receivedPackets.takeFirst();
+		} while (!receivedPackets.isEmpty());
 	}
 
 	QList<QLowEnergyCharacteristic> list = preferredService()->characteristics();
@@ -170,10 +174,8 @@ dc_status_t BLEObject::write(const void *data, size_t size, size_t *actual)
 			QLowEnergyService::WriteWithoutResponse :
 			QLowEnergyService::WriteWithResponse;
 
-	if (IS_SHEARWATER(device))
-		bytes.prepend("\1\0", 2);
-
 	preferredService()->writeCharacteristic(c, bytes, mode);
+	if (actual) *actual = size;
 	return DC_STATUS_SUCCESS;
 }
 
@@ -186,22 +188,13 @@ dc_status_t BLEObject::read(void *data, size_t size, size_t *actual)
 		if (list.isEmpty())
 			return DC_STATUS_IO;
 
-		int msec = BLE_TIMEOUT;
-		while (msec > 0 && receivedPackets.isEmpty()) {
-			waitFor(100);
-			msec -= 100;
-		};
+		WAITFOR(!receivedPackets.isEmpty(), BLE_TIMEOUT);
+		if (receivedPackets.isEmpty())
+			return DC_STATUS_IO;
 	}
 
-	// Still no packet?
-	if (receivedPackets.isEmpty())
-		return DC_STATUS_IO;
-
 	QByteArray packet = receivedPackets.takeFirst();
 
-	if (IS_SHEARWATER(device))
-		packet.remove(0,2);
-
 	if ((size_t)packet.size() > size)
 		return DC_STATUS_NOMEMORY;
 
@@ -236,7 +229,7 @@ dc_status_t BLEObject::setHwCredit(unsigned int c)
 	while (msec > 0 && !isCharacteristicWritten) {
 		waitFor(100);
 		msec -= 100;
-	};
+	}
 	if (!isCharacteristicWritten)
 		return DC_STATUS_TIMEOUT;
 	return DC_STATUS_SUCCESS;
@@ -278,9 +271,8 @@ dc_status_t BLEObject::setupHwTerminalIo(QList<QLowEnergyCharacteristic> allC)
 	return setHwCredit(MAXIMAL_HW_CREDIT);
 }
 
-dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *devaddr)
+dc_status_t qt_ble_open(void **io, dc_context_t *, const char *devaddr, dc_user_device_t *user_device)
 {
-	Q_UNUSED(context)
 	debugCounter = 0;
 	QLoggingCategory::setFilterRules(QStringLiteral("qt.bluetooth* = true"));
 
@@ -310,7 +302,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 #endif
 	qDebug() << "qt_ble_open(" << devaddr << ")";
 
-	if (IS_SHEARWATER(io->user_device))
+	if (IS_SHEARWATER(user_device))
 		controller->setRemoteAddressType(QLowEnergyController::RandomAddress);
 
 	// Try to connect to the device
@@ -321,7 +313,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	while (msec > 0 && controller->state() == QLowEnergyController::ConnectingState) {
 		waitFor(100);
 		msec -= 100;
-	};
+	}
 
 	switch (controller->state()) {
 	case QLowEnergyController::ConnectedState:
@@ -342,7 +334,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	// We need to discover services etc here!
 	// Note that ble takes ownership of controller and henceforth deleting ble will
 	// take care of deleting controller.
-	BLEObject *ble = new BLEObject(controller, io->user_device);
+	BLEObject *ble = new BLEObject(controller, user_device);
 	ble->connect(controller, SIGNAL(serviceDiscovered(QBluetoothUuid)), SLOT(addService(QBluetoothUuid)));
 
 	qDebug() << "  .. discovering services";
@@ -353,7 +345,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	while (msec > 0 && controller->state() == QLowEnergyController::DiscoveringState) {
 		waitFor(100);
 		msec -= 100;
-	};
+	}
 
 	qDebug() << " .. done discovering services";
 	if (ble->preferredService() == nullptr) {
@@ -368,7 +360,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	while (msec > 0 && ble->preferredService()->state() == QLowEnergyService::DiscoveringServices) {
 		waitFor(100);
 		msec -= 100;
-	};
+	}
 
 	if (ble->preferredService()->state() != QLowEnergyService::ServiceDiscovered) {
 		qDebug() << "failed to find suitable service on" << devaddr;
@@ -386,7 +378,7 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	if (!list.isEmpty()) {
 		const QLowEnergyCharacteristic &c = list.constLast();
 
-		if (IS_HW(io->user_device)) {
+		if (IS_HW(user_device)) {
 			dc_status_t r = ble->setupHwTerminalIo(list);
 			if (r != DC_STATUS_SUCCESS) {
 				delete ble;
@@ -422,15 +414,14 @@ dc_status_t qt_ble_open(dc_custom_io_t *io, dc_context_t *context, const char *d
 	}
 
 	// Fill in info
-	io->userdata = (void *)ble;
+	*io = (void *)ble;
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t qt_ble_close(dc_custom_io_t *io)
+dc_status_t qt_ble_close(void *io)
 {
-	BLEObject *ble = (BLEObject *) io->userdata;
+	BLEObject *ble = (BLEObject *) io;
 
-	io->userdata = NULL;
 	delete ble;
 
 	return DC_STATUS_SUCCESS;
@@ -443,19 +434,18 @@ static void checkThreshold()
 	}
 }
 
-dc_status_t qt_ble_read(dc_custom_io_t *io, void* data, size_t size, size_t *actual)
+dc_status_t qt_ble_read(void *io, void* data, size_t size, size_t *actual)
 {
 	checkThreshold();
-	BLEObject *ble = (BLEObject *) io->userdata;
+	BLEObject *ble = (BLEObject *) io;
 	return ble->read(data, size, actual);
 }
 
-dc_status_t qt_ble_write(dc_custom_io_t *io, const void* data, size_t size, size_t *actual)
+dc_status_t qt_ble_write(void *io, const void* data, size_t size, size_t *actual)
 {
 	checkThreshold();
-	BLEObject *ble = (BLEObject *) io->userdata;
+	BLEObject *ble = (BLEObject *) io;
 	return ble->write(data, size, actual);
 }
 
 } /* extern "C" */
-#endif
