@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "qthelper.h"
-#include "helpers.h"
+#include "subsurface-string.h"
+#include "subsurface-string.h"
 #include "gettextfromc.h"
 #include "statistics.h"
 #include "membuffer.h"
@@ -9,7 +10,11 @@
 #include "divecomputer.h"
 #include "time.h"
 #include "gettextfromc.h"
+#include "metadata.h"
 #include <sys/time.h>
+#include "exif.h"
+#include "file.h"
+#include "imagedownloader.h"
 #include "prefs-macros.h"
 #include <QFile>
 #include <QRegExp>
@@ -28,6 +33,7 @@
 #include <QFont>
 #include <QApplication>
 #include <QTextDocument>
+#include <QProgressDialog>	// TODO: remove with convertThumbnails()
 #include <cstdarg>
 #include <cstdint>
 
@@ -36,7 +42,6 @@
 const char *existing_filename;
 static QLocale loc;
 
-#define translate(_context, arg) trGettext(arg)
 static const QString DEGREE_SIGNS("dD" UTF8_DEGREE);
 
 QString weight_string(int weight_in_grams)
@@ -57,15 +62,15 @@ QString distance_string(int distanceInMeters)
 	QString str;
 	if(get_units()->length == units::METERS) {
 		if (distanceInMeters >= 1000)
-			str = QString(translate("gettextFromC", "%1km")).arg(distanceInMeters / 1000);
+			str = gettextFromC::tr("%1km").arg(distanceInMeters / 1000);
 		else
-			str = QString(translate("gettextFromC", "%1m")).arg(distanceInMeters);
+			str = gettextFromC::tr("%1m").arg(distanceInMeters);
 	} else {
 		double miles = m_to_mile(distanceInMeters);
 		if (miles >= 1.0)
-			str = QString(translate("gettextFromC", "%1mi")).arg((int)miles);
+			str = gettextFromC::tr("%1mi").arg((int)miles);
 		else
-			str = QString(translate("gettextFromC", "%1yd")).arg((int)(miles * 1760));
+			str = gettextFromC::tr("%1yd").arg((int)(miles * 1760));
 	}
 	return str;
 }
@@ -81,8 +86,8 @@ extern "C" const char *printGPSCoords(int lat, int lon)
 		return strdup("");
 
 	if (prefs.coordinates_traditional) {
-		lath = lat >= 0 ? translate("gettextFromC", "N") : translate("gettextFromC", "S");
-		lonh = lon >= 0 ? translate("gettextFromC", "E") : translate("gettextFromC", "W");
+		lath = lat >= 0 ? gettextFromC::tr("N") : gettextFromC::tr("S");
+		lonh = lon >= 0 ? gettextFromC::tr("E") : gettextFromC::tr("W");
 		lat = abs(lat);
 		lon = abs(lon);
 		latdeg = lat / 1000000U;
@@ -216,10 +221,10 @@ static bool parseSpecialCoords(const QString& txt, double& latitude, double& lon
 
 bool parseGpsText(const QString &gps_text, double *latitude, double *longitude)
 {
-	static const QString POS_LAT = QString("+N") + translate("gettextFromC", "N");
-	static const QString NEG_LAT = QString("-S") + translate("gettextFromC", "S");
-	static const QString POS_LON = QString("+E") + translate("gettextFromC", "E");
-	static const QString NEG_LON = QString("-W") + translate("gettextFromC", "W");
+	static const QString POS_LAT = QString("+N") + gettextFromC::tr("N");
+	static const QString NEG_LAT = QString("-S") + gettextFromC::tr("S");
+	static const QString POS_LON = QString("+E") + gettextFromC::tr("E");
+	static const QString NEG_LON = QString("-W") + gettextFromC::tr("W");
 
 	//remove the useless spaces (but keep the ones separating numbers)
 	static const QRegExp SPACE_CLEANER("\\s*([" + POS_LAT + NEG_LAT + POS_LON +
@@ -549,7 +554,7 @@ void set_filename(const char *filename)
 	existing_filename = copy_string(filename);
 }
 
-const QString get_dc_nickname(const char *model, uint32_t deviceid)
+QString get_dc_nickname(const char *model, uint32_t deviceid)
 {
 	const DiveComputerNode *existNode = dcList.getExact(model, deviceid);
 
@@ -563,10 +568,10 @@ QString get_depth_string(int mm, bool showunit, bool showdecimal)
 {
 	if (prefs.units.length == units::METERS) {
 		double meters = mm / 1000.0;
-		return QString("%L1%2").arg(meters, 0, 'f', (showdecimal && meters < 20.0) ? 1 : 0).arg(showunit ? translate("gettextFromC", "m") : "");
+		return QString("%L1%2").arg(meters, 0, 'f', (showdecimal && meters < 20.0) ? 1 : 0).arg(showunit ? gettextFromC::tr("m") : QString());
 	} else {
 		double feet = mm_to_feet(mm);
-		return QString("%L1%2").arg(feet, 0, 'f', 0).arg(showunit ? translate("gettextFromC", "ft") : "");
+		return QString("%L1%2").arg(feet, 0, 'f', 0).arg(showunit ? gettextFromC::tr("ft") : QString());
 	}
 }
 
@@ -578,18 +583,18 @@ QString get_depth_string(depth_t depth, bool showunit, bool showdecimal)
 QString get_depth_unit()
 {
 	if (prefs.units.length == units::METERS)
-		return QString("%1").arg(translate("gettextFromC", "m"));
+		return gettextFromC::tr("m");
 	else
-		return QString("%1").arg(translate("gettextFromC", "ft"));
+		return gettextFromC::tr("ft");
 }
 
 QString get_weight_string(weight_t weight, bool showunit)
 {
 	QString str = weight_string(weight.grams);
 	if (get_units()->weight == units::KG) {
-		str = QString("%1%2").arg(str).arg(showunit ? translate("gettextFromC", "kg") : "");
+		str = QString("%1%2").arg(str, showunit ? gettextFromC::tr("kg") : QString());
 	} else {
-		str = QString("%1%2").arg(str).arg(showunit ? translate("gettextFromC", "lbs") : "");
+		str = QString("%1%2").arg(str, showunit ? gettextFromC::tr("lbs") : QString());
 	}
 	return str;
 }
@@ -597,9 +602,9 @@ QString get_weight_string(weight_t weight, bool showunit)
 QString get_weight_unit()
 {
 	if (prefs.units.weight == units::KG)
-		return QString("%1").arg(translate("gettextFromC", "kg"));
+		return gettextFromC::tr("kg");
 	else
-		return QString("%1").arg(translate("gettextFromC", "lbs"));
+		return gettextFromC::tr("lbs");
 }
 
 QString get_temperature_string(temperature_t temp, bool showunit)
@@ -608,10 +613,10 @@ QString get_temperature_string(temperature_t temp, bool showunit)
 		return ""; //temperature not defined
 	} else if (prefs.units.temperature == units::CELSIUS) {
 		double celsius = mkelvin_to_C(temp.mkelvin);
-		return QString("%L1%2%3").arg(celsius, 0, 'f', 1).arg(showunit ? (UTF8_DEGREE) : "").arg(showunit ? translate("gettextFromC", "C") : "");
+		return QString("%L1%2%3").arg(celsius, 0, 'f', 1).arg(showunit ? (UTF8_DEGREE) : "").arg(showunit ? gettextFromC::tr("C") : QString());
 	} else {
 		double fahrenheit = mkelvin_to_F(temp.mkelvin);
-		return QString("%L1%2%3").arg(fahrenheit, 0, 'f', 1).arg(showunit ? (UTF8_DEGREE) : "").arg(showunit ? translate("gettextFromC", "F") : "");
+		return QString("%L1%2%3").arg(fahrenheit, 0, 'f', 1).arg(showunit ? (UTF8_DEGREE) : "").arg(showunit ? gettextFromC::tr("F") : QString());
 	}
 }
 
@@ -647,10 +652,10 @@ QString get_pressure_string(pressure_t pressure, bool showunit)
 {
 	if (prefs.units.pressure == units::BAR) {
 		double bar = pressure.mbar / 1000.0;
-		return QString("%L1%2").arg(bar, 0, 'f', 1).arg(showunit ? translate("gettextFromC", "bar") : "");
+		return QString("%L1%2").arg(bar, 0, 'f', 1).arg(showunit ? gettextFromC::tr("bar") : QString());
 	} else {
 		double psi = mbar_to_PSI(pressure.mbar);
-		return QString("%L1%2").arg(psi, 0, 'f', 0).arg(showunit ? translate("gettextFromC", "psi") : "");
+		return QString("%L1%2").arg(psi, 0, 'f', 0).arg(showunit ? gettextFromC::tr("psi") : QString());
 	}
 }
 
@@ -705,20 +710,6 @@ QString getPrintingTemplatePathBundle()
 	return path;
 }
 
-void copyPath(QString src, QString dst)
-{
-	QDir dir(src);
-	if (!dir.exists())
-		return;
-	foreach (QString d, dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot)) {
-		QString dst_path = dst + QDir::separator() + d;
-		dir.mkpath(dst_path);
-		copyPath(src + QDir::separator() + d, dst_path);
-	}
-	foreach (QString f, dir.entryList(QDir::Files))
-		QFile::copy(src + QDir::separator() + f, dst + QDir::separator() + f);
-}
-
 int gettimezoneoffset(timestamp_t when)
 {
 	QDateTime dt1, dt2;
@@ -953,7 +944,7 @@ QString get_dive_surfint_string(timestamp_t when, QString daysText, QString hour
 	mins = (when + 30 - days * 3600 * 24 - hrs * 3600) / 60;
 
 	QString displayInt;
-	if (maxdays && days > maxdays) displayInt = QString(translate("gettextFromC", "more than %1 days")).arg(maxdays);
+	if (maxdays && days > maxdays) displayInt = gettextFromC::tr("more than %1 days").arg(maxdays);
 	else if (days) displayInt = QString("%1%2%3%4%5%6%7%8").arg(days).arg(daysText).arg(separator)
 		.arg(hrs).arg(hoursText).arg(separator)
 		.arg(mins).arg(minutesText);
@@ -976,13 +967,13 @@ QString get_short_dive_date_string(timestamp_t when)
 	return loc.toString(ts.toUTC(), QString(prefs.date_format_short) + " " + prefs.time_format);
 }
 
-const char *get_dive_date_c_string(timestamp_t when)
+char *get_dive_date_c_string(timestamp_t when)
 {
 	QString text = get_dive_date_string(when);
 	return copy_qstring(text);
 }
 
-extern "C" const char *get_current_date()
+extern "C" char *get_current_date()
 {
 	QDateTime ts(QDateTime::currentDateTime());;
 	QString current_date;
@@ -1054,7 +1045,6 @@ extern "C" void reverseGeoLookup(degrees_t latitude, degrees_t longitude, uint32
 QHash<QString, QByteArray> hashOf;
 QMutex hashOfMutex;
 QHash<QByteArray, QString> localFilenameOf;
-QHash <QString, QImage > thumbnailCache;
 
 static QByteArray getHash(const QString &filename)
 {
@@ -1077,22 +1067,80 @@ const QString hashfile_name()
 	return QString(system_default_directory()).append("/hashes");
 }
 
+static QString thumbnailDir()
+{
+	return QString(system_default_directory()) + "/thumbnails/";
+}
+
+// Calculate thumbnail filename by hashing name of file.
+QString thumbnailFileName(const QString &filename)
+{
+	if (filename.isEmpty())
+		return QString();
+	QCryptographicHash hash(QCryptographicHash::Sha1);
+	hash.addData(filename.toUtf8());
+	return thumbnailDir() + hash.result().toHex();
+}
+
 extern "C" char *hashfile_name_string()
 {
 	return copy_qstring(hashfile_name());
 }
 
+// During a transition period, convert old thumbnail-hashes to individual files
+// TODO: remove this code in due course
+static void convertThumbnails(const QHash <QString, QImage> &thumbnails)
+{
+	if (thumbnails.empty())
+		return;
+	// This is a singular occurrence, therefore translating the strings seems not worth it
+	QProgressDialog progress("Convert thumbnails...", "Abort", 0, thumbnails.size());
+	progress.setWindowModality(Qt::WindowModal);
+
+	int count = 0;
+	for (const QString &name: thumbnails.keys()) {
+		const QImage thumbnail = thumbnails[name];
+
+		if (thumbnail.isNull())
+			continue;
+
+		// This is duplicate code (see qt-models/divepicturemodel.cpp)
+		// Not a problem, since this routine will be removed in due course.
+		QString filename = thumbnailFileName(name);
+		if (filename.isEmpty())
+			continue;
+
+		QSaveFile file(filename);
+		if (!file.open(QIODevice::WriteOnly))
+			return;
+		QDataStream stream(&file);
+
+		quint32 type = MEDIATYPE_PICTURE;
+		stream << type;
+		stream << thumbnail;
+		file.commit();
+
+		progress.setValue(++count);
+		if (progress.wasCanceled())
+			break;
+	}
+}
+
 void read_hashes()
 {
 	QFile hashfile(hashfile_name());
-	QMutexLocker locker(&hashOfMutex);
 	if (hashfile.open(QIODevice::ReadOnly)) {
 		QDataStream stream(&hashfile);
 		stream >> localFilenameOf;
+		QMutexLocker locker(&hashOfMutex);
 		stream >> hashOf;
-		stream >> thumbnailCache;
+		locker.unlock();
+		QHash <QString, QImage> thumbnailCache;
+		stream >> thumbnailCache;		// For backwards compatibility
 		hashfile.close();
+		convertThumbnails(thumbnailCache);
 	}
+	QMutexLocker locker(&hashOfMutex);
 	localFilenameOf.remove("");
 	QMutableHashIterator<QString, QByteArray> iter(hashOf);
 	while (iter.hasNext()) {
@@ -1100,6 +1148,9 @@ void read_hashes()
 		if (iter.value().isEmpty())
 			iter.remove();
 	}
+
+	// Make sure that the thumbnail directory exists
+	QDir().mkpath(thumbnailDir());
 }
 
 void write_hashes()
@@ -1111,7 +1162,7 @@ void write_hashes()
 		QDataStream stream(&hashfile);
 		stream << localFilenameOf;
 		stream << hashOf;
-		stream << thumbnailCache;
+		stream << QHash<QString,QImage>();	// Empty thumbnailCache - for backwards compatibility
 		hashfile.commit();
 	} else {
 		qWarning() << "Cannot open hashfile for writing: " << hashfile.fileName();
@@ -1164,12 +1215,6 @@ void learnHash(const QString &originalName, const QString &localName, const QByt
 	hashOf[originalName] = hash;
 }
 
-static bool haveHash(const QString &filename)
-{
-	QMutexLocker locker(&hashOfMutex);
-	return hashOf.contains(filename);
-}
-
 QString localFilePath(const QString &originalFilename)
 {
 	QMutexLocker locker(&hashOfMutex);
@@ -1180,23 +1225,13 @@ QString localFilePath(const QString &originalFilename)
 		return originalFilename;
 }
 
-// This needs to operate on a copy of picture as it frees it after finishing!
-void hashPicture(struct picture *picture)
+// This works on a copy of the string, because it runs in asynchronous context
+void hashPicture(QString filename)
 {
-	if (!picture)
-		return;
-	QByteArray oldHash = getHash(QString(picture->filename));
-	QByteArray hash = hashFile(localFilePath(picture->filename));
+	QByteArray oldHash = getHash(filename);
+	QByteArray hash = hashFile(localFilePath(filename));
 	if (!hash.isNull() && hash != oldHash)
 		mark_divelist_changed(true);
-	picture_free(picture);
-}
-
-extern "C" void cache_picture(struct picture *picture)
-{
-	QString filename = picture->filename;
-	if (!haveHash(filename))
-		QtConcurrent::run(hashPicture, clone_picture(picture));
 }
 
 QStringList imageExtensionFilters() {
@@ -1231,15 +1266,6 @@ extern "C" const char *local_file_path(struct picture *picture)
 	return copy_qstring(localFilePath(picture->filename));
 }
 
-extern "C" bool picture_exists(struct picture *picture)
-{
-	QString localPath = localFilePath(picture->filename);
-	if (localPath.isEmpty())
-		return false;
-	QByteArray hash = hashFile(localPath);
-	return !hash.isEmpty() && getHash(QString(picture->filename)) == hash;
-}
-
 const QString picturedir()
 {
 	return QString(system_default_directory()).append("/picturedata/");
@@ -1250,25 +1276,6 @@ extern "C" char *picturedir_string()
 	return copy_qstring(picturedir());
 }
 
-/* when we get a picture from git storage (local or remote) and can't find the picture
- * based on its hash, we create a local copy with the hash as filename and the appropriate
- * suffix */
-extern "C" void savePictureLocal(struct picture *picture, const char *hash, const char *data, int len)
-{
-	QString dirname = picturedir();
-	QDir localPictureDir(dirname);
-	localPictureDir.mkpath(dirname);
-	QString suffix(picture->filename);
-	suffix.replace(QRegularExpression(".*\\."), "");
-	QString filename(dirname + hash + "." + suffix);
-	QSaveFile out(filename);
-	if (out.open(QIODevice::WriteOnly)) {
-		out.write(data, len);
-		out.commit();
-		add_hash(filename, QByteArray::fromHex(hash));
-	}
-}
-
 QString get_gas_string(struct gasmix gas)
 {
 	uint o2 = (get_o2(&gas) + 5) / 10, he = (get_he(&gas) + 5) / 10;
@@ -1282,6 +1289,14 @@ QString get_divepoint_gas_string(struct dive *d, const divedatapoint &p)
 	return get_gas_string(d->cylinder[idx].gasmix);
 }
 
+QString get_taglist_string(struct tag_entry *tag_list)
+{
+	char *buffer = taglist_get_tagstring(tag_list);
+	QString ret = QString::fromUtf8(buffer);
+	free(buffer);
+	return ret;
+}
+
 weight_t string_to_weight(const char *str)
 {
 	const char *end;
@@ -1714,400 +1729,6 @@ extern "C" void unlock_planner()
 	planLock.unlock();
 }
 
-QString asprintf_loc(const char *cformat, ...)
-{
-	va_list ap;
-	va_start(ap, cformat);
-	QString res = vasprintf_loc(cformat, ap);
-	va_end(ap);
-	return res;
-}
-
-struct vasprintf_flags {
-	bool alternate_form : 1;	// TODO: unsupported
-	bool zero : 1;
-	bool left : 1;
-	bool space : 1;
-	bool sign : 1;
-	bool thousands : 1;		// ignored
-};
-
-enum length_modifier_t {
-	LM_NONE,
-	LM_CHAR,
-	LM_SHORT,
-	LM_LONG,
-	LM_LONGLONG,
-	LM_LONGDOUBLE,
-	LM_INTMAX,
-	LM_SIZET,
-	LM_PTRDIFF
-};
-
-// Helper function to insert '+' or ' ' after last space
-static QString insert_sign(QString s, char sign)
-{
-	// For space we can take a shortcut: insert in front
-	if (sign == ' ')
-		return sign + s;
-	int size = s.size();
-	int pos;
-	for (pos = 0; pos < size && s[pos].isSpace(); ++pos)
-		;	// Pass
-	return s.left(pos) + '+' + s.mid(pos);
-}
-
-static QString fmt_string(const QString &s, vasprintf_flags flags, int field_width, int precision)
-{
-	int size = s.size();
-	if (precision >= 0 && size > precision)
-		return s.left(precision);
-	return flags.left ? s.leftJustified(field_width) :
-			    s.rightJustified(field_width);
-}
-
-// Formatting of integers and doubles using Qt's localized functions.
-// The code is somewhat complex because Qt doesn't support all stdio
-// format options, notably '+' and ' '.
-// TODO: Since this is a templated function, remove common code
-template <typename T>
-static QString fmt_int(T i, vasprintf_flags flags, int field_width, int precision, int base)
-{
-	// If precision is given, things are a bit different: we have to pad with zero *and* space.
-	// Therefore, treat this case separately.
-	if (precision > 1) {
-		// For negative numbers, increase precision by one, so that we get
-		// the correct number of printed digits
-		if (i < 0)
-			++precision;
-		QChar fillChar = '0';
-		QString res = QStringLiteral("%L1").arg(i, precision, base, fillChar);
-		if (i >= 0 && flags.space)
-			res = ' ' + res;
-		else if (i >= 0 && flags.sign)
-			res = '+' + res;
-		return fmt_string(res, flags, field_width, -1);
-	}
-
-	// If we have to prepend a '+' or a space character, remove that from the field width
-	char sign = 0;
-	if (i >= 0 && (flags.space || flags.sign) && field_width > 0) {
-		sign = flags.sign ? '+' : ' ';
-		--field_width;
-	}
-	if (flags.left)
-		field_width = -field_width;
-	QChar fillChar = flags.zero && !flags.left ? '0' : ' ';
-	QString res = QStringLiteral("%L1").arg(i, field_width, base, fillChar);
-	return sign ? insert_sign(res, sign) : res;
-}
-
-static QString fmt_float(double d, char type, vasprintf_flags flags, int field_width, int precision)
-{
-	// If we have to prepend a '+' or a space character, remove that from the field width
-	char sign = 0;
-	if (d >= 0.0 && (flags.space || flags.sign) && field_width > 0) {
-		sign = flags.sign ? '+' : ' ';
-		--field_width;
-	}
-	if (flags.left)
-		field_width = -field_width;
-	QChar fillChar = flags.zero && !flags.left ? '0' : ' ';
-	QString res = QStringLiteral("%L1").arg(d, field_width, type, precision, fillChar);
-	return sign ? insert_sign(res, sign) : res;
-}
-
-// Helper to extract integers from C-style format strings.
-// The default returned value, if no digits are found is 0.
-static int parse_fmt_int(const char **act)
-{
-	if (!isdigit(**act))
-		return 0;
-	int res = 0;
-	while (isdigit(**act)) {
-		res = res * 10 + **act - '0';
-		++(*act);
-	}
-	return res;
-}
-
-QString vasprintf_loc(const char *fmt, va_list ap)
-{
-	const char *act = fmt;
-	QString ret;
-	for (;;) {
-		// Get all bytes up to next '%' character and add them as UTF-8
-		const char *begin = act;
-		while (*act && *act != '%')
-			++act;
-		int len = act - begin;
-		if (len > 0)
-			ret += QString::fromUtf8(begin, len);
-
-		// We found either a '%' or the end of the format string
-		if (!*act)
-			break;
-		++act;	// Jump over '%'
-
-		if (*act == '%') {
-			++act;
-			ret += '%';
-			continue;
-		}
-		// Flags
-		vasprintf_flags flags = { 0 };
-		for (;; ++act) {
-			switch(*act) {
-			case '#':
-				flags.alternate_form = true;
-				continue;
-			case '0':
-				flags.zero = true;
-				continue;
-			case '-':
-				flags.left = true;
-				continue;
-			case ' ':
-				flags.space = true;
-				continue;
-			case '+':
-				flags.sign = true;
-				continue;
-			case '\'':
-				flags.thousands = true;
-				continue;
-			}
-			break;
-		}
-
-		// Field width
-		int field_width;
-		if (*act == '*') {
-			field_width = va_arg(ap, int);
-			++act;
-		} else {
-			field_width = parse_fmt_int(&act);
-		}
-
-		// Precision
-		int precision = -1;
-		if (*act == '.') {
-			++act;
-			if (*act == '*') {
-				precision = va_arg(ap, int);
-				++act;
-			} else {
-				precision = parse_fmt_int(&act);
-			}
-		}
-
-		// Length modifier
-		enum length_modifier_t length_modifier = LM_NONE;
-		switch(*act) {
-		case 'h':
-			++act;
-			length_modifier = LM_CHAR;
-			if (*act == 'h') {
-				length_modifier = LM_SHORT;
-				++act;
-			}
-			break;
-		case 'l':
-			++act;
-			length_modifier = LM_LONG;
-			if (*act == 'l') {
-				length_modifier = LM_LONGLONG;
-				++act;
-			}
-			break;
-		case 'q':
-			++act;
-			length_modifier = LM_LONGLONG;
-			break;
-		case 'L':
-			++act;
-			length_modifier = LM_LONGDOUBLE;
-			break;
-		case 'j':
-			++act;
-			length_modifier = LM_INTMAX;
-			break;
-		case 'z':
-		case 'Z':
-			++act;
-			length_modifier = LM_SIZET;
-			break;
-		case 't':
-			++act;
-			length_modifier = LM_PTRDIFF;
-			break;
-		}
-
-		char type = *act++;
-		// Bail out if we reached end of the format string
-		if (!type)
-			break;
-
-		int base = 10;
-		if (type == 'o')
-			base = 8;
-		else if (type == 'x' || type == 'X')
-			base = 16;
-
-		switch(type) {
-		case 'd': case 'i': {
-			switch(length_modifier) {
-			case LM_LONG:
-				ret += fmt_int(va_arg(ap, long), flags, field_width, precision, base);
-				break;
-			case LM_LONGLONG:
-				ret += fmt_int(va_arg(ap, long long), flags, field_width, precision, base);
-				break;
-			case LM_INTMAX:
-				ret += fmt_int(va_arg(ap, intmax_t), flags, field_width, precision, base);
-				break;
-			case LM_SIZET:
-				ret += fmt_int(va_arg(ap, ssize_t), flags, field_width, precision, base);
-				break;
-			case LM_PTRDIFF:
-				ret += fmt_int(va_arg(ap, ptrdiff_t), flags, field_width, precision, base);
-				break;
-			case LM_CHAR:
-				// char is promoted to int when passed through '...'
-				ret += fmt_int(static_cast<char>(va_arg(ap, int)), flags, field_width, precision, base);
-				break;
-			case LM_SHORT:
-				// short is promoted to int when passed through '...'
-				ret += fmt_int(static_cast<short>(va_arg(ap, int)), flags, field_width, precision, base);
-				break;
-			default:
-				ret += fmt_int(va_arg(ap, int), flags, field_width, precision, base);
-				break;
-			}
-			break;
-		}
-		case 'o': case 'u': case 'x': case 'X': {
-			QString s;
-			switch(length_modifier) {
-			case LM_LONG:
-				s = fmt_int(va_arg(ap, unsigned long), flags, field_width, precision, base);
-				break;
-			case LM_LONGLONG:
-				s = fmt_int(va_arg(ap, unsigned long long), flags, field_width, precision, base);
-				break;
-			case LM_INTMAX:
-				s = fmt_int(va_arg(ap, uintmax_t), flags, field_width, precision, base);
-				break;
-			case LM_SIZET:
-				s = fmt_int(va_arg(ap, size_t), flags, field_width, precision, base);
-				break;
-			case LM_PTRDIFF:
-				s = fmt_int(va_arg(ap, ptrdiff_t), flags, field_width, precision, base);
-				break;
-			case LM_CHAR:
-				// char is promoted to int when passed through '...'
-				s = fmt_int(static_cast<unsigned char>(va_arg(ap, int)), flags, field_width, precision, base);
-				break;
-			case LM_SHORT:
-				// short is promoted to int when passed through '...'
-				s = fmt_int(static_cast<unsigned short>(va_arg(ap, int)), flags, field_width, precision, base);
-				break;
-			default:
-				s = fmt_int(va_arg(ap, unsigned int), flags, field_width, precision, base);
-				break;
-			}
-			if (type == 'X')
-				s = s.toUpper();
-			ret += s;
-			break;
-		}
-		case 'e': case 'E': case 'f': case 'F': case 'g': case 'G': {
-			// It seems that Qt is not able to format long doubles,
-			// therefore we have to cast down to double.
-			double f = length_modifier == LM_LONGDOUBLE ?
-				static_cast<double>(va_arg(ap, long double)) :
-				va_arg(ap, double);
-			ret += fmt_float(f, type, flags, field_width, precision);
-			break;
-		}
-		case 'c':
-			if (length_modifier == LM_LONG) {
-				// Cool, on some platforms wint_t is short, on some int.
-#if WINT_MAX < UINT_MAX
-				wint_t wc = static_cast<wint_t>(va_arg(ap, int));
-#else
-				wint_t wc = va_arg(ap, wint_t);
-#endif
-				ret += QChar(wc);
-			} else {
-				ret += static_cast<char>(va_arg(ap, int));
-			}
-			break;
-		case 's': {
-			QString s = length_modifier == LM_LONG ?
-				QString::fromWCharArray(va_arg(ap, wchar_t *)) :
-				QString::fromUtf8(va_arg(ap, char *));
-			ret += fmt_string(s, flags, field_width, precision);
-			break;
-		}
-		case 'p':
-			ret += QString("0x%1").arg(reinterpret_cast<long long>(va_arg(ap, void *)), field_width, 16);
-			break;
-		}
-	}
-	return ret;
-}
-
-// Put a formated string respecting the default locale into a C-style array in UTF-8 encoding.
-// The only complication arises from the fact that we don't want to cut through multi-byte UTF-8 code points.
-extern "C" int snprintf_loc(char *dst, size_t size, const char *cformat, ...)
-{
-	va_list ap;
-	va_start(ap, cformat);
-	int res = vsnprintf_loc(dst, size, cformat, ap);
-	va_end(ap);
-	return res;
-}
-
-extern "C" int vsnprintf_loc(char *dst, size_t size, const char *cformat, va_list ap)
-{
-	QByteArray utf8 = vasprintf_loc(cformat, ap).toUtf8();
-	const char *data = utf8.constData();
-	size_t utf8_size = utf8.size();
-	if (size == 0)
-		return utf8_size;
-	if (size < utf8_size + 1) {
-		memcpy(dst, data, size - 1);
-		if ((data[size - 1] & 0xC0) == 0x80) {
-			// We truncated a multi-byte UTF-8 encoding.
-			--size;
-			// Jump to last copied byte.
-			if (size > 0)
-				--size;
-			while(size > 0 && (dst[size] & 0xC0) == 0x80)
-				--size;
-			dst[size] = 0;
-		} else {
-			dst[size - 1] = 0;
-		}
-	} else {
-		memcpy(dst, data, utf8_size + 1); // QByteArray guarantees a trailing 0
-	}
-	return utf8_size;
-}
-
-// This function is defined here instead of membuffer.c, because it needs to access QString.
-extern "C" void put_vformat_loc(struct membuffer *b, const char *fmt, va_list args)
-{
-	QByteArray utf8 = vasprintf_loc(fmt, args).toUtf8();
-	const char *data = utf8.constData();
-	size_t utf8_size = utf8.size();
-
-	make_room(b, utf8_size);
-	memcpy(b->buffer + b->len, data, utf8_size);
-	b->len += utf8_size;
-}
-
 char *copy_qstring(const QString &s)
 {
 	return strdup(qPrintable(s));
