@@ -4,6 +4,7 @@
 #pragma clang diagnostic ignored "-Wmissing-field-initializers"
 #endif
 
+#include "ssrf.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -17,6 +18,7 @@
 #include <git2.h>
 
 #include "dive.h"
+#include "subsurface-string.h"
 #include "divelist.h"
 #include "device.h"
 #include "membuffer.h"
@@ -373,7 +375,11 @@ static void save_one_event(struct membuffer *b, struct dive *dive, struct event
 	put_format(b, "event %d:%02d", FRACTION(ev->time.seconds, 60));
 	show_index(b, ev->type, "type=", "");
 	show_index(b, ev->flags, "flags=", "");
-	show_index(b, ev->value, "value=", "");
+
+	if (!strcmp(ev->name,"modechange"))
+		show_utf8(b, " divemode=", divemode_text[ev->value], "");
+	else
+		show_index(b, ev->value, "value=", "");
 	show_utf8(b, " name=", ev->name, "");
 	if (event_is_gaschange(ev)) {
 		struct gasmix *mix = get_gasmix_from_event(dive, ev);
@@ -606,7 +612,6 @@ static int save_one_picture(git_repository *repo, struct dir *dir, struct pictur
 	char sign = '+';
 	char *hash;
 	unsigned h;
-	int error;
 
 	show_utf8(&buf, "filename ", pic->filename, "\n");
 	show_gps(&buf, pic->latitude, pic->longitude);
@@ -623,21 +628,8 @@ static int save_one_picture(git_repository *repo, struct dir *dir, struct pictur
 	/* Use full hh:mm:ss format to make it all sort nicely */
 	h = offset / 3600;
 	offset -= h *3600;
-	error = blob_insert(repo, dir, &buf, "%c%02u=%02u=%02u",
+	return blob_insert(repo, dir, &buf, "%c%02u=%02u=%02u",
 		sign, h, FRACTION(offset, 60));
-#if 0
-	/* storing pictures into git was a mistake. This makes for HUGE git repositories */
-	if (!error) {
-		/* next store the actual picture; we prefix all picture names
-		 * with "PIC-" to make things easier on the parsing side */
-		struct membuffer namebuf = { 0 };
-		const char *localfn = local_file_path(pic);
-		put_format(&namebuf, "PIC-%s", pic->hash);
-		error = blob_insert_fromdisk(repo, dir, localfn, mb_cstring(&namebuf));
-		free((void *)localfn);
-	}
-#endif
-	return error;
 }
 
 static int save_pictures(git_repository *repo, struct dir *dir, struct dive *dive)
@@ -1030,11 +1022,11 @@ static int notify_cb(git_checkout_notify_t why,
 	const git_diff_file *workdir,
 	void *payload)
 {
-	(void) baseline;
-	(void) target;
-	(void) workdir;
-	(void) payload;
-	(void) why;
+	UNUSED(baseline);
+	UNUSED(target);
+	UNUSED(workdir);
+	UNUSED(payload);
+	UNUSED(why);
 	report_error("File '%s' does not match in working tree", path);
 	return 0; /* Continue with checkout */
 }
@@ -1069,7 +1061,7 @@ static int get_authorship(git_repository *repo, git_signature **authorp)
 	/* Default name information, with potential OS overrides */
 	struct user_info user = {
 		.name = "Subsurface",
-		.email = "subsurace@subsurface-divelog.org"
+		.email = "subsurface-app-account@subsurface-divelog.org"
 	};
 
 	subsurface_user_info(&user);
