@@ -14,6 +14,7 @@
 #include <fcntl.h>
 
 #include "dive.h"
+#include "subsurface-string.h"
 #include "divelist.h"
 #include "device.h"
 #include "membuffer.h"
@@ -300,7 +301,10 @@ static void save_one_event(struct membuffer *b, struct dive *dive, struct event
 	put_format(b, "  <event time='%d:%02d min'", FRACTION(ev->time.seconds, 60));
 	show_index(b, ev->type, "type='", "'");
 	show_index(b, ev->flags, "flags='", "'");
-	show_index(b, ev->value, "value='", "'");
+	if (!strcmp(ev->name,"modechange"))
+		show_utf8(b, divemode_text[ev->value], " divemode='", "'",1);
+	else
+		show_index(b, ev->value, "value='", "'");
 	show_utf8(b, ev->name, " name='", "'", 1);
 	if (event_is_gaschange(ev)) {
 		struct gasmix *mix = get_gasmix_from_event(dive, ev);
@@ -398,7 +402,7 @@ static void save_dc(struct membuffer *b, struct dive *dive, struct divecomputer
 	if (dc->duration.seconds && dc->duration.seconds != dive->dc.duration.seconds)
 		put_duration(b, dc->duration, " duration='", " min'");
 	if (dc->divemode != OC) {
-		for (enum dive_comp_type i = 0; i < NUM_DC_TYPE; i++)
+		for (enum divemode_t i = 0; i < NUM_DIVEMODE; i++)
 			if (dc->divemode == i)
 				show_utf8(b, divemode_text[i], " dctype='", "'", 1);
 		if (dc->no_o2sensors)
