@@ -4,7 +4,6 @@
  * core logic for the Info & Stats page -
  * char *get_minutes(int seconds);
  * void process_all_dives(struct dive *dive, struct dive **prev_dive);
- * void get_selected_dives_text(char *buffer, int size);
  */
 #include "gettext.h"
 #include <string.h>
@@ -45,10 +44,10 @@ static void process_temperatures(struct dive *dp, stats_t *stats)
 	}
 }
 
-static void process_dive(struct dive *dp, stats_t *stats)
+static void process_dive(struct dive *dive, stats_t *stats)
 {
 	int old_tadt, sac_time = 0;
-	int32_t duration = dp->duration.seconds;
+	int32_t duration = dive->duration.seconds;
 
 	old_tadt = stats->total_average_depth_time.seconds;
 	stats->total_time.seconds += duration;
@@ -56,31 +55,31 @@ static void process_dive(struct dive *dp, stats_t *stats)
 		stats->longest_time.seconds = duration;
 	if (stats->shortest_time.seconds == 0 || duration < stats->shortest_time.seconds)
 		stats->shortest_time.seconds = duration;
-	if (dp->maxdepth.mm > stats->max_depth.mm)
-		stats->max_depth.mm = dp->maxdepth.mm;
-	if (stats->min_depth.mm == 0 || dp->maxdepth.mm < stats->min_depth.mm)
-		stats->min_depth.mm = dp->maxdepth.mm;
+	if (dive->maxdepth.mm > stats->max_depth.mm)
+		stats->max_depth.mm = dive->maxdepth.mm;
+	if (stats->min_depth.mm == 0 || dive->maxdepth.mm < stats->min_depth.mm)
+		stats->min_depth.mm = dive->maxdepth.mm;
 
-	process_temperatures(dp, stats);
+	process_temperatures(dive, stats);
 
 	/* Maybe we should drop zero-duration dives */
 	if (!duration)
 		return;
-	if (dp->meandepth.mm) {
+	if (dive->meandepth.mm) {
 		stats->total_average_depth_time.seconds += duration;
 		stats->avg_depth.mm = lrint((1.0 * old_tadt * stats->avg_depth.mm +
-					duration * dp->meandepth.mm) /
+					duration * dive->meandepth.mm) /
 					stats->total_average_depth_time.seconds);
 	}
-	if (dp->sac > 100) { /* less than .1 l/min is bogus, even with a pSCR */
+	if (dive->sac > 100) { /* less than .1 l/min is bogus, even with a pSCR */
 		sac_time = stats->total_sac_time.seconds + duration;
 		stats->avg_sac.mliter = lrint((1.0 * stats->total_sac_time.seconds * stats->avg_sac.mliter +
-					 duration * dp->sac) /
+					 duration * dive->sac) /
 					 sac_time);
-		if (dp->sac > stats->max_sac.mliter)
-			stats->max_sac.mliter = dp->sac;
-		if (stats->min_sac.mliter == 0 || dp->sac < stats->min_sac.mliter)
-			stats->min_sac.mliter = dp->sac;
+		if (dive->sac > stats->max_sac.mliter)
+			stats->max_sac.mliter = dive->sac;
+		if (stats->min_sac.mliter == 0 || dive->sac < stats->min_sac.mliter)
+			stats->min_sac.mliter = dive->sac;
 		stats->total_sac_time.seconds = sac_time;
 	}
 }
@@ -124,7 +123,7 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 	free(stats_by_type);
 
 	size = sizeof(stats_t) * (dive_table.nr + 1);
-	tsize = sizeof(stats_t) * (NUM_DC_TYPE + 1);
+	tsize = sizeof(stats_t) * (NUM_DIVEMODE + 1);
 	stats_yearly = malloc(size);
 	stats_monthly = malloc(size);
 	stats_by_trip = malloc(size);
@@ -141,13 +140,13 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 	 * field in the statistics window */
 	stats_by_type[0].location = strdup(translate("gettextFromC", "All (by type stats)"));
 	stats_by_type[0].is_trip = true;
-	stats_by_type[1].location = strdup(translate("gettextFromC","OC"));
+	stats_by_type[1].location = strdup(translate("gettextFromC", divemode_text_ui[OC]));
 	stats_by_type[1].is_trip = true;
-	stats_by_type[2].location = strdup(translate("gettextFromC","CCR"));
+	stats_by_type[2].location = strdup(translate("gettextFromC", divemode_text_ui[CCR]));
 	stats_by_type[2].is_trip = true;
-	stats_by_type[3].location = strdup(translate("gettextFromC","pSCR"));
+	stats_by_type[3].location = strdup(translate("gettextFromC", divemode_text_ui[PSCR]));
 	stats_by_type[3].is_trip = true;
-	stats_by_type[4].location = strdup(translate("gettextFromC","Freedive"));
+	stats_by_type[4].location = strdup(translate("gettextFromC", divemode_text_ui[FREEDIVE]));
 	stats_by_type[4].is_trip = true;
 
 	/* this relies on the fact that the dives in the dive_table
@@ -235,74 +234,6 @@ void process_selected_dives(void)
 	stats_selection.selection_size = nr;
 }
 
-/* this gets called when at least two but not all dives are selected */
-static void get_ranges(char *buffer, int size)
-{
-	int i, len;
-	int first = -1, last = -1;
-	struct dive *dive;
-
-	snprintf(buffer, size, "%s", translate("gettextFromC", "for dives #"));
-	for_each_dive (i, dive) {
-		if (!dive->selected)
-			continue;
-		if (dive->number < 1) {
-			/* uhh - weird numbers - bail */
-			snprintf(buffer, size, "%s", translate("gettextFromC", "for selected dives"));
-			return;
-		}
-		len = strlen(buffer);
-		if (last == -1) {
-			snprintf(buffer + len, size - len, "%d", dive->number);
-			first = last = dive->number;
-		} else {
-			if (dive->number == last + 1) {
-				last++;
-				continue;
-			} else {
-				if (first == last)
-					snprintf(buffer + len, size - len, ", %d", dive->number);
-				else if (first + 1 == last)
-					snprintf(buffer + len, size - len, ", %d, %d", last, dive->number);
-				else
-					snprintf(buffer + len, size - len, "-%d, %d", last, dive->number);
-				first = last = dive->number;
-			}
-		}
-	}
-	len = strlen(buffer);
-	if (first != last) {
-		if (first + 1 == last)
-			snprintf(buffer + len, size - len, ", %d", last);
-		else
-			snprintf(buffer + len, size - len, "-%d", last);
-	}
-}
-
-void get_selected_dives_text(char *buffer, size_t size)
-{
-	if (amount_selected == 1) {
-		if (current_dive)
-			snprintf(buffer, size, translate("gettextFromC", "for dive #%d"), current_dive->number);
-		else
-			snprintf(buffer, size, "%s", translate("gettextFromC", "for selected dive"));
-	} else if (amount_selected == (unsigned int)dive_table.nr) {
-		snprintf(buffer, size, "%s", translate("gettextFromC", "for all dives"));
-	} else if (amount_selected == 0) {
-		snprintf(buffer, size, "%s", translate("gettextFromC", "(no dives)"));
-	} else {
-		get_ranges(buffer, size);
-		if (strlen(buffer) == size - 1) {
-			/* add our own ellipse... the way Pango does this is ugly
-			 * as it will leave partial numbers there which I don't like */
-			size_t offset = 4;
-			while (offset < size && isdigit(buffer[size - offset]))
-				offset++;
-			strcpy(buffer + size - offset, "...");
-		}
-	}
-}
-
 #define SOME_GAS 5000 // 5bar drop in cylinder pressure makes cylinder used
 
 bool has_gaschange_event(struct dive *dive, struct divecomputer *dc, int idx)
