@@ -4,9 +4,9 @@
 #include <QDateTime>
 #include <QTextDocument>
 
-#include "../qthelper.h"
-#include "../helpers.h"
-#include "../../qt-models/tankinfomodel.h"
+#include "core/qthelper.h"
+#include "core/subsurface-string.h"
+#include "qt-models/tankinfomodel.h"
 
 static QString EMPTY_DIVE_STRING = QStringLiteral("");
 enum returnPressureSelector {START_PRESSURE, END_PRESSURE};
@@ -192,9 +192,7 @@ QString DiveObjectHelper::notes() const
 
 QString DiveObjectHelper::tags() const
 {
-	static char buffer[256];
-	taglist_get_tagstring(m_dive->tag_list, buffer, 256);
-	return QString(buffer);
+	return get_taglist_string(m_dive->tag_list);
 }
 
 QString DiveObjectHelper::gas() const
@@ -362,6 +360,12 @@ QString DiveObjectHelper::tripMeta() const
 	return ret;
 }
 
+int DiveObjectHelper::tripNrDives() const
+{
+	struct dive_trip *dt = m_dive->divetrip;
+	return dt ? dt->nrdives : 0;
+}
+
 int DiveObjectHelper::maxcns() const
 {
 	return m_dive->maxcns;
