@@ -2,7 +2,6 @@
 #ifndef DIVE_QOBJECT_H
 #define DIVE_QOBJECT_H
 
-#include "../dive.h"
 #include "CylinderObjectHelper.h"
 #include <QObject>
 #include <QString>
@@ -42,6 +41,7 @@ class DiveObjectHelper : public QObject {
 	Q_PROPERTY(QList<CylinderObjectHelper*> cylinderObjects READ cylinderObjects CONSTANT)
 	Q_PROPERTY(QString trip READ trip CONSTANT)
 	Q_PROPERTY(QString tripMeta READ tripMeta CONSTANT)
+	Q_PROPERTY(int tripNrDives READ tripNrDives CONSTANT)
 	Q_PROPERTY(int maxcns READ maxcns CONSTANT)
 	Q_PROPERTY(int otu READ otu CONSTANT)
 	Q_PROPERTY(QString sumWeight READ sumWeight CONSTANT)
@@ -85,6 +85,7 @@ public:
 	QList<CylinderObjectHelper*> cylinderObjects() const;
 	QString trip() const;
 	QString tripMeta() const;
+	int tripNrDives() const;
 	int maxcns() const;
 	int otu() const;
 	QString sumWeight() const;
