@@ -6,7 +6,7 @@
 #include <QDate>
 #include <QNetworkProxy>
 
-#include "core/helpers.h"
+#include "core/qthelper.h"
 
 DiveComputerSettings::DiveComputerSettings(QObject *parent):
 	QObject(parent)
@@ -392,6 +392,11 @@ bool TechnicalDetailsSettings::rulerGraph() const
 	return prefs.rulergraph;
 }
 
+bool TechnicalDetailsSettings::showSCROCpO2() const
+{
+	return prefs.show_scr_ocpo2;
+}
+
 bool TechnicalDetailsSettings::showCCRSetpoint() const
 {
 	return prefs.show_ccr_setpoint;
@@ -667,6 +672,18 @@ void TechnicalDetailsSettings::setShowCCRSetpoint(bool value)
 	emit showCCRSetpointChanged(value);
 }
 
+void TechnicalDetailsSettings::setShowSCROCpO2(bool value)
+{
+	if (value == prefs.show_scr_ocpo2)
+		return;
+
+	QSettings s;
+	s.beginGroup(group);
+	s.setValue("show_scr_ocpo2", value);
+	prefs.show_scr_ocpo2 = value;
+	emit showSCROCpO2Changed(value);
+}
+
 void TechnicalDetailsSettings::setShowCCRSensors(bool value)
 {
 	if (value == prefs.show_ccr_sensors)
@@ -1122,9 +1139,8 @@ void CloudStorageSettings::setBaseUrl(const QString& value)
 	prefs.cloud_git_url = copy_qstring(QString(prefs.cloud_base_url) + "/git");
 }
 
-void CloudStorageSettings::setGitUrl(const QString& value)
+void CloudStorageSettings::setGitUrl(const QString&)
 {
-	Q_UNUSED(value); /* no op */
 }
 
 void CloudStorageSettings::setGitLocalOnly(bool value)
@@ -1786,6 +1802,11 @@ int GeneralSettingsObjectWrapper::pscrRatio() const
 	return prefs.pscr_ratio;
 }
 
+bool GeneralSettingsObjectWrapper::autoRecalculateThumbnails() const
+{
+	return prefs.auto_recalculate_thumbnails;
+}
+
 void GeneralSettingsObjectWrapper::setDefaultFilename(const QString& value)
 {
 	if (value == prefs.default_filename)
@@ -1879,6 +1900,18 @@ void GeneralSettingsObjectWrapper::setPscrRatio(int value)
 	emit pscrRatioChanged(value);
 }
 
+void GeneralSettingsObjectWrapper::setAutoRecalculateThumbnails(bool value)
+{
+	if (value == prefs.auto_recalculate_thumbnails)
+		return;
+
+	QSettings s;
+	s.beginGroup(group);
+	s.setValue("auto_recalculate_thumbnails", value);
+	prefs.auto_recalculate_thumbnails = value;
+	emit autoRecalculateThumbnailsChanged(value);
+}
+
 DisplaySettingsObjectWrapper::DisplaySettingsObjectWrapper(QObject *parent) :
 	QObject(parent)
 {
@@ -2222,6 +2255,7 @@ void SettingsObjectWrapper::load()
 	GET_BOOL("gf_low_at_maxdepth", gf_low_at_maxdepth);
 	GET_BOOL("show_ccr_setpoint",show_ccr_setpoint);
 	GET_BOOL("show_ccr_sensors",show_ccr_sensors);
+	GET_BOOL("show_scr_ocpo2",show_scr_ocpo2);
 	GET_BOOL("zoomed_plot", zoomed_plot);
 	set_gf(prefs.gflow, prefs.gfhigh);
 	set_vpmb_conservatism(prefs.vpmb_conservatism);
@@ -2249,6 +2283,7 @@ void SettingsObjectWrapper::load()
 	GET_INT("defaultsetpoint", defaultsetpoint);
 	GET_INT("o2consumption", o2consumption);
 	GET_INT("pscr_ratio", pscr_ratio);
+	GET_BOOL("auto_recalculate_thumbnails", auto_recalculate_thumbnails);
 	s.endGroup();
 
 	s.beginGroup("Display");
