@@ -135,6 +135,7 @@ class TechnicalDetailsSettings : public QObject {
 	Q_PROPERTY(bool rulergraph       READ rulerGraph      WRITE setRulerGraph      NOTIFY rulerGraphChanged)
 	Q_PROPERTY(bool show_ccr_setpoint READ showCCRSetpoint WRITE setShowCCRSetpoint NOTIFY showCCRSetpointChanged)
 	Q_PROPERTY(bool show_ccr_sensors  READ showCCRSensors  WRITE setShowCCRSensors  NOTIFY showCCRSensorsChanged)
+	Q_PROPERTY(bool show_scr_ocpo2   READ showSCROCpO2    WRITE setShowSCROCpO2    NOTIFY showSCROCpO2Changed)
 	Q_PROPERTY(bool zoomed_plot      READ zoomedPlot      WRITE setZoomedPlot      NOTIFY zoomedPlotChanged)
 	Q_PROPERTY(bool show_sac             READ showSac            WRITE setShowSac            NOTIFY showSacChanged)
 	Q_PROPERTY(bool display_unused_tanks READ displayUnusedTanks WRITE setDisplayUnusedTanks NOTIFY displayUnusedTanksChanged)
@@ -165,6 +166,7 @@ public:
 	bool rulerGraph() const;
 	bool showCCRSetpoint() const;
 	bool showCCRSensors() const;
+	bool showSCROCpO2() const;
 	bool zoomedPlot() const;
 	bool showSac() const;
 	bool displayUnusedTanks() const;
@@ -193,6 +195,7 @@ public slots:
 	void setRulerGraph(bool value);
 	void setShowCCRSetpoint(bool value);
 	void setShowCCRSensors(bool value);
+	void setShowSCROCpO2(bool value);
 	void setZoomedPlot(bool value);
 	void setShowSac(bool value);
 	void setDisplayUnusedTanks(bool value);
@@ -221,6 +224,7 @@ signals:
 	void rulerGraphChanged(bool value);
 	void showCCRSetpointChanged(bool value);
 	void showCCRSensorsChanged(bool value);
+	void showSCROCpO2Changed(bool value);
 	void zoomedPlotChanged(bool value);
 	void showSacChanged(bool value);
 	void displayUnusedTanksChanged(bool value);
@@ -548,13 +552,14 @@ private:
 
 class GeneralSettingsObjectWrapper : public QObject {
 	Q_OBJECT
-	Q_PROPERTY(QString default_filename      READ defaultFilename       WRITE setDefaultFilename       NOTIFY defaultFilenameChanged)
-	Q_PROPERTY(QString default_cylinder      READ defaultCylinder       WRITE setDefaultCylinder       NOTIFY defaultCylinderChanged)
-	Q_PROPERTY(short default_file_behavior   READ defaultFileBehavior   WRITE setDefaultFileBehavior   NOTIFY defaultFileBehaviorChanged)
-	Q_PROPERTY(bool use_default_file         READ useDefaultFile        WRITE setUseDefaultFile        NOTIFY useDefaultFileChanged)
-	Q_PROPERTY(int defaultsetpoint           READ defaultSetPoint       WRITE setDefaultSetPoint       NOTIFY defaultSetPointChanged)
-	Q_PROPERTY(int o2consumption             READ o2Consumption         WRITE setO2Consumption         NOTIFY o2ConsumptionChanged)
-	Q_PROPERTY(int pscr_ratio                READ pscrRatio             WRITE setPscrRatio             NOTIFY pscrRatioChanged)
+	Q_PROPERTY(QString default_filename         READ defaultFilename           WRITE setDefaultFilename           NOTIFY defaultFilenameChanged)
+	Q_PROPERTY(QString default_cylinder         READ defaultCylinder           WRITE setDefaultCylinder           NOTIFY defaultCylinderChanged)
+	Q_PROPERTY(short default_file_behavior      READ defaultFileBehavior       WRITE setDefaultFileBehavior       NOTIFY defaultFileBehaviorChanged)
+	Q_PROPERTY(bool use_default_file            READ useDefaultFile            WRITE setUseDefaultFile            NOTIFY useDefaultFileChanged)
+	Q_PROPERTY(int defaultsetpoint              READ defaultSetPoint           WRITE setDefaultSetPoint           NOTIFY defaultSetPointChanged)
+	Q_PROPERTY(int o2consumption                READ o2Consumption             WRITE setO2Consumption             NOTIFY o2ConsumptionChanged)
+	Q_PROPERTY(int pscr_ratio                   READ pscrRatio                 WRITE setPscrRatio                 NOTIFY pscrRatioChanged)
+	Q_PROPERTY(bool auto_recalculate_thumbnails READ autoRecalculateThumbnails WRITE setAutoRecalculateThumbnails NOTIFY autoRecalculateThumbnailsChanged)
 
 public:
 	GeneralSettingsObjectWrapper(QObject *parent);
@@ -565,15 +570,17 @@ public:
 	int defaultSetPoint() const;
 	int o2Consumption() const;
 	int pscrRatio() const;
+	bool autoRecalculateThumbnails() const;
 
 public slots:
-	void setDefaultFilename       (const QString& value);
-	void setDefaultCylinder       (const QString& value);
-	void setDefaultFileBehavior   (short value);
-	void setUseDefaultFile        (bool value);
-	void setDefaultSetPoint       (int value);
-	void setO2Consumption         (int value);
-	void setPscrRatio             (int value);
+	void setDefaultFilename           (const QString& value);
+	void setDefaultCylinder           (const QString& value);
+	void setDefaultFileBehavior       (short value);
+	void setUseDefaultFile            (bool value);
+	void setDefaultSetPoint           (int value);
+	void setO2Consumption             (int value);
+	void setPscrRatio                 (int value);
+	void setAutoRecalculateThumbnails (bool value);
 
 signals:
 	void defaultFilenameChanged(const QString& value);
@@ -583,6 +590,7 @@ signals:
 	void defaultSetPointChanged(int value);
 	void o2ConsumptionChanged(int value);
 	void pscrRatioChanged(int value);
+	void autoRecalculateThumbnailsChanged(int value);
 private:
 	const QString group = QStringLiteral("GeneralSettings");
 };
