@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "subsurfacestartup.h"
+#include "subsurface-string.h"
 #include "version.h"
 #include <stdbool.h>
 #include <string.h>
@@ -42,6 +43,7 @@ struct preferences default_prefs = {
 	.gf_low_at_maxdepth = false,
 	.show_ccr_setpoint = false,
 	.show_ccr_sensors = false,
+	.show_scr_ocpo2 = false,
 	.font_size = -1,
 	.display_invalid_dives = false,
 	.show_sac = false,
@@ -97,6 +99,7 @@ struct preferences default_prefs = {
 #else
 	.cloud_timeout = 5,
 #endif
+	.auto_recalculate_thumbnails = true,
 };
 
 int run_survey;
@@ -281,6 +284,7 @@ void setup_system_prefs(void)
 {
 	const char *env;
 
+	subsurface_OS_pref_setup();
 	default_prefs.divelist_font = strdup(system_divelist_default_font);
 	default_prefs.font_size = system_divelist_default_font_size;
 
