@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/configuredivecomputerdialog.h"
 
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include "desktop-widgets/mainwindow.h"
 #include "core/display.h"
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
@@ -21,10 +21,8 @@ GasSpinBoxItemDelegate::~GasSpinBoxItemDelegate()
 {
 }
 
-QWidget *GasSpinBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+QWidget *GasSpinBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem&, const QModelIndex&) const
 {
-	Q_UNUSED(option)
-	Q_UNUSED(index)
 	// Create the spinbox and give it it's settings
 	QSpinBox *sb = new QSpinBox(parent);
 	if (type == PERCENT) {
@@ -67,10 +65,8 @@ GasTypeComboBoxItemDelegate::~GasTypeComboBoxItemDelegate()
 {
 }
 
-QWidget *GasTypeComboBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+QWidget *GasTypeComboBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem&, const QModelIndex&) const
 {
-	Q_UNUSED(option)
-	Q_UNUSED(index)
 	// Create the combobox and populate it
 	QComboBox *cb = new QComboBox(parent);
 	cb->addItem(QString("Disabled"));
@@ -362,9 +358,8 @@ ConfigureDiveComputerDialog::~ConfigureDiveComputerDialog()
 	delete config;
 }
 
-void ConfigureDiveComputerDialog::closeEvent(QCloseEvent *event)
+void ConfigureDiveComputerDialog::closeEvent(QCloseEvent*)
 {
-	Q_UNUSED(event)
 	dc_close();
 
 	QSettings settings;
@@ -463,7 +458,6 @@ void ConfigureDiveComputerDialog::populateDeviceDetailsOSTC3()
 	deviceDetails->aGFLow = ui.aGFLowSpinBox->value();
 	deviceDetails->calibrationGas = ui.calibrationGasSpinBox->value();
 	deviceDetails->flipScreen = ui.flipScreenCheckBox->isChecked();
-	deviceDetails->setPointFallback = ui.setPointFallbackCheckBox->isChecked();
 	deviceDetails->leftButtonSensitivity = ui.leftButtonSensitivity->value();
 	deviceDetails->rightButtonSensitivity = ui.rightButtonSensitivity->value();
 	deviceDetails->bottomGasConsumption = ui.bottomGasConsumption->value();
@@ -1000,7 +994,6 @@ void ConfigureDiveComputerDialog::reloadValuesOSTC3()
 	ui.aGFLowSpinBox->setValue(deviceDetails->aGFLow);
 	ui.calibrationGasSpinBox->setValue(deviceDetails->calibrationGas);
 	ui.flipScreenCheckBox->setChecked(deviceDetails->flipScreen);
-	ui.setPointFallbackCheckBox->setChecked(deviceDetails->setPointFallback);
 	ui.leftButtonSensitivity->setValue(deviceDetails->leftButtonSensitivity);
 	ui.rightButtonSensitivity->setValue(deviceDetails->rightButtonSensitivity);
 	ui.bottomGasConsumption->setValue(deviceDetails->bottomGasConsumption);
