@@ -18,12 +18,12 @@
 #include <QNetworkReply>
 #include <QStandardPaths>
 #include <QMessageBox>
+#include <QHeaderView>
 #include "core/qthelper.h"
 #include "desktop-widgets/undocommands.h"
 #include "desktop-widgets/divelistview.h"
 #include "qt-models/divepicturemodel.h"
 #include "core/metrics.h"
-#include "core/helpers.h"
 
 DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false), sortColumn(0),
 	currentOrder(Qt::DescendingOrder), dontEmitDiveChangedSignal(false), selectionSaved(false)
@@ -424,13 +424,12 @@ void DiveListView::reload(DiveTripModel::Layout layout, bool forceSort)
 
 	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
 	QAbstractItemModel *oldModel = m->sourceModel();
-	if (oldModel) {
-		oldModel->deleteLater();
-	}
 	tripModel = new DiveTripModel(this);
 	tripModel->setLayout(layout);
 
 	m->setSourceModel(tripModel);
+	if (oldModel)
+		delete oldModel;
 
 	if (!forceSort)
 		return;
@@ -515,9 +514,8 @@ void DiveListView::toggleColumnVisibilityByIndex()
 	setColumnWidth(lastVisibleColumn(), 10);
 }
 
-void DiveListView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
+void DiveListView::currentChanged(const QModelIndex &current, const QModelIndex&)
 {
-	Q_UNUSED(previous)
 	if (!isVisible())
 		return;
 	if (!current.isValid())
@@ -1044,8 +1042,7 @@ void DiveListView::updateLastImageTimeOffset(const int offset)
 	s.setValue("LastImageTimeOffset", offset);
 }
 
-void DiveListView::mouseDoubleClickEvent(QMouseEvent * event)
+void DiveListView::mouseDoubleClickEvent(QMouseEvent*)
 {
-	(void) event;
 	return;
 }
