@@ -1,9 +1,10 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/downloadfromdivecomputer.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/divelistview.h"
 #include "core/display.h"
+#include "core/subsurface-string.h"
 #include "core/uemis.h"
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include "qt-models/models.h"
@@ -84,7 +85,7 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	ui.downloadCancelRetryButton->setText(tr("Download"));
 
 	QString deviceText = dc->dc_device();
-#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_IO)
+#if defined(BT_SUPPORT)
 	ui.bluetoothMode->setText(tr("Choose Bluetooth download mode"));
 	ui.bluetoothMode->setChecked(dc->downloadMode() == DC_TRANSPORT_BLUETOOTH);
 	btDeviceSelectionDialog = 0;
@@ -240,9 +241,8 @@ void DownloadFromDCWidget::on_vendor_currentIndexChanged(const QString &vendor)
 	fill_device_list(dcType);
 }
 
-void DownloadFromDCWidget::on_product_currentIndexChanged(const QString &product)
+void DownloadFromDCWidget::on_product_currentIndexChanged(const QString&)
 {
-	Q_UNUSED(product)
 	updateDeviceEnabled();
 }
 
@@ -318,7 +318,7 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 	dc->setProduct(data->product());
 	dc->setDevice(data->devName());
 
-#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_IO)
+#if defined(BT_SUPPORT)
 	dc->setDownloadMode(ui.bluetoothMode->isChecked() ? DC_TRANSPORT_BLUETOOTH : DC_TRANSPORT_SERIAL);
 #endif
 
@@ -477,7 +477,7 @@ void DownloadFromDCWidget::updateDeviceEnabled()
 	descriptor = descriptorLookup.value(ui.vendor->currentText() + ui.product->currentText());
 
 	// call dc_descriptor_get_transport to see if the dc_transport_t is DC_TRANSPORT_SERIAL
-	if (dc_descriptor_get_transport(descriptor) == DC_TRANSPORT_SERIAL) {
+	if (dc_descriptor_get_transports(descriptor) & DC_TRANSPORT_SERIAL) {
 		// if the dc_transport_t is DC_TRANSPORT_SERIAL, then enable the device node box.
 		ui.device->setEnabled(true);
 	} else {
