@@ -14,7 +14,9 @@
 #include <QToolBar>
 #include <QStatusBar>
 
+#include "core/gettextfromc.h"
 #include "core/version.h"
+#include "core/subsurface-string.h"
 #include "desktop-widgets/divelistview.h"
 #include "desktop-widgets/downloadfromdivecomputer.h"
 #include "desktop-widgets/subsurfacewebservices.h"
@@ -33,7 +35,7 @@
 #include "desktop-widgets/printdialog.h"
 #endif
 #include "qt-models/tankinfomodel.h"
-#include "qt-models/weigthsysteminfomodel.h"
+#include "qt-models/weightsysteminfomodel.h"
 #include "qt-models/yearlystatisticsmodel.h"
 #include "qt-models/diveplannermodel.h"
 #include "qt-models/cylindermodel.h"
@@ -76,10 +78,13 @@ extern "C" int updateProgress(const char *text)
 	if (verbose)
 		qDebug() << "git storage:" << text;
 	if (progressDialog) {
+		// apparently we don't always get enough space to show the full label
+		// so let's manually make enough space (but don't shrink the existing size)
+		int width = QFontMetrics(qApp->font()).width(text) + 100;
+		if (width > progressDialog->width())
+			progressDialog->resize(width + 20, progressDialog->height());
 		progressDialog->setLabelText(text);
 		progressDialog->setValue(++progressCounter);
-		int width = QFontMetrics(qApp->font()).width(text) + 100;
-		progressDialog->resize(width, progressDialog->height());
 		if (progressCounter == 100)
 			progressCounter = 0; // yes this is silly, but we really don't know how long it will take
 	}
@@ -99,7 +104,9 @@ extern "C" void showErrorFromC(char *buf)
 MainWindow::MainWindow() : QMainWindow(),
 	actionNextDive(0),
 	actionPreviousDive(0),
+#ifndef NO_USERMANUAL
 	helpView(0),
+#endif
 	state(VIEWALL),
 	survey(0)
 {
@@ -530,7 +537,7 @@ void MainWindow::current_dive_changed(int divenr)
 	if (divenr >= 0) {
 		select_dive(divenr);
 	}
-	graphics()->plotDive();
+	graphics()->plotDive(nullptr, false, true);
 	information()->updateDiveInfo();
 	configureToolbar();
 	MapWidget::instance()->reload();
@@ -541,6 +548,18 @@ void MainWindow::on_actionNew_triggered()
 	on_actionClose_triggered();
 }
 
+static QString lastUsedDir()
+{
+	QSettings settings;
+	QString lastDir = QDir::homePath();
+
+	settings.beginGroup("FileDialog");
+	if (settings.contains("LastDir"))
+		if (QDir(settings.value("LastDir").toString()).exists())
+			lastDir = settings.value("LastDir").toString();
+	return lastDir;
+}
+
 void MainWindow::on_actionOpen_triggered()
 {
 	if (!okToClose(tr("Please save or cancel the current dive edit before opening a new file.")))
@@ -609,13 +628,21 @@ void MainWindow::on_actionCloudstorageopen_triggered()
 	refreshDisplay();
 }
 
+// Return whether saving to cloud is OK. If it isn't, show an error return false.
+static bool saveToCloudOK()
+{
+	if (!dive_table.nr) {
+		report_error(qPrintable(QObject::tr("Don't save an empty log to the cloud")));
+		return false;
+	}
+	return true;
+}
+
 void MainWindow::on_actionCloudstoragesave_triggered()
 {
 	QString filename;
-	if (!dive_table.nr) {
-		report_error(qPrintable(tr("Don't save an empty log to the cloud")));
+	if (!saveToCloudOK())
 		return;
-	}
 	if (getCloudURL(filename))
 		return;
 
@@ -744,7 +771,7 @@ void MainWindow::closeCurrentFile()
 
 	clear_events();
 
-	dcList.dcMap.clear();
+	dcList.dcs.clear();
 }
 
 void MainWindow::updateCloudOnlineStatus()
@@ -773,18 +800,6 @@ void MainWindow::on_actionClose_triggered()
 	}
 }
 
-QString MainWindow::lastUsedDir()
-{
-	QSettings settings;
-	QString lastDir = QDir::homePath();
-
-	settings.beginGroup("FileDialog");
-	if (settings.contains("LastDir"))
-		if (QDir(settings.value("LastDir").toString()).exists())
-			lastDir = settings.value("LastDir").toString();
-	return lastDir;
-}
-
 void MainWindow::updateLastUsedDir(const QString &dir)
 {
 	QSettings s;
@@ -878,7 +893,6 @@ void MainWindow::on_actionDivelogs_de_triggered()
 void MainWindow::on_actionEditDeviceNames_triggered()
 {
 	DiveComputerManagementDialog::instance()->init();
-	DiveComputerManagementDialog::instance()->update();
 	DiveComputerManagementDialog::instance()->show();
 }
 
@@ -1086,7 +1100,7 @@ void MainWindow::on_actionAddDive_triggered()
 	graphics()->setAddState();
 	DivePlannerPointsModel::instance()->createSimpleDive();
 	configureToolbar();
-	graphics()->plotDive();
+	graphics()->plotDive(nullptr, false, true);
 	fixup_dc_duration(&displayed_dive.dc);
 	displayed_dive.duration = displayed_dive.dc.duration;
 
@@ -1096,25 +1110,7 @@ void MainWindow::on_actionAddDive_triggered()
 
 void MainWindow::on_actionEditDive_triggered()
 {
-	if (information()->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
-		QMessageBox::warning(this, tr("Warning"), tr("Please, first finish the current edition before trying to do another."));
-		return;
-	}
-
-	const bool isTripEdit = dive_list()->selectedTrips().count() >= 1;
-	if (!current_dive || isTripEdit || (current_dive->dc.model && strcmp(current_dive->dc.model, "manually added dive"))) {
-		QMessageBox::warning(this, tr("Warning"), tr("Trying to edit a dive that's not a manually added dive."));
-		return;
-	}
-
-	DivePlannerPointsModel::instance()->clear();
-	disableShortcuts();
-	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
-	graphics()->setAddState();
-	MapWidget::instance()->endGetDiveCoordinates();
-	setApplicationState("EditDive");
-	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
-	information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
+	editCurrentDive();
 }
 
 void MainWindow::on_actionRenumber_triggered()
@@ -1246,6 +1242,61 @@ void MainWindow::on_actionViewAll_triggered()
 	ui.bottomSplitter->setCollapsible(1,false);
 }
 
+void MainWindow::enterEditState()
+{
+	stateBeforeEdit = state;
+	if (state == VIEWALL || state == INFO_MAXIMIZED)
+		return;
+	toggleCollapsible(true);
+	beginChangeState(EDIT);
+	ui.topSplitter->setSizes({ EXPANDED, EXPANDED });
+	ui.mainSplitter->setSizes({ EXPANDED, COLLAPSED });
+	int appW = qApp->desktop()->size().width();
+	QList<int> infoProfileSizes { (int)lrint(appW * 0.3), (int)lrint(appW * 0.7) };
+
+	QSettings settings;
+	settings.beginGroup("MainWindow");
+	if (settings.value("mainSplitter").isValid()) {
+		ui.topSplitter->restoreState(settings.value("topSplitter").toByteArray());
+		if (ui.topSplitter->sizes().first() == 0 || ui.topSplitter->sizes().last() == 0)
+			ui.topSplitter->setSizes(infoProfileSizes);
+	} else {
+		ui.topSplitter->setSizes(infoProfileSizes);
+	}
+}
+
+void MainWindow::exitEditState()
+{
+	if (stateBeforeEdit == state)
+		return;
+	enterState(stateBeforeEdit);
+}
+
+void MainWindow::enterState(CurrentState newState)
+{
+	state = newState;
+	switch (state) {
+	case VIEWALL:
+		on_actionViewAll_triggered();
+		break;
+	case MAP_MAXIMIZED:
+		on_actionViewMap_triggered();
+		break;
+	case INFO_MAXIMIZED:
+		on_actionViewInfo_triggered();
+		break;
+	case LIST_MAXIMIZED:
+		on_actionViewList_triggered();
+		break;
+	case PROFILE_MAXIMIZED:
+		on_actionViewProfile_triggered();
+		break;
+	case EDIT:
+	default:
+		break;
+	}
+}
+
 void MainWindow::beginChangeState(CurrentState s)
 {
 	if (state == VIEWALL && state != s) {
@@ -1268,7 +1319,7 @@ void MainWindow::on_actionPreviousDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + nrdc - 1) % nrdc;
 	configureToolbar();
-	graphics()->plotDive();
+	graphics()->plotDive(nullptr, false, true);
 	information()->updateDiveInfo();
 }
 
@@ -1277,7 +1328,7 @@ void MainWindow::on_actionNextDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + 1) % nrdc;
 	configureToolbar();
-	graphics()->plotDive();
+	graphics()->plotDive(nullptr, false, true);
 	information()->updateDiveInfo();
 }
 
@@ -1308,18 +1359,16 @@ void MainWindow::on_action_Check_for_Updates_triggered()
 void MainWindow::on_actionUserManual_triggered()
 {
 #ifndef NO_USERMANUAL
-	if (!helpView) {
-		helpView = new UserManual();
-	}
+	if (!helpView)
+		helpView = new UserManual(this);
 	helpView->show();
 #endif
 }
 
 void MainWindow::on_actionUserSurvey_triggered()
 {
-	if(!survey) {
+	if(!survey)
 		survey = new UserSurvey(this);
-	}
 	survey->show();
 }
 
@@ -1451,24 +1500,7 @@ void MainWindow::initialUiSetup()
 		restoreState(settings.value("windowState", 0).toByteArray());
 	}
 
-	state = (CurrentState)settings.value("lastState", 0).toInt();
-	switch (state) {
-	case VIEWALL:
-		on_actionViewAll_triggered();
-		break;
-	case MAP_MAXIMIZED:
-		on_actionViewMap_triggered();
-		break;
-	case INFO_MAXIMIZED:
-		on_actionViewInfo_triggered();
-		break;
-	case LIST_MAXIMIZED:
-		on_actionViewList_triggered();
-		break;
-	case PROFILE_MAXIMIZED:
-		on_actionViewProfile_triggered();
-		break;
-	}
+	enterState((CurrentState)settings.value("lastState", 0).toInt());
 	settings.endGroup();
 	show();
 }
@@ -1532,18 +1564,6 @@ void MainWindow::closeEvent(QCloseEvent *event)
 		return;
 	}
 
-#ifndef NO_USERMANUAL
-	if (helpView && helpView->isVisible()) {
-		helpView->close();
-		helpView->deleteLater();
-	}
-#endif
-
-	if (survey && survey->isVisible()) {
-		survey->close();
-		survey->deleteLater();
-	}
-
 	if (unsaved_changes() && (askSaveChanges() == false)) {
 		event->ignore();
 		return;
@@ -1626,10 +1646,8 @@ void MainWindow::updateRecentFiles()
 	updateRecentFilesMenu();
 }
 
-void MainWindow::recentFileTriggered(bool checked)
+void MainWindow::recentFileTriggered(bool)
 {
-	Q_UNUSED(checked);
-
 	if (!okToClose(tr("Please save or cancel the current dive edit before opening a new file.")))
 		return;
 
@@ -1704,6 +1722,8 @@ int MainWindow::file_save(void)
 		return file_save_as();
 
 	is_cloud = (strncmp(existing_filename, "http", 4) == 0);
+	if (is_cloud && !saveToCloudOK())
+		return -1;
 
 	if (information()->isEditing())
 		information()->acceptChanges();
@@ -1881,6 +1901,9 @@ void MainWindow::on_actionImportDiveLog_triggered()
 
 void MainWindow::editCurrentDive()
 {
+	if (!current_dive)
+		return;
+
 	if (information()->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
 		QMessageBox::warning(this, tr("Warning"), tr("Please, first finish the current edition before trying to do another."));
 		return;
@@ -1889,19 +1912,21 @@ void MainWindow::editCurrentDive()
 	struct dive *d = current_dive;
 	QString defaultDC(d->dc.model);
 	DivePlannerPointsModel::instance()->clear();
+	disableShortcuts();
 	if (defaultDC == "manually added dive") {
-		disableShortcuts();
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
 		graphics()->setAddState();
 		setApplicationState("EditDive");
 		DivePlannerPointsModel::instance()->loadFromDive(d);
 		information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
 	} else if (defaultDC == "planned dive") {
-		disableShortcuts();
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 		setApplicationState("EditPlannedDive");
 		DivePlannerPointsModel::instance()->loadFromDive(d);
 		information()->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
+	} else {
+		setApplicationState("EditDive");
+		information()->enableEdition();
 	}
 }
 
@@ -2036,7 +2061,7 @@ void MainWindow::hideProgressBar()
 {
 	if (progressDialog) {
 		progressDialog->setValue(100);
-		progressDialog->deleteLater();
+		delete progressDialog;
 		progressDialog = NULL;
 	}
 }
