@@ -5,15 +5,14 @@
 #include <QModelIndex>
 
 #include "mapwidget.h"
-#include "core/dive.h"
 #include "core/divesite.h"
 #include "map-widget/qmlmapwidgethelper.h"
 #include "qt-models/maplocationmodel.h"
 #include "mainwindow.h"
 #include "divelistview.h"
 
-static const QUrl urlMapWidget = QUrl(QStringLiteral("qrc:/MapWidget.qml"));
-static const QUrl urlMapWidgetError = QUrl(QStringLiteral("qrc:/MapWidgetError.qml"));
+static const QUrl urlMapWidget = QUrl(QStringLiteral("qrc:/qml/MapWidget.qml"));
+static const QUrl urlMapWidgetError = QUrl(QStringLiteral("qrc:/qml/MapWidgetError.qml"));
 static bool isReady = false;
 static bool skipReload = false;
 
