@@ -40,12 +40,14 @@ void PreferencesGraph::refreshSettings()
 	ui->vpmb_conservatism->setValue(prefs.vpmb_conservatism);
 	ui->show_ccr_setpoint->setChecked(prefs.show_ccr_setpoint);
 	ui->show_ccr_sensors->setChecked(prefs.show_ccr_sensors);
+	ui->show_scr_ocpo2->setChecked(prefs.show_scr_ocpo2);
 	ui->defaultSetpoint->setValue((double)prefs.defaultsetpoint / 1000.0);
 	ui->psro2rate->setValue(prefs.o2consumption / 1000.0);
 	ui->pscrfactor->setValue(lrint(1000.0 / prefs.pscr_ratio));
 
 	ui->display_unused_tanks->setChecked(prefs.display_unused_tanks);
 	ui->show_average_depth->setChecked(prefs.show_average_depth);
+	ui->auto_recalculate_thumbnails->setChecked(prefs.auto_recalculate_thumbnails);
 	ui->show_icd->setChecked(prefs.show_icd);
 }
 
@@ -55,6 +57,7 @@ void PreferencesGraph::syncSettings()
 	general->setDefaultSetPoint(lrint(ui->defaultSetpoint->value() * 1000.0));
 	general->setO2Consumption(lrint(ui->psro2rate->value() *1000.0));
 	general->setPscrRatio(lrint(1000.0 / ui->pscrfactor->value()));
+	general->setAutoRecalculateThumbnails(ui->auto_recalculate_thumbnails->isChecked());
 
 	auto pp_gas = SettingsObjectWrapper::instance()->pp_gas;
 	pp_gas->setPheThreshold(ui->pheThreshold->value());
@@ -71,6 +74,7 @@ void PreferencesGraph::syncSettings()
 	tech->setVpmbConservatism(ui->vpmb_conservatism->value());
 	tech->setShowCCRSetpoint(ui->show_ccr_setpoint->isChecked());
 	tech->setShowCCRSensors(ui->show_ccr_sensors->isChecked());
+	tech->setShowSCROCpO2(ui->show_scr_ocpo2->isChecked());
 	tech->setDisplayUnusedTanks(ui->display_unused_tanks->isChecked());
 	tech->setShowAverageDepth(ui->show_average_depth->isChecked());
 	tech->setShowIcd(ui->show_icd->isChecked());
