@@ -13,13 +13,12 @@
 
 #include "core/file.h"
 #include "desktop-widgets/mainwindow.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include "libdivecomputer/parser.h"
 #include "desktop-widgets/divelistview.h"
 #include "core/display.h"
 #include "profile-widget/profilewidget2.h"
 #include "desktop-widgets/undocommands.h"
-#include "core/qthelper.h"
 #include "core/metadata.h"
 
 class MinMaxAvgWidgetPrivate {
@@ -68,9 +67,8 @@ double MinMaxAvgWidget::minimum() const
 	return d->minValue->text().toDouble();
 }
 
-MinMaxAvgWidget::MinMaxAvgWidget(QWidget *parent) : d(new MinMaxAvgWidgetPrivate(this))
+MinMaxAvgWidget::MinMaxAvgWidget(QWidget*) : d(new MinMaxAvgWidgetPrivate(this))
 {
-	Q_UNUSED(parent)
 }
 
 MinMaxAvgWidget::~MinMaxAvgWidget()
@@ -261,9 +259,8 @@ void ShiftTimesDialog::buttonClicked(QAbstractButton *button)
 	}
 }
 
-void ShiftTimesDialog::showEvent(QShowEvent *event)
+void ShiftTimesDialog::showEvent(QShowEvent*)
 {
-	Q_UNUSED(event)
 	ui.timeEdit->setTime(QTime(0, 0, 0, 0));
 	when = get_times(); //get time of first selected dive
 	ui.currentTime->setText(get_dive_date_string(when));
@@ -415,10 +412,15 @@ void ShiftImageTimesDialog::updateInvalid()
 
 void ShiftImageTimesDialog::timeEditChanged(const QTime &time)
 {
+	QDateTimeEdit::Section timeEditSection = ui.timeEdit->currentSection();
+	ui.timeEdit->setEnabled(false);
 	m_amount = time.hour() * 3600 + time.minute() * 60;
 	if (ui.backwards->isChecked())
 		m_amount *= -1;
 	updateInvalid();
+	ui.timeEdit->setEnabled(true);
+	ui.timeEdit->setFocus();
+	ui.timeEdit->setSelectedSection(timeEditSection);
 }
 
 void ShiftImageTimesDialog::timeEditChanged()
