@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/subsurfacewebservices.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include "core/webservice.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/usersurvey.h"
@@ -12,7 +12,7 @@
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include <errno.h>
 #include "core/cloudstorage.h"
-#include "core/dive.h"
+#include "core/subsurface-string.h"
 
 #include <QDir>
 #include <QHttpMultiPart>
@@ -415,7 +415,6 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 		struct dive *d;
 		struct dive_site *ds;
 		bool changed = false;
-		(void)changed;
 		clear_table(&gps_location_table);
 		QByteArray url = tr("Webservice").toUtf8();
 		parse_xml_buffer(url.data(), downloadedData.data(), downloadedData.length(), &gps_location_table, NULL);
@@ -692,7 +691,6 @@ out:
 DivelogsDeWebServices *DivelogsDeWebServices::instance()
 {
 	static DivelogsDeWebServices *self = new DivelogsDeWebServices(MainWindow::instance());
-	self->setAttribute(Qt::WA_QuitOnClose, false);
 	return self;
 }
 
@@ -958,9 +956,8 @@ void DivelogsDeWebServices::uploadFinished()
 	}
 }
 
-void DivelogsDeWebServices::setStatusText(int status)
+void DivelogsDeWebServices::setStatusText(int)
 {
-	Q_UNUSED(status)
 }
 
 void DivelogsDeWebServices::downloadError(QNetworkReply::NetworkError)
