@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "TabDivePhotos.h"
 #include "ui_TabDivePhotos.h"
+#include "core/imagedownloader.h"
 
 #include <qt-models/divepicturemodel.h>
 
@@ -53,29 +54,38 @@ void TabDivePhotos::contextMenuEvent(QContextMenuEvent *event)
 	popup.addSeparator();
 	popup.addAction(tr("Delete selected images"), this, SLOT(removeSelectedPhotos()));
 	popup.addAction(tr("Delete all images"), this, SLOT(removeAllPhotos()));
+	popup.addAction(tr("Recalculate selected thumbnails"), this, SLOT(recalculateSelectedThumbnails()));
 	popup.exec(event->globalPos());
 	event->accept();
 }
 
-void TabDivePhotos::removeSelectedPhotos()
+QVector<QString> TabDivePhotos::getSelectedFilenames() const
 {
-	bool last = false;
+	QVector<QString> selectedPhotos;
 	if (!ui->photosView->selectionModel()->hasSelection())
-		return;
-	QModelIndexList indexes =  ui->photosView->selectionModel()->selectedRows();
+		return selectedPhotos;
+	QModelIndexList indexes = ui->photosView->selectionModel()->selectedRows();
 	if (indexes.count() == 0)
 		indexes = ui->photosView->selectionModel()->selectedIndexes();
-	QModelIndex photo = indexes.first();
-	do {
-		photo = indexes.first();
-		last = indexes.count() == 1;
+	selectedPhotos.reserve(indexes.count());
+	for (const auto &photo: indexes) {
 		if (photo.isValid()) {
 			QString fileUrl = photo.data(Qt::DisplayPropertyRole).toString();
-			if (fileUrl.length() > 0)
-				DivePictureModel::instance()->removePicture(fileUrl, last);
+			if (!fileUrl.isEmpty())
+				selectedPhotos.push_back(fileUrl);
 		}
-		indexes.removeFirst();
-	} while(!indexes.isEmpty());
+	}
+	return selectedPhotos;
+}
+
+void TabDivePhotos::removeSelectedPhotos()
+{
+	DivePictureModel::instance()->removePictures(getSelectedFilenames());
+}
+
+void TabDivePhotos::recalculateSelectedThumbnails()
+{
+	Thumbnailer::instance()->calculateThumbnails(getSelectedFilenames());
 }
 
 //TODO: This looks overly wrong. We shouldn't call MainWindow to retrieve the DiveList to add Images.
