@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "qmlmanager.h"
+#include "qmlprefs.h"
 #include <QUrl>
 #include <QSettings>
 #include <QDebug>
@@ -12,31 +13,36 @@
 #include <QElapsedTimer>
 #include <QTimer>
 #include <QDateTime>
+#include <QClipboard>
+#include <QFile>
 
 #include <QBluetoothLocalDevice>
 
 #include "qt-models/divelistmodel.h"
 #include "qt-models/gpslistmodel.h"
 #include "qt-models/completionmodels.h"
+#include "qt-models/messagehandlermodel.h"
 #include "core/divelist.h"
 #include "core/device.h"
-#include "core/pref.h"
 #include "core/qthelper.h"
 #include "core/qt-gui.h"
 #include "core/git-access.h"
 #include "core/cloudstorage.h"
-#include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include "core/membuffer.h"
 #include "qt-models/tankinfomodel.h"
 #include "core/downloadfromdcthread.h"
+#include "core/subsurface-string.h"
+#include "core/pref.h"
+#include "core/subsurface-qt/SettingsObjectWrapper.h"
+
+#include "core/ssrf.h"
 
 QMLManager *QMLManager::m_instance = NULL;
+bool noCloudToCloud = false;
 
 #define RED_FONT QLatin1Literal("<font color=\"red\">")
 #define END_FONT QLatin1Literal("</font>")
 
-#define NOCLOUD_LOCALSTORAGE format_string("%s/cloudstorage/localrepo[master]", system_default_directory())
-
 extern "C" void showErrorFromC(char *buf)
 {
 	QString error(buf);
@@ -128,36 +134,61 @@ void QMLManager::btRescan()
 
 QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	m_verboseEnabled(false),
-	reply(0),
 	deletedDive(0),
 	deletedTrip(0),
 	m_updateSelectedDive(-1),
 	m_selectedDiveTimestamp(0),
-	m_credentialStatus(CS_UNKNOWN),
 	alreadySaving(false),
-	m_device_data(new DCDeviceData(this)),
-	m_libdcLog(false)
+	m_device_data(new DCDeviceData)
 {
+	LOG_STP("qmlmgr starting");
 	m_instance = this;
 	m_lastDevicePixelRatio = qApp->devicePixelRatio();
 	timer.start();
 	connect(qobject_cast<QApplication *>(QApplication::instance()), &QApplication::applicationStateChanged, this, &QMLManager::applicationStateChanged);
 
-	QString libdcLogFileName = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + "/libdivecomputer.log";
-	logfile_name = copy_qstring(libdcLogFileName);
+#if defined(Q_OS_ANDROID) || defined(Q_OS_IOS)
 #if defined(Q_OS_ANDROID)
-	appLogFileName = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + "/subsurface.log";
-	appLogFile.setFileName(appLogFileName);
-	if (!appLogFile.open(QIODevice::ReadWrite|QIODevice::Truncate)) {
-		appLogFileOpen = false;
-		appendTextToLog("Failed to open logfile " + appLogFileName
-				+ " at " + QDateTime::currentDateTime().toString());
-	} else {
-		appLogFileOpen = true;
+	// on Android we first try the GenericDataLocation (typically /storage/emulated/0) and if that fails
+	// (as happened e.g. on a Sony Xperia phone) we try several other default locations, with the TempLocation as last resort
+	QStringList fileLocations =
+		QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation) +
+		QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation) +
+		QStandardPaths::standardLocations(QStandardPaths::DownloadLocation) +
+		QStandardPaths::standardLocations(QStandardPaths::TempLocation);
+#elif defined(Q_OS_IOS)
+	// on iOS we should save the data to the DocumentsLocation so it becomes accessible to the user
+	QStringList fileLocations =
+		QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation);
+#endif
+	appLogFileOpen = false;
+	for (const QString &fileLocation : fileLocations) {
+		appLogFileName = fileLocation + "/subsurface.log";
+		appLogFile.setFileName(appLogFileName);
+		if (!appLogFile.open(QIODevice::ReadWrite|QIODevice::Truncate)) {
+			appendTextToLog("Failed to open logfile " + appLogFileName
+					+ " at " + QDateTime::currentDateTime().toString()
+					+ " error: " + appLogFile.errorString());
+		} else {
+			// found a directory that works
+			appLogFileOpen = true;
+			break;
+		}
+	}
+	if (appLogFileOpen) {
 		appendTextToLog("Successfully opened logfile " + appLogFileName
 				+ " at " + QDateTime::currentDateTime().toString());
+		// if we were able to write the overall logfile, also write the libdivecomputer logfile
+		QString libdcLogFileName = appLogFileName.replace("/subsurface.log", "/libdivecomputer.log");
+		// remove the existing libdivecomputer logfile so we don't copy an old one by mistake
+		QFile libdcLog(libdcLogFileName);
+		libdcLog.remove();
+		logfile_name = copy_qstring(libdcLogFileName);
+	} else {
+		appendTextToLog("No writeable location found, in-memory log only and no libdivecomputer log");
 	}
 #endif
+	LOG_STP("qmlmgr log started");
 	set_error_cb(&showErrorFromC);
 	appendTextToLog("Starting " + getUserAgent());
 	appendTextToLog(QStringLiteral("built with libdivecomputer v%1").arg(dc_version(NULL)));
@@ -166,23 +197,28 @@ QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	git_libgit2_version(&git_maj, &git_min, &git_rev);
 	appendTextToLog(QStringLiteral("built with libgit2 %1.%2.%3").arg(git_maj).arg(git_min).arg(git_rev));
 	setStartPageText(tr("Starting..."));
+	LOG_STP("qmlmgr start page");
 
 	// ensure that we start the BTDiscovery - this should be triggered by the export of the class
 	// to QML, but that doesn't seem to always work
 	BTDiscovery *btDiscovery = BTDiscovery::instance();
 	m_btEnabled = btDiscovery->btAvailable();
+	LOG_STP("qmlmgr bt available");
 	connect(&btDiscovery->localBtDevice, &QBluetoothLocalDevice::hostModeStateChanged,
 		this, &QMLManager::btHostModeChange);
-	setShowPin(false);
+	QMLPrefs::instance()->setShowPin(false);
 	// create location manager service
 	locationProvider = new GpsLocation(&appendTextToLogStandalone, this);
 	progress_callback = &progressCallback;
 	connect(locationProvider, SIGNAL(haveSourceChanged()), this, SLOT(hasLocationSourceChanged()));
 	setLocationServiceAvailable(locationProvider->hasLocationsSource());
+	LOG_STP("qmlmgr gps started");
 	set_git_update_cb(&gitProgressCB);
+	LOG_STP("qmlmgr git update");
 
 	// make sure we know if the current cloud repo has been successfully synced
 	syncLoadFromCloud();
+	LOG_STP("qmlmgr sync load cloud");
 }
 
 void QMLManager::applicationStateChanged(Qt::ApplicationState state)
@@ -233,12 +269,12 @@ void QMLManager::openLocalThenRemote(QString url)
 		 *    no cloud repo solves this.
 		 */
 
-		if (m_credentialStatus != CS_NOCLOUD)
-			setCredentialStatus(CS_NEED_TO_VERIFY);
+		if (QMLPrefs::instance()->credentialStatus() != QMLPrefs::CS_NOCLOUD)
+			QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_NEED_TO_VERIFY);
 	} else {
 		// if we can load from the cache, we know that we have a valid cloud account
-		if (m_credentialStatus == CS_UNKNOWN)
-			setCredentialStatus(CS_VERIFIED);
+		if (QMLPrefs::instance()->credentialStatus() == QMLPrefs::CS_UNKNOWN)
+			QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_VERIFIED);
 		prefs.unit_system = git_prefs.unit_system;
 		if (git_prefs.unit_system == IMPERIAL)
 			git_prefs.units = IMPERIAL_units;
@@ -256,11 +292,11 @@ void QMLManager::openLocalThenRemote(QString url)
 		appendTextToLog(QStringLiteral("%1 dives loaded from cache").arg(dive_table.nr));
 		setNotificationText(tr("%1 dives loaded from local dive data file").arg(dive_table.nr));
 	}
-	if (m_credentialStatus == CS_NEED_TO_VERIFY) {
+	if (QMLPrefs::instance()->credentialStatus() == QMLPrefs::CS_NEED_TO_VERIFY) {
 		appendTextToLog(QStringLiteral("have cloud credentials, but still needs PIN"));
-		setShowPin(true);
+		QMLPrefs::instance()->setShowPin(true);
 	}
-	if (m_oldStatus == CS_NOCLOUD) {
+	if (QMLPrefs::instance()->oldStatus() == QMLPrefs::CS_NOCLOUD) {
 		// if we switch to credentials from CS_NOCLOUD, we take things online temporarily
 		prefs.git_local_only = false;
 		appendTextToLog(QStringLiteral("taking things online to be able to switch to cloud account"));
@@ -291,60 +327,65 @@ void QMLManager::mergeLocalRepo()
 	process_dives(true, false);
 }
 
-void QMLManager::clearCredentials()
-{
-	setCloudUserName(NULL);
-	setCloudPassword(NULL);
-	setCloudPin(NULL);
-}
-
-void QMLManager::cancelCredentialsPinSetup()
+void QMLManager::copyAppLogToClipboard()
 {
 	/*
-	 * The user selected <cancel> on the final stage of the
-	 * cloud account generation (entering the emailed PIN).
-	 *
-	 * Resets the cloud credential status to CS_UNKNOWN, resulting
-	 * in a return to the first crededentials page, with the
-	 * email and passwd still filled in. In case of a cancel
-	 * of registration (from the PIN page), the email address
-	 * was probably misspelled, so the user never received a PIN to
-	 * complete the process.
-	 *
-	 * Notice that this function is also used to switch to a different
-	 * cloud account, so the name is not perfect.
+	 * The user clicked the button, so copy the log file
+	 * to the clipboard for easy access
 	 */
-	QSettings s;
 
-	setCredentialStatus(CS_UNKNOWN);
-	s.beginGroup("CloudStorage");
-	s.setValue("email", m_cloudUserName);
-	s.setValue("password", m_cloudPassword);
-	s.setValue("cloud_verification_status", m_credentialStatus);
-	s.sync();
-	setStartPageText(tr("Starting..."));
+	// Add heading and append subsurface.log
+	QString copyString = "---------- subsurface.log ----------\n";
+	copyString += MessageHandlerModel::self()->logAsString();
 
-	setShowPin(false);
+	// Add heading and append libdivecomputer.log
+	QFile f(logfile_name);
+	if (f.open(QFile::ReadOnly | QFile::Text)) {
+		copyString += "\n\n\n---------- libdivecomputer.log ----------\n";
+
+		QTextStream in(&f);
+		copyString += in.readAll();
+	}
+	LOG_STP_CLIPBOARD(&copyString);
+
+	copyString += "---------- finish ----------\n";
+
+#if defined(Q_OS_ANDROID)
+	// on Android, the clipboard is effectively limited in size, but there is no
+	// predefined hard limit. All remote procedure calls use a shared Binder
+	// transaction buffer that is limited to 1MB. To work around this let's truncate
+	// the log once it is more than half a million characters. Qt doesn't tell us if
+	// the clipboard transaction fails, hopefully this will typically leave enough
+	// margin of error.
+	if (copyString.size() > 500000) {
+		copyString.truncate(500000);
+		copyString += "\n\n---------- truncated ----------\n";
+	}
+#endif
+
+	// and copy to clipboard
+	QApplication::clipboard()->setText(copyString, QClipboard::Clipboard);
 }
 
 void QMLManager::finishSetup()
 {
 	// Initialize cloud credentials.
-	setCloudUserName(prefs.cloud_storage_email);
-	setCloudPassword(prefs.cloud_storage_password);
+	QMLPrefs::instance()->setCloudUserName(prefs.cloud_storage_email);
+	QMLPrefs::instance()->setCloudPassword(prefs.cloud_storage_password);
 	setSyncToCloud(!prefs.git_local_only);
-	setCredentialStatus((cloud_status_qml) prefs.cloud_verification_status);
+	QMLPrefs::instance()->setCredentialStatus((QMLPrefs::cloud_status_qml) prefs.cloud_verification_status);
 	// if the cloud credentials are valid, we should get the GPS Webservice ID as well
 	QString url;
-	if (!m_cloudUserName.isEmpty() &&
-	    !m_cloudPassword.isEmpty() &&
+	if (!QMLPrefs::instance()->cloudUserName().isEmpty() &&
+	    !QMLPrefs::instance()->cloudPassword().isEmpty() &&
 	    getCloudURL(url) == 0) {
 		// we know that we are the first ones to access git storage, so we don't need to test,
 		// but we need to make sure we stay the only ones accessing git storage
 		alreadySaving = true;
 		openLocalThenRemote(url);
-	} else if (!empty_string(existing_filename) && m_credentialStatus != CS_UNKNOWN) {
-		setCredentialStatus(CS_NOCLOUD);
+	} else if (!empty_string(existing_filename) &&
+				QMLPrefs::instance()->credentialStatus() != QMLPrefs::CS_UNKNOWN) {
+		QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_NOCLOUD);
 		saveCloudCredentials();
 		appendTextToLog(tr("working in no-cloud mode"));
 		int error = parse_file(existing_filename);
@@ -358,17 +399,17 @@ void QMLManager::finishSetup()
 			appendTextToLog(QString("working in no-cloud mode, finished loading %1 dives from %2").arg(dive_table.nr).arg(existing_filename));
 		}
 	} else {
-		setCredentialStatus(CS_UNKNOWN);
+		QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_UNKNOWN);
 		appendTextToLog(tr("no cloud credentials"));
 		setStartPageText(RED_FONT + tr("Please enter valid cloud credentials.") + END_FONT);
 	}
-	setDistanceThreshold(prefs.distance_threshold);
-	setTimeThreshold(prefs.time_threshold / 60);
+	QMLPrefs::instance()->setDistanceThreshold(prefs.distance_threshold);
+	QMLPrefs::instance()->setTimeThreshold(prefs.time_threshold / 60);
 }
 
 QMLManager::~QMLManager()
 {
-#if defined(Q_OS_ANDROID)
+#if defined(Q_OS_ANDROID) || defined(Q_OS_IOS)
 	if (appLogFileOpen)
 		appLogFile.close();
 #endif
@@ -382,9 +423,9 @@ QMLManager *QMLManager::instance()
 
 void QMLManager::savePreferences()
 {
-	auto location = SettingsObjectWrapper::instance()->location_settings;
-	location->setTimeThreshold(m_timeThreshold * 60);
-	location->setDistanceThreshold(m_distanceThreshold);
+    auto location = SettingsObjectWrapper::instance()->location_settings;
+    location->setTimeThreshold(QMLPrefs::instance()->timeThreshold() * 60);
+    location->setDistanceThreshold(QMLPrefs::instance()->distanceThreshold());
 }
 
 #define CLOUDURL QString(prefs.cloud_base_url)
@@ -396,42 +437,55 @@ void QMLManager::saveCloudCredentials()
 	bool cloudCredentialsChanged = false;
 	// make sure we only have letters, numbers, and +-_. in password and email address
 	QRegularExpression regExp("^[a-zA-Z0-9@.+_-]+$");
-	if (m_credentialStatus != CS_NOCLOUD) {
+	if (QMLPrefs::instance()->credentialStatus() != QMLPrefs::CS_NOCLOUD) {
 		// in case of NO_CLOUD, the email address + passwd do not care, so do not check it.
-		if (m_cloudPassword.isEmpty() || !regExp.match(m_cloudPassword).hasMatch() || !regExp.match(m_cloudUserName).hasMatch()) {
+		if (QMLPrefs::instance()->cloudPassword().isEmpty() ||
+			!regExp.match(QMLPrefs::instance()->cloudPassword()).hasMatch() ||
+			!regExp.match(QMLPrefs::instance()->cloudUserName()).hasMatch()) {
 			setStartPageText(RED_FONT + tr("Cloud storage email and password can only consist of letters, numbers, and '.', '-', '_', and '+'.") + END_FONT);
 			return;
 		}
 		// use the same simplistic regex as the backend to check email addresses
 		regExp = QRegularExpression("^[a-zA-Z0-9.+_-]+@[a-zA-Z0-9.+_-]+\\.[a-zA-Z0-9]+");
-		if (!regExp.match(m_cloudUserName).hasMatch()) {
+		if (!regExp.match(QMLPrefs::instance()->cloudUserName()).hasMatch()) {
 			setStartPageText(RED_FONT + tr("Invalid format for email address") + END_FONT);
 			return;
 		}
 	}
 	s.beginGroup("CloudStorage");
-	s.setValue("email", m_cloudUserName);
-	s.setValue("password", m_cloudPassword);
-	s.setValue("cloud_verification_status", m_credentialStatus);
+	s.setValue("email", QMLPrefs::instance()->cloudUserName());
+	s.setValue("password", QMLPrefs::instance()->cloudPassword());
+	s.setValue("cloud_verification_status", QMLPrefs::instance()->credentialStatus());
 	s.sync();
-	if (!same_string(prefs.cloud_storage_email, qPrintable(m_cloudUserName))) {
+	if (!same_string(prefs.cloud_storage_email,
+		qPrintable(QMLPrefs::instance()->cloudUserName()))) {
 		free((void *)prefs.cloud_storage_email);
-		prefs.cloud_storage_email = copy_qstring(m_cloudUserName);
+		prefs.cloud_storage_email = copy_qstring(QMLPrefs::instance()->cloudUserName());
 		cloudCredentialsChanged = true;
 	}
 
-	cloudCredentialsChanged |= !same_string(prefs.cloud_storage_password, qPrintable(m_cloudPassword));
+	cloudCredentialsChanged |= !same_string(prefs.cloud_storage_password,
+								qPrintable(QMLPrefs::instance()->cloudPassword()));
 
-	if (m_credentialStatus != CS_NOCLOUD && !cloudCredentialsChanged) {
+	if (QMLPrefs::instance()->credentialStatus() != QMLPrefs::CS_NOCLOUD &&
+		!cloudCredentialsChanged) {
 		// just go back to the dive list
-		setCredentialStatus(m_oldStatus);
+		QMLPrefs::instance()->setCredentialStatus(QMLPrefs::instance()->oldStatus());
 	}
 
-	if (!same_string(prefs.cloud_storage_password, qPrintable(m_cloudPassword))) {
+	if (!same_string(prefs.cloud_storage_password,
+					qPrintable(QMLPrefs::instance()->cloudPassword()))) {
 		free((void *)prefs.cloud_storage_password);
-		prefs.cloud_storage_password = copy_qstring(m_cloudPassword);
+		prefs.cloud_storage_password = copy_qstring(QMLPrefs::instance()->cloudPassword());
 	}
-	if (m_cloudUserName.isEmpty() || m_cloudPassword.isEmpty()) {
+	if (QMLPrefs::instance()->oldStatus() == QMLPrefs::CS_NOCLOUD && cloudCredentialsChanged && dive_table.nr) {
+		// we came from NOCLOUD and are connecting to a cloud account;
+		// since we already have dives in the table, let's remember that so we can keep them
+		noCloudToCloud = true;
+		appendTextToLog("transitioning from no-cloud to cloud and have dives");
+	}
+	if (QMLPrefs::instance()->cloudUserName().isEmpty() ||
+		QMLPrefs::instance()->cloudPassword().isEmpty()) {
 		setStartPageText(RED_FONT + tr("Please enter valid cloud credentials.") + END_FONT);
 	} else if (cloudCredentialsChanged) {
 		// let's make sure there are no unsaved changes
@@ -454,7 +508,8 @@ void QMLManager::saveCloudCredentials()
 		currentGitLocalOnly = prefs.git_local_only;
 		prefs.git_local_only = false;
 		openLocalThenRemote(url);
-	} else if (prefs.cloud_verification_status == CS_NEED_TO_VERIFY && !m_cloudPin.isEmpty()) {
+	} else if (prefs.cloud_verification_status == QMLPrefs::CS_NEED_TO_VERIFY &&
+				!QMLPrefs::instance()->cloudPin().isEmpty()) {
 		// the user entered a PIN?
 		tryRetrieveDataFromBackend();
 	}
@@ -469,7 +524,8 @@ void QMLManager::tryRetrieveDataFromBackend()
 		setStartPageText(tr("Testing cloud credentials"));
 		appendTextToLog("Have credentials, let's see if they are valid");
 		CloudStorageAuthenticate *csa = new CloudStorageAuthenticate(this);
-		csa->backend(prefs.cloud_storage_email, prefs.cloud_storage_password, m_cloudPin);
+		csa->backend(prefs.cloud_storage_email, prefs.cloud_storage_password,
+						QMLPrefs::instance()->cloudPin());
 		// let's wait here for the signal to avoid too many more nested functions
 		QTimer myTimer;
 		myTimer.setSingleShot(true);
@@ -485,7 +541,7 @@ void QMLManager::tryRetrieveDataFromBackend()
 			return;
 		}
 		myTimer.stop();
-		setCloudPin("");
+		QMLPrefs::instance()->setCloudPin("");
 		if (prefs.cloud_verification_status == CS_INCORRECT_USER_PASSWD) {
 			appendTextToLog(QStringLiteral("Incorrect cloud credentials"));
 			setStartPageText(RED_FONT + tr("Incorrect cloud credentials") + END_FONT);
@@ -496,22 +552,22 @@ void QMLManager::tryRetrieveDataFromBackend()
 			appendTextToLog(QStringLiteral("Need to verify the email address - enter PIN"));
 			setStartPageText(RED_FONT + tr("Cannot connect to cloud storage - cloud account not verified") + END_FONT);
 			revertToNoCloudIfNeeded();
-			setShowPin(true);
+			QMLPrefs::instance()->setShowPin(true);
 			return;
 		}
-		if (m_showPin)
-			setShowPin(false);
+		if (QMLPrefs::instance()->showPin())
+			QMLPrefs::instance()->setShowPin(false);
 
 		// now check the redirect URL to make sure everything is set up on the cloud server
 		connect(manager(), &QNetworkAccessManager::authenticationRequired, this, &QMLManager::provideAuth, Qt::UniqueConnection);
 		QUrl url(CLOUDREDIRECTURL);
-		request = QNetworkRequest(url);
+		QNetworkRequest request(url);
 		request.setRawHeader("User-Agent", getUserAgent().toUtf8());
 		request.setRawHeader("Accept", "text/html");
-		reply = manager()->get(request);
+		QNetworkReply *reply = manager()->get(request);
 		connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(handleError(QNetworkReply::NetworkError)));
 		connect(reply, &QNetworkReply::sslErrors, this, &QMLManager::handleSslErrors);
-		connect(reply, &QNetworkReply::finished, this, &QMLManager::retrieveUserid, Qt::UniqueConnection);
+		connect(reply, &QNetworkReply::finished, this, &QMLManager::retrieveUserid);
 	}
 }
 
@@ -522,7 +578,7 @@ void QMLManager::provideAuth(QNetworkReply *reply, QAuthenticator *auth)
 		// OK, credentials have been tried and didn't work, so they are invalid
 		appendTextToLog("Cloud credentials are invalid");
 		setStartPageText(RED_FONT + tr("Cloud credentials are invalid") + END_FONT);
-		setCredentialStatus(CS_INCORRECT_USER_PASSWD);
+		QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_INCORRECT_USER_PASSWD);
 		reply->disconnect();
 		reply->abort();
 		reply->deleteLater();
@@ -534,6 +590,7 @@ void QMLManager::provideAuth(QNetworkReply *reply, QAuthenticator *auth)
 
 void QMLManager::handleSslErrors(const QList<QSslError> &errors)
 {
+	auto *reply = qobject_cast<QNetworkReply *>(sender());
 	setStartPageText(RED_FONT + tr("Cannot open cloud storage: Error creating https connection") + END_FONT);
 	Q_FOREACH (QSslError e, errors) {
 		appendTextToLog(e.errorString());
@@ -545,6 +602,7 @@ void QMLManager::handleSslErrors(const QList<QSslError> &errors)
 
 void QMLManager::handleError(QNetworkReply::NetworkError nError)
 {
+	auto *reply = qobject_cast<QNetworkReply *>(sender());
 	QString errorString = reply->errorString();
 	appendTextToLog(QStringLiteral("handleError ") + nError + QStringLiteral(": ") + errorString);
 	setStartPageText(RED_FONT + tr("Cannot open cloud storage: %1").arg(errorString) + END_FONT);
@@ -555,6 +613,7 @@ void QMLManager::handleError(QNetworkReply::NetworkError nError)
 
 void QMLManager::retrieveUserid()
 {
+	auto *reply = qobject_cast<QNetworkReply *>(sender());
 	if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) != 302) {
 		appendTextToLog(QStringLiteral("Cloud storage connection not working correctly: (%1) %2")
 				.arg(reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt())
@@ -563,7 +622,7 @@ void QMLManager::retrieveUserid()
 		revertToNoCloudIfNeeded();
 		return;
 	}
-	setCredentialStatus(CS_VERIFIED);
+	QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_VERIFIED);
 	QString userid(prefs.userid);
 	if (userid.isEmpty()) {
 		if (empty_string(prefs.cloud_storage_email) || empty_string(prefs.cloud_storage_password)) {
@@ -582,7 +641,7 @@ void QMLManager::retrieveUserid()
 		s.setValue("subsurface_webservice_uid", prefs.userid);
 		s.sync();
 	}
-	setCredentialStatus(CS_VERIFIED);
+	QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_VERIFIED);
 	setStartPageText(tr("Cloud credentials valid, loading dives..."));
 	// this only gets called with "alreadySaving" already locked
 	loadDivesWithValidCredentials();
@@ -607,7 +666,13 @@ void QMLManager::loadDivesWithValidCredentials()
 	}
 	appendTextToLog("Cloud sync brought newer data, reloading the dive list");
 
-	clear_dive_file_data();
+	// if we aren't switching from no-cloud mode, let's clear the dive data
+	if (!noCloudToCloud) {
+		appendTextToLog("Clear out in memory dive data");
+		clear_dive_file_data();
+	} else {
+		appendTextToLog("Switching from no cloud mode; keep in memory dive data");
+	}
 	if (git != dummy_git_repository) {
 		appendTextToLog(QString("have repository and branch %1").arg(branch));
 		error = git_load_dives(git, branch);
@@ -632,13 +697,14 @@ successful_exit:
 	setLoadFromCloud(true);
 	// if we came from local storage mode, let's merge the local data into the local cache
 	// for the remote data - which then later gets merged with the remote data if necessary
-	if (m_oldStatus == CS_NOCLOUD) {
+	if (noCloudToCloud) {
 		git_storage_update_progress(qPrintable(tr("Loading dives from local storage ('no cloud' mode)")));
 		dive_table.preexisting = dive_table.nr;
 		mergeLocalRepo();
 		DiveListModel::instance()->clear();
 		DiveListModel::instance()->addAllDives();
 		appendTextToLog(QStringLiteral("%1 dives loaded after importing nocloud local storage").arg(dive_table.nr));
+		noCloudToCloud = false;
 		saveChangesLocal();
 		if (m_syncToCloud == false) {
 			appendTextToLog(QStringLiteral("taking things back offline now that storage is synced"));
@@ -660,7 +726,7 @@ void QMLManager::revertToNoCloudIfNeeded()
 		currentGitLocalOnly = false;
 		prefs.git_local_only = true;
 	}
-	if (m_oldStatus == CS_NOCLOUD) {
+	if (QMLPrefs::instance()->oldStatus() == QMLPrefs::CS_NOCLOUD) {
 		// we tried to switch to a cloud account and had previously used local data,
 		// but connecting to the cloud account (and subsequently merging the local
 		// and cloud data) failed - so let's delete the cloud credentials and go
@@ -674,9 +740,9 @@ void QMLManager::revertToNoCloudIfNeeded()
 		prefs.cloud_storage_email = NULL;
 		free((void *)prefs.cloud_storage_password);
 		prefs.cloud_storage_password = NULL;
-		setCloudUserName("");
-		setCloudPassword("");
-		setCredentialStatus(CS_NOCLOUD);
+		QMLPrefs::instance()->setCloudUserName("");
+		QMLPrefs::instance()->setCloudPassword("");
+		QMLPrefs::instance()->setCredentialStatus(QMLPrefs::CS_NOCLOUD);
 		set_filename(NOCLOUD_LOCALSTORAGE);
 		setStartPageText(RED_FONT + tr("Failed to connect to cloud server, reverting to no cloud status") + END_FONT);
 	}
@@ -720,8 +786,8 @@ static void setupDivesite(struct dive *d, struct dive_site *ds, double lat, doub
 		ds->longitude.udeg = lrint(lon * 1000000);
 	} else {
 		degrees_t latData, lonData;
-		latData.udeg = lrint(lat);
-		lonData.udeg = lrint(lon);
+		latData.udeg = lrint(lat * 1000000);
+		lonData.udeg = lrint(lon * 1000000);
 		d->dive_site_uuid = create_dive_site_with_gps(locationtext, latData, lonData, d->when);
 	}
 }
@@ -836,6 +902,7 @@ bool QMLManager::checkLocation(DiveObjectHelper *myDive, struct dive *d, QString
 	bool diveChanged = false;
 	uint32_t uuid = 0;
 	struct dive_site *ds = get_dive_site_for_dive(d);
+	qDebug() << "checkLocation" << location << "gps" << gps << "dive had" << myDive->location() << "gps" << myDive->gas();
 	if (myDive->location() != location) {
 		diveChanged = true;
 		if (!ds)
@@ -850,13 +917,16 @@ bool QMLManager::checkLocation(DiveObjectHelper *myDive, struct dive *d, QString
 	if (myDive->gps() != gps) {
 		double lat, lon;
 		if (parseGpsText(gps, &lat, &lon)) {
+			qDebug() << "parsed GPS, using it";
 			// there are valid GPS coordinates - just use them
 			setupDivesite(d, ds, lat, lon, qPrintable(myDive->location()));
 			diveChanged = true;
 		} else if (gps == GPS_CURRENT_POS) {
+			qDebug() << "gps was our default text for no GPS";
 			// user asked to use current pos
 			QString gpsString = getCurrentPosition();
 			if (gpsString != GPS_CURRENT_POS) {
+				qDebug() << "but now I got a valid location" << gpsString;
 				if (parseGpsText(qPrintable(gpsString), &lat, &lon)) {
 					setupDivesite(d, ds, lat, lon, qPrintable(myDive->location()));
 					diveChanged = true;
@@ -902,13 +972,10 @@ bool QMLManager::checkDuration(DiveObjectHelper *myDive, struct dive *d, QString
 			m = r6.cap(1).toInt();
 		}
 		d->dc.duration.seconds = d->duration.seconds = h * 3600 + m * 60 + s;
-		if (same_string(d->dc.model, "manually added dive")) {
-			free(d->dc.sample);
-			d->dc.sample = 0;
-			d->dc.samples = 0;
-		} else {
+		if (same_string(d->dc.model, "manually added dive"))
+			free_samples(&d->dc);
+		else
 			appendTextToLog("Cannot change the duration on a dive that wasn't manually added");
-		}
 		return true;
 	}
 	return false;
@@ -925,9 +992,7 @@ bool QMLManager::checkDepth(DiveObjectHelper *myDive, dive *d, QString depth)
 			d->maxdepth.mm = depthValue;
 			if (same_string(d->dc.model, "manually added dive")) {
 				d->dc.maxdepth.mm = d->maxdepth.mm;
-				free(d->dc.sample);
-				d->dc.sample = 0;
-				d->dc.samples = 0;
+				free_samples(&d->dc);
 			}
 			return true;
 		}
@@ -1080,10 +1145,11 @@ void QMLManager::commitChanges(QString diveId, QString date, QString location, Q
 			// so we have depth > 0, a manually added dive and no samples
 			// let's create an actual profile so the desktop version can work it
 			// first clear out the mean depth (or the fake_dc() function tries
-			// to be too clever
+			// to be too clever)
 			d->meandepth.mm = d->dc.meandepth.mm = 0;
-			d->dc = *fake_dc(&d->dc, true);
+			fake_dc(&d->dc);
 		}
+		fixup_dive(d);
 		DiveListModel::instance()->updateDive(modelIdx, d);
 		invalidate_dive_cache(d);
 		mark_divelist_changed(true);
@@ -1136,7 +1202,7 @@ void QMLManager::openNoCloudRepo()
 void QMLManager::saveChangesLocal()
 {
 	if (unsaved_changes()) {
-		if (m_credentialStatus == CS_NOCLOUD) {
+		if (QMLPrefs::instance()->credentialStatus() == QMLPrefs::CS_NOCLOUD) {
 			if (empty_string(existing_filename)) {
 				char *filename = NOCLOUD_LOCALSTORAGE;
 				git_create_local_repo(filename);
@@ -1292,7 +1358,12 @@ QString QMLManager::getCurrentPosition()
 	if (!hasLocationSource)
 		return tr("Unknown GPS location");
 
-	return locationProvider->currentPosition();
+	QString positionResponse = locationProvider->currentPosition();
+	if (positionResponse == GPS_CURRENT_POS)
+		connect(locationProvider, &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged, Qt::UniqueConnection);
+	else
+		disconnect(locationProvider, &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged);
+	return positionResponse;
 }
 
 void QMLManager::applyGpsData()
@@ -1374,52 +1445,6 @@ void QMLManager::setVerboseEnabled(bool verboseMode)
 	emit verboseEnabledChanged();
 }
 
-void QMLManager::setCloudPassword(const QString &cloudPassword)
-{
-	m_cloudPassword = cloudPassword;
-	emit cloudPasswordChanged();
-}
-
-void QMLManager::setCloudPin(const QString &cloudPin)
-{
-	m_cloudPin = cloudPin;
-	emit cloudPinChanged();
-}
-
-void QMLManager::setCloudUserName(const QString &cloudUserName)
-{
-	m_cloudUserName = cloudUserName.toLower();
-	emit cloudUserNameChanged();
-}
-
-void QMLManager::setDistanceThreshold(int distance)
-{
-	m_distanceThreshold = distance;
-	emit distanceThresholdChanged();
-}
-
-void QMLManager::setTimeThreshold(int time)
-{
-	m_timeThreshold = time;
-	locationProvider->setGpsTimeThreshold(m_timeThreshold * 60);
-	emit timeThresholdChanged();
-}
-
-void QMLManager::setTheme(QString theme)
-{
-	QSettings s;
-	s.beginGroup("Theme");
-	s.setValue("currentTheme", theme);
-	emit themeChanged();
-}
-
-QString QMLManager::theme() const
-{
-	QSettings s;
-	s.beginGroup("Theme");
-	return s.value("currentTheme", "Blue").toString();
-}
-
 void QMLManager::syncLoadFromCloud()
 {
 	QSettings s;
@@ -1442,28 +1467,6 @@ void QMLManager::setStartPageText(const QString& text)
 	emit startPageTextChanged();
 }
 
-void QMLManager::setCredentialStatus(const cloud_status_qml value)
-{
-	if (m_credentialStatus != value) {
-		setOldStatus(m_credentialStatus);
-		if (value == CS_NOCLOUD) {
-			appendTextToLog("Switching to no cloud mode");
-			set_filename(NOCLOUD_LOCALSTORAGE);
-			clearCredentials();
-		}
-		m_credentialStatus = value;
-		emit credentialStatusChanged();
-	}
-}
-
-void QMLManager::setOldStatus(const cloud_status_qml value)
-{
-	if (m_oldStatus != value) {
-		m_oldStatus = value;
-		emit oldStatusChanged();
-	}
-}
-
 QString QMLManager::getNumber(const QString& diveId)
 {
 	int dive_id = diveId.toInt();
@@ -1595,37 +1598,18 @@ QStringList QMLManager::cylinderInit() const
 	return cylinders;
 }
 
-void QMLManager::setShowPin(bool enable)
-{
-	m_showPin = enable;
-	emit showPinChanged();
-}
-
 void QMLManager::setProgressMessage(QString text)
 {
 	m_progressMessage = text;
 	emit progressMessageChanged();
 }
 
-void QMLManager::setLibdcLog(bool value)
-{
-	m_libdcLog = value;
-	DCDeviceData::instance()->setSaveLog(value);
-	emit libdcLogChanged();
-}
-
-void QMLManager::setDeveloper(bool value)
-{
-	m_developer = value;
-	emit developerChanged();
-}
-
 void QMLManager::setBtEnabled(bool value)
 {
 	m_btEnabled = value;
 }
 
-#if defined (Q_OS_ANDROID)
+#if defined(Q_OS_ANDROID) || defined(Q_OS_IOS)
 
 void writeToAppLogFile(QString logText)
 {
@@ -1644,7 +1628,9 @@ void QMLManager::writeToAppLogFile(QString logText)
 		appLogFile.flush();
 	}
 }
+#endif
 
+#if defined(Q_OS_ANDROID)
 //HACK to color the system bar on Android, use qtandroidextras and call the appropriate Java methods
 //this code is based on code in the Kirigami example app for Android (under LGPL-2) Copyright 2017 Marco Martin
 
@@ -1668,10 +1654,124 @@ void QMLManager::setStatusbarColor(QColor color)
 	});
 }
 #else
-void QMLManager::setStatusbarColor(QColor color)
+void QMLManager::setStatusbarColor(QColor)
 {
-	Q_UNUSED(color)
 	// noop
 }
 
 #endif
+
+QString QMLManager::DC_vendor() const
+{
+	return m_device_data->vendor();
+}
+
+QString QMLManager::DC_product() const
+{
+	return m_device_data->product();
+}
+
+QString QMLManager::DC_devName() const
+{
+	return m_device_data->devName();
+}
+
+QString QMLManager::DC_devBluetoothName() const
+{
+	return m_device_data->devBluetoothName();
+}
+
+QString QMLManager::DC_descriptor() const
+{
+	return m_device_data->descriptor();
+}
+
+bool QMLManager::DC_forceDownload() const
+{
+	return m_device_data->forceDownload();
+}
+
+bool QMLManager::DC_bluetoothMode() const
+{
+	return m_device_data->bluetoothMode();
+}
+
+bool QMLManager::DC_createNewTrip() const
+{
+	return m_device_data->createNewTrip();
+}
+
+bool QMLManager::DC_saveDump() const
+{
+	return m_device_data->saveDump();
+}
+
+int QMLManager::DC_deviceId() const
+{
+	return m_device_data->deviceId();
+}
+
+void QMLManager::DC_setDeviceId(int deviceId)
+{
+	m_device_data->setDeviceId(deviceId);
+}
+
+void QMLManager::DC_setVendor(const QString& vendor)
+{
+	m_device_data->setVendor(vendor);
+}
+
+void QMLManager::DC_setProduct(const QString& product)
+{
+	m_device_data->setProduct(product);
+}
+
+void QMLManager::DC_setDevName(const QString& devName)
+{
+	m_device_data->setDevName(devName);
+}
+
+void QMLManager::DC_setDevBluetoothName(const QString& devBluetoothName)
+{
+	m_device_data->setDevBluetoothName(devBluetoothName);
+}
+
+void QMLManager::DC_setBluetoothMode(bool mode)
+{
+	m_device_data->setBluetoothMode(mode);
+}
+
+void QMLManager::DC_setForceDownload(bool force)
+{
+	m_device_data->setForceDownload(force);
+}
+
+void QMLManager::DC_setCreateNewTrip(bool create)
+{
+	m_device_data->setCreateNewTrip(create);
+}
+
+void QMLManager::DC_setSaveDump(bool dumpMode)
+{
+	m_device_data->setSaveDump(dumpMode);
+}
+
+QStringList QMLManager::getProductListFromVendor(const QString &vendor)
+{
+	return m_device_data->getProductListFromVendor(vendor);
+}
+
+int QMLManager::getMatchingAddress(const QString &vendor, const QString &product)
+{
+	return m_device_data->getMatchingAddress(vendor, product);
+}
+
+int QMLManager::getDetectedVendorIndex()
+{
+	return m_device_data->getDetectedVendorIndex();
+}
+
+int QMLManager::getDetectedProductIndex(const QString &currentVendorText)
+{
+	return m_device_data->getDetectedProductIndex(currentVendorText);
+}
