@@ -8,11 +8,12 @@
 #include "core/gettextfromc.h"
 #include "core/metrics.h"
 #include "core/membuffer.h"
+#include "core/subsurface-string.h"
 
 extern struct ev_select *ev_namelist;
 extern int evn_used;
 
-DiveEventItem::DiveEventItem(QObject *parent) : DivePixmapItem(parent),
+DiveEventItem::DiveEventItem(QGraphicsItem *parent) : DivePixmapItem(parent),
 	vAxis(NULL),
 	hAxis(NULL),
 	dataModel(NULL),
@@ -83,6 +84,11 @@ void DiveEventItem::setupPixmap(struct gasmix *lastgasmix)
 #define EVENT_PIXMAP_BIGGER(PIX) QPixmap(QString(PIX)).scaled(sz_bigger, sz_bigger, Qt::KeepAspectRatio, Qt::SmoothTransformation)
 	if (empty_string(internalEvent->name)) {
 		setPixmap(EVENT_PIXMAP(":status-warning-icon"));
+	} else if (same_string_caseinsensitive(internalEvent->name, "modechange")) {
+		if (internalEvent->value == 0)
+			setPixmap(EVENT_PIXMAP(":bailout-icon"));
+		else
+			setPixmap(EVENT_PIXMAP(":onCCRLoop-icon"));
 	} else if (internalEvent->type == SAMPLE_EVENT_BOOKMARK) {
 		setPixmap(EVENT_PIXMAP(":dive-bookmark-icon"));
 	} else if (event_is_gaschange(internalEvent)) {
@@ -162,7 +168,7 @@ void DiveEventItem::setupPixmap(struct gasmix *lastgasmix)
 void DiveEventItem::setupToolTipString(struct gasmix *lastgasmix)
 {
 	// we display the event on screen - so translate
-	QString name = gettextFromC::instance()->tr(internalEvent->name);
+	QString name = gettextFromC::tr(internalEvent->name);
 	int value = internalEvent->value;
 	int type = internalEvent->type;
 
@@ -187,6 +193,8 @@ void DiveEventItem::setupToolTipString(struct gasmix *lastgasmix)
 			free_buffer(&mb);
 		}
 		*lastgasmix = *mix;
+	} else if (same_string(internalEvent->name, "modechange")) {
+		name += QString(": %1").arg(gettextFromC::tr(divemode_text_ui[internalEvent->value]));
 	} else if (value) {
 		if (type == SAMPLE_EVENT_PO2 && same_string(internalEvent->name, "SP change")) {
 			name += QString(": %1bar").arg((double)value / 1000, 0, 'f', 1);
@@ -208,11 +216,9 @@ void DiveEventItem::setupToolTipString(struct gasmix *lastgasmix)
 	setToolTip(name);
 }
 
-void DiveEventItem::eventVisibilityChanged(const QString &eventName, bool visible)
+void DiveEventItem::eventVisibilityChanged(const QString&, bool)
 {
 	//WARN: lookslike we should implement this.
-	Q_UNUSED(eventName);
-	Q_UNUSED(visible);
 }
 
 bool DiveEventItem::shouldBeHidden()
