@@ -1,7 +1,8 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "profile-widget/profilewidget2.h"
 #include "qt-models/diveplotdatamodel.h"
-#include "core/helpers.h"
+#include "core/subsurface-string.h"
+#include "core/qthelper.h"
 #include "core/profile.h"
 #include "profile-widget/diveeventitem.h"
 #include "profile-widget/divetextitem.h"
@@ -21,6 +22,7 @@
 #include "desktop-widgets/simplewidgets.h"
 #include "desktop-widgets/divepicturewidget.h"
 #include "core/qthelper.h"
+#include "core/gettextfromc.h"
 #endif
 
 #include <libdivecomputer/parser.h>
@@ -32,6 +34,7 @@
 #include <QWheelEvent>
 #include <QSettings>
 #include <QMenu>
+#include <QElapsedTimer>
 
 #ifndef QT_NO_DEBUG
 #include <QTableView>
@@ -109,6 +112,7 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	ccrsensor1GasItem(new PartialPressureGasItem()),
 	ccrsensor2GasItem(new PartialPressureGasItem()),
 	ccrsensor3GasItem(new PartialPressureGasItem()),
+	ocpo2GasItem(new PartialPressureGasItem()),
 #ifndef SUBSURFACE_MOBILE
 	diveCeiling(new DiveCalculatedCeiling(this)),
 	decoModelParameters(new DiveTextItem()),
@@ -147,6 +151,11 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	addActionShortcut(Qt::Key_Down, &ProfileWidget2::keyDownAction);
 	addActionShortcut(Qt::Key_Left, &ProfileWidget2::keyLeftAction);
 	addActionShortcut(Qt::Key_Right, &ProfileWidget2::keyRightAction);
+
+	connect(DivePictureModel::instance(), &DivePictureModel::dataChanged, this, &ProfileWidget2::updatePictures);
+	connect(DivePictureModel::instance(), SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(plotPictures()));
+	connect(DivePictureModel::instance(), &DivePictureModel::rowsRemoved, this, &ProfileWidget2::removePictures);
+	connect(DivePictureModel::instance(), &DivePictureModel::modelReset, this, &ProfileWidget2::plotPictures);
 #endif // SUBSURFACE_MOBILE
 
 #if !defined(QT_NO_DEBUG) && defined(SHOW_PLOT_INFO_TABLE)
@@ -198,6 +207,7 @@ void ProfileWidget2::addItemsToScene()
 	scene()->addItem(ccrsensor1GasItem);
 	scene()->addItem(ccrsensor2GasItem);
 	scene()->addItem(ccrsensor3GasItem);
+	scene()->addItem(ocpo2GasItem);
 #ifndef SUBSURFACE_MOBILE
 	scene()->addItem(toolTipItem);
 	scene()->addItem(diveCeiling);
@@ -311,10 +321,11 @@ void ProfileWidget2::setupItemOnScene()
 	createPPGas(pn2GasItem, DivePlotDataModel::PN2, PN2, PN2_ALERT, NULL, &prefs.pp_graphs.pn2_threshold);
 	createPPGas(pheGasItem, DivePlotDataModel::PHE, PHE, PHE_ALERT, NULL, &prefs.pp_graphs.phe_threshold);
 	createPPGas(po2GasItem, DivePlotDataModel::PO2, PO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(o2SetpointGasItem, DivePlotDataModel::O2SETPOINT, PO2_ALERT, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
+	createPPGas(o2SetpointGasItem, DivePlotDataModel::O2SETPOINT, O2SETPOINT, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
 	createPPGas(ccrsensor1GasItem, DivePlotDataModel::CCRSENSOR1, CCRSENSOR1, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
 	createPPGas(ccrsensor2GasItem, DivePlotDataModel::CCRSENSOR2, CCRSENSOR2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
 	createPPGas(ccrsensor3GasItem, DivePlotDataModel::CCRSENSOR3, CCRSENSOR3, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
+	createPPGas(ocpo2GasItem, DivePlotDataModel::SCR_OC_PO2, SCR_OCPO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
 
 #undef CREATE_PP_GAS
 #ifndef SUBSURFACE_MOBILE
@@ -323,10 +334,11 @@ void ProfileWidget2::setupItemOnScene()
 	connect(SettingsObjectWrapper::instance()->pp_gas, &PartialPressureGasSettings::showPheChanged, pheGasItem, &PartialPressureGasItem::setVisible);
 	connect(SettingsObjectWrapper::instance()->pp_gas, &PartialPressureGasSettings::showPo2Changed, po2GasItem, &PartialPressureGasItem::setVisible);
 	connect(SettingsObjectWrapper::instance()->pp_gas, &PartialPressureGasSettings::showPn2Changed, pn2GasItem, &PartialPressureGasItem::setVisible);
-	connect(SettingsObjectWrapper::instance()->pp_gas, &PartialPressureGasSettings::showPo2Changed, o2SetpointGasItem, &PartialPressureGasItem::setVisible);
 
 	//WARNING: The old code was broken, I'm not sure what should trigger the visibility of those graphs, since the old code didn't triggered them
 	// because it was using a wrong settings.
+	connect(SettingsObjectWrapper::instance()->techDetails, &TechnicalDetailsSettings::showCCRSetpointChanged, o2SetpointGasItem, &PartialPressureGasItem::setVisible);
+	connect(SettingsObjectWrapper::instance()->techDetails, &TechnicalDetailsSettings::showSCROCpO2Changed, ocpo2GasItem, &PartialPressureGasItem::setVisible);
 	connect(SettingsObjectWrapper::instance()->techDetails, &TechnicalDetailsSettings::showCCRSensorsChanged, ccrsensor1GasItem, &PartialPressureGasItem::setVisible);
 	connect(SettingsObjectWrapper::instance()->techDetails, &TechnicalDetailsSettings::showCCRSensorsChanged, ccrsensor2GasItem, &PartialPressureGasItem::setVisible);
 	connect(SettingsObjectWrapper::instance()->techDetails, &TechnicalDetailsSettings::showCCRSensorsChanged, ccrsensor3GasItem, &PartialPressureGasItem::setVisible);
@@ -352,7 +364,7 @@ void ProfileWidget2::replot(struct dive *d)
 	if (!replotEnabled)
 		return;
 	dataModel->clear();
-	plotDive(d, true);
+	plotDive(d, true, false);
 }
 
 void ProfileWidget2::createPPGas(PartialPressureGasItem *item, int verticalColumn, color_index_t color, color_index_t colorAlert,
@@ -385,19 +397,21 @@ void ProfileWidget2::setupItemSizes()
 	itemPos.depth.pos.off.setX(-2);
 	itemPos.depth.pos.off.setY(3);
 	itemPos.depth.expanded.setP1(QPointF(0, 0));
+#ifndef SUBSURFACE_MOBILE
 	itemPos.depth.expanded.setP2(QPointF(0, 85));
+#else
+	itemPos.depth.expanded.setP2(QPointF(0, 65));
+#endif
 	itemPos.depth.shrinked.setP1(QPointF(0, 0));
 	itemPos.depth.shrinked.setP2(QPointF(0, 55));
 	itemPos.depth.intermediate.setP1(QPointF(0, 0));
 	itemPos.depth.intermediate.setP2(QPointF(0, 65));
-#ifdef SUBSURFACE_MOBILE
-	itemPos.depth.expanded.setP2(QPointF(0, 65));
-#endif
 
 	// Time Axis Config
 	itemPos.time.pos.on.setX(3);
+#ifndef SUBSURFACE_MOBILE
 	itemPos.time.pos.on.setY(95);
-#ifdef SUBSURFACE_MOBILE
+#else
 	itemPos.time.pos.on.setY(89.5);
 #endif
 	itemPos.time.pos.off.setX(3);
@@ -407,15 +421,17 @@ void ProfileWidget2::setupItemSizes()
 
 	// Partial Gas Axis Config
 	itemPos.partialPressure.pos.on.setX(97);
+#ifndef SUBSURFACE_MOBILE
 	itemPos.partialPressure.pos.on.setY(75);
-#ifdef SUBSURFACE_MOBILE
+#else
 	itemPos.partialPressure.pos.on.setY(70);
 #endif
 	itemPos.partialPressure.pos.off.setX(110);
 	itemPos.partialPressure.pos.off.setY(63);
 	itemPos.partialPressure.expanded.setP1(QPointF(0, 0));
+#ifndef SUBSURFACE_MOBILE
 	itemPos.partialPressure.expanded.setP2(QPointF(0, 19));
-#ifdef SUBSURFACE_MOBILE
+#else
 	itemPos.partialPressure.expanded.setP2(QPointF(0, 20));
 #endif
 	itemPos.partialPressureWithTankBar = itemPos.partialPressure;
@@ -439,21 +455,20 @@ void ProfileWidget2::setupItemSizes()
 
 	// Temperature axis config
 	itemPos.temperature.pos.on.setX(3);
-	itemPos.temperature.pos.on.setY(60);
-	itemPos.temperatureAll.pos.on.setY(51);
 	itemPos.temperature.pos.off.setX(-10);
 	itemPos.temperature.pos.off.setY(40);
 	itemPos.temperature.expanded.setP1(QPointF(0, 20));
 	itemPos.temperature.expanded.setP2(QPointF(0, 33));
 	itemPos.temperature.shrinked.setP1(QPointF(0, 2));
 	itemPos.temperature.shrinked.setP2(QPointF(0, 12));
+#ifndef SUBSURFACE_MOBILE
+	itemPos.temperature.pos.on.setY(60);
+	itemPos.temperatureAll.pos.on.setY(51);
 	itemPos.temperature.intermediate.setP1(QPointF(0, 2));
 	itemPos.temperature.intermediate.setP2(QPointF(0, 12));
-#ifdef SUBSURFACE_MOBILE
+#else
 	itemPos.temperature.pos.on.setY(51);
 	itemPos.temperatureAll.pos.on.setY(47);
-	itemPos.temperature.expanded.setP1(QPointF(0, 20));
-	itemPos.temperature.expanded.setP2(QPointF(0, 33));
 	itemPos.temperature.intermediate.setP1(QPointF(0, 2));
 	itemPos.temperature.intermediate.setP2(QPointF(0, 12));
 #endif
@@ -480,8 +495,9 @@ void ProfileWidget2::setupItemSizes()
 	itemPos.dcLabel.off.setY(100);
 
 	itemPos.tankBar.on.setX(0);
+#ifndef SUBSURFACE_MOBILE
 	itemPos.tankBar.on.setY(91.95);
-#ifdef SUBSURFACE_MOBILE
+#else
 	itemPos.tankBar.on.setY(86.4);
 #endif
 }
@@ -521,12 +537,14 @@ void ProfileWidget2::resetZoom()
 }
 
 // Currently just one dive, but the plan is to enable All of the selected dives.
-void ProfileWidget2::plotDive(struct dive *d, bool force)
+void ProfileWidget2::plotDive(struct dive *d, bool force, bool doClearPictures)
 {
 	static bool firstCall = true;
 #ifndef SUBSURFACE_MOBILE
-	QTime measureDuration; // let's measure how long this takes us (maybe we'll turn of TTL calculation later
+	QElapsedTimer measureDuration; // let's measure how long this takes us (maybe we'll turn of TTL calculation later
 	measureDuration.start();
+#else
+	Q_UNUSED(doClearPictures);
 #endif
 	if (currentState != ADD && currentState != PLAN) {
 		if (!d) {
@@ -588,9 +606,8 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	// data that we have
 	struct divecomputer *currentdc = select_dc(&displayed_dive);
 	Q_ASSERT(currentdc);
-	if (!currentdc || !currentdc->samples) {
-		currentdc = fake_dc(currentdc, false);
-	}
+	if (!currentdc || !currentdc->samples)
+		fake_dc(currentdc);
 
 	bool setpointflag = (currentdc->divemode == CCR) && prefs.pp_graphs.po2 && current_dive;
 	bool sensorflag = setpointflag && prefs.show_ccr_sensors;
@@ -598,6 +615,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	ccrsensor1GasItem->setVisible(sensorflag);
 	ccrsensor2GasItem->setVisible(sensorflag && (currentdc->no_o2sensors > 1));
 	ccrsensor3GasItem->setVisible(sensorflag && (currentdc->no_o2sensors > 2));
+	ocpo2GasItem->setVisible((currentdc->divemode == PSCR) && prefs.show_scr_ocpo2);
 
 	/* This struct holds all the data that's about to be plotted.
 	 * I'm not sure this is the best approach ( but since we are
@@ -705,6 +723,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 		ccrsensor1GasItem->setVisible(prefs.show_ccr_sensors);
 		ccrsensor2GasItem->setVisible(prefs.show_ccr_sensors && (currentdc->no_o2sensors > 1));
 		ccrsensor3GasItem->setVisible(prefs.show_ccr_sensors && (currentdc->no_o2sensors > 1));
+		ocpo2GasItem->setVisible((currentdc->divemode == PSCR) && prefs.show_scr_ocpo2);
 		temperatureItem->setVisible(false);
 	} else {
 		tankItem->setVisible(prefs.tankbar);
@@ -716,6 +735,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 		ccrsensor1GasItem->setVisible(false);
 		ccrsensor2GasItem->setVisible(false);
 		ccrsensor3GasItem->setVisible(false);
+		ocpo2GasItem->setVisible(false);
 	}
 #endif
 	tankItem->setData(dataModel, &plotInfo, &displayed_dive);
@@ -729,7 +749,9 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	struct event *event = currentdc->events;
 	struct event *ev;
 	struct gasmix lastgasmix = *get_gasmix(&displayed_dive, current_dc, 1, &ev, NULL);
+
 	while (event) {
+#ifndef SUBSURFACE_MOBILE
 		// if print mode is selected only draw headings, SP change, gas events or bookmark event
 		if (printMode) {
 			if (empty_string(event->name) ||
@@ -741,6 +763,10 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 				continue;
 			}
 		}
+#else
+		// printMode is always selected for SUBSURFACE_MOBILE due to font problems
+		// BUT events are wanted.
+#endif
 		DiveEventItem *item = new DiveEventItem();
 		item->setHorizontalAxis(timeAxis);
 		item->setVerticalAxis(profileYAxis);
@@ -751,6 +777,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 		eventItems.push_back(item);
 		event = event->next;
 	}
+
 	// Only set visible the events that should be visible
 	Q_FOREACH (DiveEventItem *event, eventItems) {
 		event->setVisible(!event->shouldBeHidden());
@@ -778,7 +805,10 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 		DivePlannerPointsModel *model = DivePlannerPointsModel::instance();
 		model->deleteTemporaryPlan();
 	}
-	plotPictures();
+	if (doClearPictures)
+		clearPictures();
+	else
+		plotPictures();
 #endif
 
 	// OK, how long did this take us? Anything above the second is way too long,
@@ -803,9 +833,8 @@ void ProfileWidget2::dateTimeChanged()
 	emit dateTimeChangedItems();
 }
 
-void ProfileWidget2::actionRequestedReplot(bool triggered)
+void ProfileWidget2::actionRequestedReplot(bool)
 {
-	Q_UNUSED(triggered);
 	settingsChanged();
 }
 
@@ -972,7 +1001,7 @@ void ProfileWidget2::mouseDoubleClickEvent(QMouseEvent *event)
 
 		int minutes = lrint(timeAxis->valueAt(mappedPos) / 60);
 		int milimeters = lrint(profileYAxis->valueAt(mappedPos) / M_OR_FT(1, 1)) * M_OR_FT(1, 1);
-		plannerModel->addStop(milimeters, minutes * 60, -1, 0, true);
+		plannerModel->addStop(milimeters, minutes * 60, -1, 0, true, UNDEF_COMP_TYPE);
 	}
 }
 
@@ -1071,6 +1100,7 @@ void ProfileWidget2::setEmptyState()
 	ccrsensor1GasItem->setVisible(false);
 	ccrsensor2GasItem->setVisible(false);
 	ccrsensor3GasItem->setVisible(false);
+	ocpo2GasItem->setVisible(false);
 #ifndef SUBSURFACE_MOBILE
 	toolTipItem->setVisible(false);
 	diveCeiling->setVisible(false);
@@ -1087,11 +1117,9 @@ void ProfileWidget2::setEmptyState()
 #ifndef SUBSURFACE_MOBILE
 	hideAll(allTissues);
 	hideAll(allPercentages);
-#endif
-	hideAll(eventItems);
-#ifndef SUBSURFACE_MOBILE
 	hideAll(handles);
 #endif
+	hideAll(eventItems);
 	hideAll(gases);
 }
 
@@ -1102,11 +1130,6 @@ void ProfileWidget2::setProfileState()
 		return;
 
 	disconnectTemporaryConnections();
-#ifndef SUBSURFACE_MOBILE
-	connect(DivePictureModel::instance(), SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(plotPictures()));
-	connect(DivePictureModel::instance(), SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(plotPictures()));
-	connect(DivePictureModel::instance(), SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SLOT(plotPictures()));
-#endif
 	/* show the same stuff that the profile shows. */
 
 	emit enableShortcuts();
@@ -1190,6 +1213,7 @@ void ProfileWidget2::setProfileState()
 	ccrsensor1GasItem->setVisible(sensorflag);
 	ccrsensor2GasItem->setVisible(sensorflag && (current_dc->no_o2sensors > 1));
 	ccrsensor3GasItem->setVisible(sensorflag && (current_dc->no_o2sensors > 2));
+	ocpo2GasItem->setVisible(current_dive && (current_dc->divemode == PSCR) && prefs.show_scr_ocpo2);
 
 	heartBeatItem->setVisible(prefs.hrgraph);
 	diveCeiling->setVisible(prefs.calcceiling);
@@ -1401,6 +1425,36 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 	setpointAction->setData(event->globalPos());
 	QAction *action = m.addAction(tr("Add bookmark"), this, SLOT(addBookmark()));
 	action->setData(event->globalPos());
+	struct event *ev = NULL;
+	enum divemode_t divemode = UNDEF_COMP_TYPE;
+	QPointF scenePos = mapToScene(mapFromGlobal(event->globalPos()));
+	QString gas = action->text();
+	qreal sec_val = timeAxis->valueAt(scenePos);
+	int seconds = (sec_val < 0.0) ? 0 : (int)sec_val;
+
+	get_current_divemode(current_dc, seconds, &ev, &divemode);
+	QMenu *changeMode = m.addMenu(tr("Change divemode"));
+	if (divemode != OC) {
+		QAction *action = new QAction(&m);
+		action->setText(gettextFromC::tr(divemode_text_ui[OC]));
+		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
+		action->setData(event->globalPos());
+		changeMode->addAction(action);
+	}
+	if (divemode != CCR) {
+		QAction *action = new QAction(&m);
+		action->setText(gettextFromC::tr(divemode_text_ui[CCR]));
+		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
+		action->setData(event->globalPos());
+		changeMode->addAction(action);
+	}
+	if (divemode != PSCR) {
+		QAction *action = new QAction(&m);
+		action->setText(gettextFromC::tr(divemode_text_ui[PSCR]));
+		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
+		action->setData(event->globalPos());
+		changeMode->addAction(action);
+	}
 
 	if (same_string(current_dc->model, "manually added dive"))
 		m.addAction(tr("Edit the profile"), this, SIGNAL(editCurrentDive()));
@@ -1483,7 +1537,7 @@ void ProfileWidget2::deleteCurrentDC()
 	delete_current_divecomputer();
 	mark_divelist_changed(true);
 	// we need to force it since it's likely the same dive and same dc_number - but that's a different dive computer now
-	plotDive(0, true);
+	plotDive(0, true, false);
 
 	emit refreshDisplay(true);
 }
@@ -1562,6 +1616,20 @@ void ProfileWidget2::addBookmark()
 	replot();
 }
 
+void ProfileWidget2::addDivemodeSwitch()
+{
+	int i;
+	QAction *action = qobject_cast<QAction *>(sender());
+	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
+	for (i = 0; i < UNDEF_COMP_TYPE; i++)
+		if (gettextFromC::tr(divemode_text_ui[i]) == action->text())
+			add_event(current_dc, lrint(timeAxis->valueAt(scenePos)), 8, 0, i,
+				QT_TRANSLATE_NOOP("gettextFromC", "modechange"));
+	invalidate_dive_cache(current_dive);
+	mark_divelist_changed(true);
+	replot();
+}
+
 void ProfileWidget2::addSetpointChange()
 {
 	QAction *action = qobject_cast<QAction *>(sender());
@@ -1707,11 +1775,8 @@ void ProfileWidget2::disconnectTemporaryConnections()
 }
 
 #ifndef SUBSURFACE_MOBILE
-void ProfileWidget2::pointInserted(const QModelIndex &parent, int start, int end)
+void ProfileWidget2::pointInserted(const QModelIndex&, int, int)
 {
-	Q_UNUSED(parent)
-	Q_UNUSED(start)
-	Q_UNUSED(end)
 	DiveHandler *item = new DiveHandler();
 	scene()->addItem(item);
 	handles << item;
@@ -1974,26 +2039,48 @@ void ProfileWidget2::keyEscAction()
 		plannerModel->cancelPlan();
 }
 
+void ProfileWidget2::clearPictures()
+{
+	pictures.clear();
+}
+
+void ProfileWidget2::updatePictures(const QModelIndex &from, const QModelIndex &to)
+{
+	DivePictureModel *m = DivePictureModel::instance();
+	for (int picNr = from.row(); picNr <= to.row(); ++picNr) {
+		int picItemNr = picNr - m->rowDDStart;
+		if (picItemNr < 0 || (size_t)picItemNr >= pictures.size())
+			return;
+		if (!pictures[picItemNr])
+			return;
+
+		pictures[picItemNr]->setPixmap(m->index(picNr, 0).data(Qt::UserRole).value<QPixmap>());
+	}
+}
+
 void ProfileWidget2::plotPictures()
 {
-	Q_FOREACH (DivePictureItem *item, pictures) {
-		item->hide();
-		item->deleteLater();
-	}
-	pictures.clear();
-
-	if (printMode)
-		return;
+	DivePictureModel *m = DivePictureModel::instance();
+	pictures.resize(m->rowDDEnd - m->rowDDStart);
 
 	double x, y, lastX = -1.0, lastY = -1.0;
-	DivePictureModel *m = DivePictureModel::instance();
 	for (int i = m->rowDDStart; i < m->rowDDEnd; i++) {
+		int picItemNr = i - m->rowDDStart;
 		int offsetSeconds = m->index(i, 1).data(Qt::UserRole).value<int>();
 		// it's a correct picture, but doesn't have a timestamp: only show on the widget near the
-		// information area.
-		if (!offsetSeconds)
+		// information area. A null pointer in the pictures array indicates that this picture is not
+		// shown.
+		if (!offsetSeconds) {
+			pictures[picItemNr].reset();
 			continue;
-		DivePictureItem *item = new DivePictureItem();
+		}
+		DivePictureItem *item = pictures[picItemNr].get();
+		if (!item) {
+			item = new DivePictureItem;
+			pictures[picItemNr].reset(item);
+			scene()->addItem(item);
+		}
+		item->setVisible(prefs.show_pictures_in_profile);
 		item->setPixmap(m->index(i, 0).data(Qt::UserRole).value<QPixmap>());
 		item->setFileUrl(m->index(i, 1).data().toString());
 		// let's put the picture at the correct time, but at a fixed "depth" on the profile
@@ -2008,10 +2095,21 @@ void ProfileWidget2::plotPictures()
 		lastX = x;
 		lastY = y;
 		item->setPos(x, y);
-		scene()->addItem(item);
-		pictures.push_back(item);
 	}
 }
+
+void ProfileWidget2::removePictures(const QModelIndex &, int first, int last)
+{
+	DivePictureModel *m = DivePictureModel::instance();
+	first = std::max(0, first - m->rowDDStart);
+	// Note that last points *to* the last item and not *past* the last item,
+	// therefore we add 1 to achieve conventional C++ semantics.
+	last = std::min((int)pictures.size(), last + 1 - m->rowDDStart);
+	if (first >= (int)pictures.size() || last <= first)
+		return;
+	pictures.erase(pictures.begin() + first, pictures.begin() + last);
+}
+
 #endif
 
 void ProfileWidget2::dropEvent(QDropEvent *event)
@@ -2030,14 +2128,14 @@ void ProfileWidget2::dropEvent(QDropEvent *event)
 			if (QString(picture->filename) == filename) {
 				picture->offset.seconds = lrint(timeAxis->valueAt(mappedPos));
 				mark_divelist_changed(true);
+#ifndef SUBSURFACE_MOBILE
+				DivePictureModel::instance()->updateDivePictureOffset(filename, picture->offset.seconds);
+				plotPictures();
+#endif
 				break;
 			}
 		}
 		copy_dive(current_dive, &displayed_dive);
-#ifndef SUBSURFACE_MOBILE
-		DivePictureModel::instance()->updateDivePictures();
-#endif
-
 
 		if (event->source() == this) {
 			event->setDropAction(Qt::MoveAction);
