@@ -3,6 +3,8 @@
 #define PROFILEWIDGET2_H
 
 #include <QGraphicsView>
+#include <vector>
+#include <memory>
 
 // /* The idea of this widget is to display and edit the profile.
 //  * It has:
@@ -71,7 +73,7 @@ public:
 
 	ProfileWidget2(QWidget *parent = 0);
 	void resetZoom();
-	void plotDive(struct dive *d = 0, bool force = false);
+	void plotDive(struct dive *d = 0, bool force = false, bool clearPictures = false);
 	void setupItem(AbstractProfilePolygonItem *item, DiveCartesianAxis *vAxis, int vData, int hData, int zValue);
 	void setPrintMode(bool mode, bool grayscale = false);
 	bool getPrintMode();
@@ -108,11 +110,13 @@ slots: // Necessary to call from QAction's signals.
 	void replot(dive *d = 0);
 #ifndef SUBSURFACE_MOBILE
 	void plotPictures();
+	void removePictures(const QModelIndex &, int first, int last);
 	void setPlanState();
 	void setAddState();
 	void changeGas();
 	void addSetpointChange();
 	void addBookmark();
+	void addDivemodeSwitch();
 	void hideEvents();
 	void unhideEvents();
 	void removeEvent();
@@ -121,6 +125,7 @@ slots: // Necessary to call from QAction's signals.
 	void deleteCurrentDC();
 	void pointInserted(const QModelIndex &parent, int start, int end);
 	void pointsRemoved(const QModelIndex &, int start, int end);
+	void updatePictures(const QModelIndex &from, const QModelIndex &to);
 
 	/* this is called for every move on the handlers. maybe we can speed up this a bit? */
 	void recreatePlannedDive();
@@ -164,6 +169,7 @@ private: /*methods*/
 	void addActionShortcut(const Qt::Key shortcut, void (ProfileWidget2::*slot)());
 	void createPPGas(PartialPressureGasItem *item, int verticalColumn, color_index_t color, color_index_t colorAlert,
 			 double *thresholdSettingsMin, double *thresholdSettingsMax);
+	void clearPictures();
 private:
 	DivePlotDataModel *dataModel;
 	int zoomLevel;
@@ -198,6 +204,7 @@ private:
 	PartialPressureGasItem *ccrsensor1GasItem;
 	PartialPressureGasItem *ccrsensor2GasItem;
 	PartialPressureGasItem *ccrsensor3GasItem;
+	PartialPressureGasItem *ocpo2GasItem;
 #ifndef SUBSURFACE_MOBILE
 	DiveCalculatedCeiling *diveCeiling;
 	DiveTextItem *decoModelParameters;
@@ -217,10 +224,12 @@ private:
 	bool printMode;
 
 	QList<QGraphicsSimpleTextItem *> gases;
-	QList<DivePictureItem *> pictures;
 
 	//specifics for ADD and PLAN
 #ifndef SUBSURFACE_MOBILE
+	// Use std::vector<> and std::unique_ptr<>, because QVector<QScopedPointer<...>> is unsupported.
+	std::vector<std::unique_ptr<DivePictureItem>> pictures;
+
 	QList<DiveHandler *> handles;
 	void repositionDiveHandlers();
 	int fixHandlerIndex(DiveHandler *activeHandler);
