@@ -14,14 +14,12 @@ public:
 		ID,
 		NICKNAME
 	};
-	DiveComputerModel(QMultiMap<QString, DiveComputerNode> &dcMap, QObject *parent = 0);
+	DiveComputerModel(QObject *parent = 0);
 	virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
 	virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
 	virtual Qt::ItemFlags flags(const QModelIndex &index) const;
 	virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
-	void update();
 	void keepWorkingList();
-	void dropWorkingList();
 
 public
 slots:
@@ -29,7 +27,7 @@ slots:
 
 private:
 	int numRows;
-	QMultiMap<QString, DiveComputerNode> dcWorkingMap;
+	QVector<DiveComputerNode> dcs;
 };
 
 #endif
