@@ -10,7 +10,6 @@ struct PictureEntry {
 	struct picture *picture;
 	QString filename;
 	QImage image;
-	QImage imageProfile;	// For the profile widget keep a copy of a constant sized image
 	int offsetSeconds;
 };
 
@@ -23,15 +22,21 @@ public:
 	virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
 	virtual void updateDivePictures();
 	void updateDivePicturesWhenDone(QList<QFuture<void>>);
-	void removePicture(const QString& fileUrl, bool last);
+	void removePictures(const QVector<QString> &fileUrls);
 	int rowDDStart, rowDDEnd;
+	void updateDivePictureOffset(const QString &filename, int offsetSeconds);
 public slots:
 	void setZoomLevel(int level);
+	void updateThumbnail(QString filename, QImage thumbnail);
 private:
 	DivePictureModel();
-	QList<PictureEntry> pictures;
+	QVector<PictureEntry> pictures;
+	int findPictureId(const QString &filename);	// Return -1 if not found
 	double zoomLevel;	// -1.0: minimum, 0.0: standard, 1.0: maximum
+	int size;
+	int defaultSize;
 	void updateThumbnails();
+	void updateZoom();
 };
 
 #endif
