@@ -1,13 +1,13 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "diveplannermodel.h"
-#include "core/dive.h"
-#include "core/helpers.h"
+#include "core/subsurface-string.h"
 #include "qt-models/cylindermodel.h"
 #include "core/planner.h"
 #include "qt-models/models.h"
 #include "core/device.h"
 #include "core/qthelper.h"
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
+#include "core/gettextfromc.h"
 #include <QApplication>
 #include <QTextDocument>
 #include <QtConcurrent>
@@ -46,15 +46,16 @@ void DivePlannerPointsModel::createSimpleDive()
 	// If we're in drop_stone_mode, don't add a first point.
 	// It will be added implicit.
 	if (!prefs.drop_stone_mode)
-		addStop(M_OR_FT(15, 45), 1 * 60, cylinderid, 0, true);
+		addStop(M_OR_FT(15, 45), 1 * 60, cylinderid, 0, true, UNDEF_COMP_TYPE);
 
-	addStop(M_OR_FT(15, 45), 20 * 60, 0, 0, true);
+	addStop(M_OR_FT(15, 45), 20 * 60, 0, 0, true, UNDEF_COMP_TYPE);
 	if (!isPlanner()) {
-		addStop(M_OR_FT(5, 15), 42 * 60, 0, cylinderid, true);
-		addStop(M_OR_FT(5, 15), 45 * 60, 0, cylinderid, true);
+		addStop(M_OR_FT(5, 15), 42 * 60, 0, cylinderid, true, UNDEF_COMP_TYPE);
+		addStop(M_OR_FT(5, 15), 45 * 60, 0, cylinderid, true, UNDEF_COMP_TYPE);
 	}
 	updateMaxDepth();
 	GasSelectionModel::instance()->repopulate();
+	DiveTypeSelectionModel::instance()->repopulate();
 }
 
 void DivePlannerPointsModel::setupStartTime()
@@ -80,6 +81,8 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 	o2pressure_t last_sp;
 	bool oldRec = recalc;
 	struct divecomputer *dc = &(d->dc);
+	struct event *evd = NULL;
+	enum divemode_t current_divemode = UNDEF_COMP_TYPE;
 	recalc = false;
 	CylindersModel::instance()->updateDive();
 	duration_t lasttime = { 0 };
@@ -99,7 +102,7 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 	if (dc->samples)
 		hasMarkedSamples = dc->sample[0].manually_entered;
 	else
-		dc = fake_dc(dc, true);
+		fake_dc(dc);
 
 	// if this dive has more than 100 samples (so it is probably a logged dive),
 	// average samples so we end up with a total of 100 samples.
@@ -112,9 +115,10 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 			break;
 		while (j * plansamples <= i * dc->samples) {
 			const sample &s = dc->sample[j];
+			const sample &prev = dc->sample[j-1];
 			if (s.time.seconds != 0 && (!hasMarkedSamples || s.manually_entered)) {
 				depthsum += s.depth.mm;
-				last_sp = s.setpoint;
+				last_sp = prev.setpoint;
 				++samplecount;
 				newtime = s.time;
 			}
@@ -123,7 +127,8 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 		if (samplecount) {
 			cylinderid = get_cylinderid_at_time(d, dc, lasttime);
 			if (newtime.seconds - lastrecordedtime.seconds > 10) {
-				addStop(depthsum / samplecount, newtime.seconds, cylinderid, last_sp.mbar, true);
+				current_divemode = get_current_divemode(dc, newtime.seconds - 1, &evd, &current_divemode);
+				addStop(depthsum / samplecount, newtime.seconds, cylinderid, last_sp.mbar, true, current_divemode);
 				lastrecordedtime = newtime;
 			}
 			lasttime = newtime;
@@ -132,8 +137,9 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 		}
 	}
 	// make sure we get the last point right so the duration is correct
+	current_divemode = get_current_divemode(dc, d->dc.duration.seconds, &evd, &current_divemode);
 	if (!hasMarkedSamples && !dc->last_manual_time.seconds)
-		addStop(0, d->dc.duration.seconds,cylinderid, last_sp.mbar, true);
+		addStop(0, d->dc.duration.seconds,cylinderid, last_sp.mbar, true, current_divemode);
 	recalc = oldRec;
 	emitDataChanged();
 }
@@ -230,9 +236,8 @@ bool DivePlannerPointsModel::recalcQ()
 	return recalc;
 }
 
-int DivePlannerPointsModel::columnCount(const QModelIndex &parent) const
+int DivePlannerPointsModel::columnCount(const QModelIndex&) const
 {
-	Q_UNUSED(parent);
 	return COLUMNS; // to disable CCSETPOINT subtract one
 }
 
@@ -252,6 +257,8 @@ QVariant DivePlannerPointsModel::data(const QModelIndex &index, int role) const
 				return (p.time - divepoints.at(index.row() - 1).time) / 60;
 			else
 				return p.time / 60;
+		case DIVEMODE:
+			return gettextFromC::tr(divemode_text_ui[p.divemode]);
 		case GAS:
 			/* Check if we have the same gasmix two or more times
 			 * If yes return more verbose string */
@@ -330,6 +337,14 @@ bool DivePlannerPointsModel::setData(const QModelIndex &index, const QVariant &v
 				CylindersModel::instance()->moveAtFirst(value.toInt());
 			CylindersModel::instance()->updateTrashIcon();
 			break;
+		case DIVEMODE:
+			if (value.toInt() < FREEDIVE) {
+				p.divemode = (enum divemode_t) value.toInt();
+				p.setpoint = p.divemode == CCR ? prefs.defaultsetpoint : 0;
+			}
+			if (index.row() == 0)
+				displayed_dive.dc.divemode = (enum divemode_t) value.toInt();
+			break;
 		}
 		editStop(index.row(), p);
 	}
@@ -367,6 +382,8 @@ QVariant DivePlannerPointsModel::headerData(int section, Qt::Orientation orienta
 			return tr("Used gas");
 		case CCSETPOINT:
 			return tr("CC setpoint");
+		case DIVEMODE:
+			return tr("Dive mode");
 		}
 	} else if (role == Qt::FontRole) {
 		return defaultModelFont();
@@ -382,9 +399,8 @@ Qt::ItemFlags DivePlannerPointsModel::flags(const QModelIndex &index) const
 		return QAbstractItemModel::flags(index);
 }
 
-int DivePlannerPointsModel::rowCount(const QModelIndex &parent) const
+int DivePlannerPointsModel::rowCount(const QModelIndex&) const
 {
-	Q_UNUSED(parent);
 	return divepoints.count();
 }
 
@@ -456,9 +472,11 @@ void DivePlannerPointsModel::setGFLow(const int gflow)
 void DivePlannerPointsModel::setRebreatherMode(int mode)
 {
 	int i;
-	displayed_dive.dc.divemode = (dive_comp_type) mode;
-	for (i=0; i < rowCount(); i++)
+	displayed_dive.dc.divemode = (divemode_t) mode;
+	for (i=0; i < rowCount(); i++) {
 		divepoints[i].setpoint = mode == CCR ? prefs.defaultsetpoint : 0;
+		divepoints[i].divemode = (enum divemode_t) mode;
+	}
 	emitDataChanged();
 }
 
@@ -657,7 +675,8 @@ int DivePlannerPointsModel::lastEnteredPoint()
 }
 
 // cylinderid_in == -1 means same gas as before.
-int DivePlannerPointsModel::addStop(int milimeters, int seconds, int cylinderid_in, int ccpoint, bool entered)
+// divemode == UNDEF_COMP_TYPE means determine from previous point.
+int DivePlannerPointsModel::addStop(int milimeters, int seconds, int cylinderid_in, int ccpoint, bool entered, enum divemode_t divemode)
 {
 	int cylinderid = 0;
 	bool usePrevious = false;
@@ -703,10 +722,18 @@ int DivePlannerPointsModel::addStop(int milimeters, int seconds, int cylinderid_
 	if (usePrevious) {
 		if (row  < divepoints.count()) {
 			cylinderid = divepoints.at(row).cylinderid;
+			if (divemode == UNDEF_COMP_TYPE)
+				divemode = divepoints.at(row).divemode;
+			ccpoint = divepoints.at(row).setpoint;
 		} else if (row > 0) {
 			cylinderid = divepoints.at(row - 1).cylinderid;
+			if (divemode == UNDEF_COMP_TYPE)
+				divemode = divepoints.at(row - 1).divemode;
+			ccpoint = divepoints.at(row -1).setpoint;
 		}
 	}
+	if (divemode == UNDEF_COMP_TYPE)
+		divemode = displayed_dive.dc.divemode;
 
 	// add the new stop
 	beginInsertRows(QModelIndex(), row, row);
@@ -716,6 +743,7 @@ int DivePlannerPointsModel::addStop(int milimeters, int seconds, int cylinderid_
 	point.cylinderid = cylinderid;
 	point.setpoint = ccpoint;
 	point.entered = entered;
+	point.divemode = divemode;
 	point.next = NULL;
 	divepoints.append(point);
 	std::sort(divepoints.begin(), divepoints.end(), divePointsLessThan);
@@ -864,11 +892,11 @@ void DivePlannerPointsModel::createTemporaryPlan()
 		lastIndex = i;
 		if (i == 0 && mode == PLAN && prefs.drop_stone_mode) {
 			/* Okay, we add a first segment where we go down to depth */
-			plan_add_segment(&diveplan, p.depth.mm / prefs.descrate, p.depth.mm, p.cylinderid, p.setpoint, true);
+			plan_add_segment(&diveplan, p.depth.mm / prefs.descrate, p.depth.mm, p.cylinderid, p.setpoint, true, p.divemode);
 			deltaT -= p.depth.mm / prefs.descrate;
 		}
 		if (p.entered)
-			plan_add_segment(&diveplan, deltaT, p.depth.mm, p.cylinderid, p.setpoint, true);
+			plan_add_segment(&diveplan, deltaT, p.depth.mm, p.cylinderid, p.setpoint, true, p.divemode);
 	}
 
 	// what does the cache do???
