@@ -6,7 +6,8 @@
  *
  */
 #include "qt-models/models.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
+#include "core/gettextfromc.h"
 
 #include <QLocale>
 
@@ -24,9 +25,8 @@ const QPixmap &trashForbiddenIcon()
 	return trash;
 }
 
-Qt::ItemFlags GasSelectionModel::flags(const QModelIndex &index) const
+Qt::ItemFlags GasSelectionModel::flags(const QModelIndex&) const
 {
-	Q_UNUSED(index);
 	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
 
@@ -67,6 +67,35 @@ QVariant GasSelectionModel::data(const QModelIndex &index, int role) const
 	}
 	return QStringListModel::data(index, role);
 }
+// Dive Type Model for the divetype combo box
+
+Qt::ItemFlags DiveTypeSelectionModel::flags(const QModelIndex&) const
+{
+	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
+}
+
+DiveTypeSelectionModel *DiveTypeSelectionModel::instance()
+{
+	static DiveTypeSelectionModel self;
+	return &self;
+}
+
+void DiveTypeSelectionModel::repopulate()
+{
+	QStringList modes = QStringList();
+	for (int i = 0; i < FREEDIVE; i++)
+		modes.append(gettextFromC::tr(divemode_text_ui[i]));
+	setStringList(modes);
+}
+
+QVariant DiveTypeSelectionModel::data(const QModelIndex &index, int role) const
+{
+	if (role == Qt::FontRole) {
+		return defaultModelFont();
+	}
+	return QStringListModel::data(index, role);
+}
+
 
 // Language Model, The Model to populate the list of possible Languages.
 
@@ -104,8 +133,7 @@ QVariant LanguageModel::data(const QModelIndex &index, int role) const
 	return QVariant();
 }
 
-int LanguageModel::rowCount(const QModelIndex &parent) const
+int LanguageModel::rowCount(const QModelIndex&) const
 {
-	Q_UNUSED(parent);
 	return languages.count();
 }
