@@ -40,6 +40,9 @@ while [[ $# -gt 0 ]] ; do
 			;;
 		-mobile)
 			# we are building Subsurface-mobile
+			# Note that this will run natively on the host OS.
+			# To cross build for Android or iOS (including simulator) 
+			# use the scripts in packaging/xxx
 			BUILD_MOBILE="1"
 			;;
 		-desktop)
@@ -66,15 +69,6 @@ while [[ $# -gt 0 ]] ; do
 	shift
 done
 
-# most of these will only be needed with -build-deps on a Mac
-CURRENT_LIBZIP="1.2.0"
-CURRENT_HIDAPI="hidapi-0.7.0"
-CURRENT_LIBCURL="curl-7_54_1"
-CURRENT_LIBUSB="v1.0.21"
-CURRENT_OPENSSL="OpenSSL_1_1_0f"
-CURRENT_LIBSSH2="libssh2-1.8.0"
-CURRENT_LIBGIT2="v0.26.0"
-
 # Verify that the Xcode Command Line Tools are installed
 if [ $PLATFORM = Darwin ] ; then
 	if [ -d /Developer/SDKs ] ; then
@@ -149,7 +143,7 @@ fi
 # QtPositioning headers aren't bundled. Download them if necessary.
 if [ $PLATFORM = Linux ] ; then
 	QT_HEADERS_PATH=`$QMAKE -query QT_INSTALL_HEADERS`
-	QT_VERSION=`$QMAKE -v | grep "Qt" | cut -d" " -f4`
+	QT_VERSION=`$QMAKE -query QT_VERSION`
 
 	if [ ! -d "$QT_HEADERS_PATH/QtLocation/$QT_VERSION/QtLocation/private" ] &&
            [ ! -d $INSTALL_ROOT/include/QtLocation/private ] ; then
@@ -187,142 +181,58 @@ if [ $PLATFORM = Darwin ] ; then
 else
 	SH_LIB_EXT=so
 
-	# check if we need to build libgit2 (and do so if necessary)
-
 	LIBGIT_ARGS=" -DLIBGIT2_DYNAMIC=ON "
-	LIBGIT=$(ldconfig -p | grep libgit2\\.so\\. | awk -F. '{ print $NF }')
+	# check if we need to build libgit2 (and do so if necessary)
+	# first check pkgconfig (that will capture our own local build if
+	# this script has been run before)
+	if pkg-config --exists libgit2 ; then
+		LIBGIT=$(pkg-config --modversion libgit2 | cut -d. -f2)
+	fi
+	if [[ "$LIBGIT" < "24" ]] ; then
+		# maybe there's a system version that's new enough?
+		LIBGIT=$(ldconfig -p | grep libgit2\\.so\\. | awk -F. '{ print $NF }')
+	fi
 fi
 
-if [[ $PLATFORM = Darwin || "$LIBGIT" < "24" ]] ; then
+if [[ $PLATFORM = Darwin && "$BUILD_DEPS" == "1" ]] ; then
 	# when building distributable binaries on a Mac, we cannot rely on anything from Homebrew,
 	# because that always requires the latest OS (how stupid is that - and they consider it a
 	# feature). So we painfully need to build the dependencies ourselves.
+	cd $SRC
+	./subsurface/scripts/get-dep-lib.sh single . libcurl
+	pushd libcurl
+	bash ./buildconf
+	mkdir -p build
+	cd build
+	CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT --with-darwinssl \
+		--disable-tftp --disable-ftp --disable-ldap --disable-ldaps --disable-imap --disable-pop3 --disable-smtp --disable-gopher --disable-smb --disable-rtsp
+	make -j4
+	make install
+	popd
 
-	if [ "$BUILD_DEPS" == "1" ] ; then
-		if [ ! -d libzip-${CURRENT_LIBZIP} ] ; then
-			curl -O https://libzip.org/download/libzip-${CURRENT_LIBZIP}.tar.gz
-			tar xzf libzip-${CURRENT_LIBZIP}.tar.gz
-		fi
-		cd libzip-${CURRENT_LIBZIP}
-		mkdir -p build
-		cd build
-		../configure CFLAGS="$OLDER_MAC" --prefix=$INSTALL_ROOT
-		make -j4
-		make install
-
-		cd $SRC
-
-		if [ ! -d hidapi ] ; then
-			git clone https://github.com/signal11/hidapi
-		fi
-		cd hidapi
-		# there is no good tag, so just build master
-		bash ./bootstrap
-		mkdir -p build
-		cd build
-		CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT
-		make -j4
-		make install
-
-		cd $SRC
-
-		if [ ! -d libcurl ] ; then
-			git clone https://github.com/curl/curl libcurl
-		fi
-		cd libcurl
-		if ! git checkout $CURRENT_LIBCURL ; then
-			echo "Can't find the right tag in libcurl - giving up"
-			exit 1
-		fi
-		bash ./buildconf
-		mkdir -p build
-		cd build
-		CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT --with-darwinssl \
-			--disable-tftp --disable-ftp --disable-ldap --disable-ldaps --disable-imap --disable-pop3 --disable-smtp --disable-gopher --disable-smb --disable-rtsp
-		make -j4
-		make install
-
-		cd $SRC
-
-		if [ ! -d libusb ] ; then
-			git clone https://github.com/libusb/libusb
-		fi
-		cd libusb
-		if ! git checkout $CURRENT_LIBUSB ; then
-			echo "Can't find the right tag in libusb - giving up"
-			exit 1
-		fi
-		bash ./bootstrap.sh
-		mkdir -p build
-		cd build
-		CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT --disable-examples
-		make -j4
-		make install
-
-		cd $SRC
-
-		if [ ! -d openssl ] ; then
-			git clone https://github.com/openssl/openssl
-		fi
-		cd openssl
-		if ! git checkout $CURRENT_OPENSSL ; then
-			echo "Can't find the right tag in openssl - giving up"
-			exit 1
-		fi
-		mkdir -p build
-		cd build
-		../Configure --prefix=$INSTALL_ROOT --openssldir=$INSTALL_ROOT $OLDER_MAC darwin64-x86_64-cc
-		make depend
-		# all the tests fail because the assume that openssl is already installed. Odd? Still thinks work
-		make -j4 -k
-		make -k install
-
-		cd $SRC
-
-		if [ ! -d libssh2 ] ; then
-			git clone https://github.com/libssh2/libssh2
-		fi
-		cd libssh2
-		if ! git checkout $CURRENT_LIBSSH2 ; then
-			echo "Can't find the right tag in libssh2 - giving up"
-			exit 1
-		fi
-		mkdir -p build
-		cd build
-		cmake $OLDER_MAC_CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_ROOT -DCMAKE_BUILD_TYPE=Release -DBUILD_SHARED_LIBS=ON -DBUILD_TESTING=OFF -DBUILD_EXAMPLES=OFF ..
-		make -j4
-		make install
-	else
-		# we are getting libusb and hidapi from pkg-config and that goes wrong
-		# or more specifically, the way libdivecomputer references
-		# the include files goes wrong
-		pkg-config --exists libusb-1.0 && LIBDC_CFLAGS=-I$(dirname $(pkg-config --cflags libusb-1.0 | sed -e 's/^-I//'))
-		pkg-config --exists hidapi && LIBDC_CFLAGS="${LIBDC_CFLAGS} -I$(dirname $(pkg-config --cflags hidapi | sed -e 's/^-I//'))"
-	fi
+	./subsurface/scripts/get-dep-lib.sh single . libssh2
+	pushd libssh2
+	mkdir -p build
+	cd build
+	cmake $OLDER_MAC_CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_ROOT -DCMAKE_BUILD_TYPE=Release -DBUILD_SHARED_LIBS=ON -DBUILD_TESTING=OFF -DBUILD_EXAMPLES=OFF ..
+	make -j4
+	make install
+	popd
+fi
 
+if [[ "$LIBGIT" < "24" ]] ; then
 	LIBGIT_ARGS=" -DLIBGIT2_INCLUDE_DIR=$INSTALL_ROOT/include -DLIBGIT2_LIBRARIES=$INSTALL_ROOT/lib/libgit2.$SH_LIB_EXT "
 
 	cd $SRC
 
-	if [ ! -d libgit2 ] ; then
-		if [[ $1 = local ]] ; then
-			git clone $SRC/../libgit2 libgit2
-		else
-			git clone https://github.com/libgit2/libgit2.git
-		fi
-	fi
-	cd libgit2
-	# let's build with a recent enough version of master for the latest features
-	git fetch origin
-	if ! git checkout $CURRENT_LIBGIT2 ; then
-		echo "Can't find the right tag in libgit2 - giving up"
-		exit 1
-	fi
+	./subsurface/scripts/get-dep-lib.sh single . libgit2
+	pushd libgit2
 	mkdir -p build
 	cd build
 	cmake $OLDER_MAC_CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_ROOT -DCMAKE_BUILD_TYPE=Release -DBUILD_CLAR=OFF ..
 	make -j4
 	make install
+	popd
 
 	if [ $PLATFORM = Darwin ] ; then
 		# in order for macdeployqt to do its job correctly, we need the full path in the dylib ID
@@ -334,6 +244,59 @@ if [[ $PLATFORM = Darwin || "$LIBGIT" < "24" ]] ; then
 	fi
 fi
 
+if [[ $PLATFORM = Darwin && "$BUILD_DEPS" == "1" ]] ; then
+	# when building distributable binaries on a Mac, we cannot rely on anything from Homebrew,
+	# because that always requires the latest OS (how stupid is that - and they consider it a
+	# feature). So we painfully need to build the dependencies ourselves.
+	cd $SRC
+	./subsurface/scripts/get-dep-lib.sh single . libzip
+	pushd libzip
+	mkdir -p build
+	cd build
+	../configure CFLAGS="$OLDER_MAC" --prefix=$INSTALL_ROOT
+	make -j4
+	make install
+	popd
+
+	./subsurface/scripts/get-dep-lib.sh single . hidapi
+	pushd hidapi
+	# there is no good tag, so just build master
+	bash ./bootstrap
+	mkdir -p build
+	cd build
+	CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT
+	make -j4
+	make install
+	popd
+
+	./subsurface/scripts/get-dep-lib.sh single . libusb
+	pushd libusb
+	bash ./bootstrap.sh
+	mkdir -p build
+	cd build
+	CFLAGS="$OLDER_MAC" ../configure --prefix=$INSTALL_ROOT --disable-examples
+	make -j4
+	make install
+	popd
+
+	./subsurface/scripts/get-dep-lib.sh single . openssl
+	pushd openssl
+	mkdir -p build
+	cd build
+	../Configure --prefix=$INSTALL_ROOT --openssldir=$INSTALL_ROOT $OLDER_MAC darwin64-x86_64-cc
+	make depend
+	# all the tests fail because the assume that openssl is already installed. Odd? Still thinks work
+	make -j4 -k
+	make -k install
+	popd
+else
+	# we are getting libusb and hidapi from pkg-config and that goes wrong
+	# or more specifically, the way libdivecomputer references
+	# the include files goes wrong
+	pkg-config --exists libusb-1.0 && LIBDC_CFLAGS=-I$(dirname $(pkg-config --cflags libusb-1.0 | sed -e 's/^-I//'))
+	pkg-config --exists hidapi && LIBDC_CFLAGS="${LIBDC_CFLAGS} -I$(dirname $(pkg-config --cflags hidapi | sed -e 's/^-I//'))"
+fi
+
 
 cd $SRC
 
@@ -359,6 +322,16 @@ if [ ! -f ../configure ] ; then
 	autoreconf --install ..
 fi
 CFLAGS="$OLDER_MAC -I$INSTALL_ROOT/include $LIBDC_CFLAGS" ../configure --prefix=$INSTALL_ROOT --disable-examples
+if [ $PLATFORM = Darwin ] ; then
+	# it seems that on my Mac some of the configure tests for libdivecomputer
+	# pass even though the feature tested for is actually missing
+	# let's hack around that
+	sed -i .bak 's/^#define HAVE_CLOCK_GETTIME 1/\/* #undef HAVE_CLOCK_GETTIME *\//' config.h
+	for i in $(find . -name Makefile)
+	do
+		sed -i .bak 's/-Wrestrict//;s/-Wno-unused-but-set-variable//' $i
+	done
+fi
 make -j4
 make install
 
@@ -443,6 +416,7 @@ if [ "$SKIP_GOOGLEMAPS" != "1" ] ; then
 	git pull --rebase
 
 	mkdir -p build
+	mkdir -p J10build
 	cd build
 	$QMAKE -query
 	$QMAKE "INCLUDEPATH=$INSTALL_ROOT/include" ../googlemaps.pro
