@@ -1,10 +1,9 @@
 // SPDX-License-Identifier: GPL-2.0
 /* qt-gui.cpp */
 /* Qt UI implementation */
-#include "core/dive.h"
 #include "core/display.h"
 #include "desktop-widgets/mainwindow.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 #include "core/pluginmanager.h"
 
 #include <QApplication>
