@@ -1,9 +1,8 @@
 // SPDX-License-Identifier: GPL-2.0
 /* qt-gui.cpp */
 /* Qt UI implementation */
-#include "core/dive.h"
 #include "core/display.h"
-#include "core/helpers.h"
+#include "core/qthelper.h"
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -18,9 +17,10 @@
 #include <QQmlContext>
 #include <QSortFilterProxyModel>
 #include "mobile-widgets/qmlmanager.h"
+#include "mobile-widgets/qmlprefs.h"
 #include "qt-models/divelistmodel.h"
 #include "qt-models/gpslistmodel.h"
-#include "mobile-widgets/qmlprofile.h"
+#include "profile-widget/qmlprofile.h"
 #include "core/downloadfromdcthread.h"
 #include "core/connectionlistmodel.h"
 #include "qt-models/diveimportedmodel.h"
@@ -30,6 +30,8 @@
 
 #include "mobile-widgets/qml/kirigami/src/kirigamiplugin.h"
 
+#include "core/ssrf.h"
+
 QObject *qqWindowObject = NULL;
 
 void set_non_bt_addresses() {
@@ -52,10 +54,11 @@ void init_ui()
 
 void run_ui()
 {
+	LOG_STP("run_ui starting");
 	qmlRegisterType<QMLManager>("org.subsurfacedivelog.mobile", 1, 0, "QMLManager");
+	qmlRegisterType<QMLPrefs>("org.subsurfacedivelog.mobile", 1, 0, "QMLPrefs");
 	qmlRegisterType<QMLProfile>("org.subsurfacedivelog.mobile", 1, 0, "QMLProfile");
 
-	qmlRegisterType<DCDeviceData>("org.subsurfacedivelog.mobile", 1, 0, "DCDeviceData");
 	qmlRegisterType<DownloadThread>("org.subsurfacedivelog.mobile", 1, 0, "DCDownloadThread");
 	qmlRegisterType<DiveImportedModel>("org.subsurfacedivelog.mobile", 1, 0, "DCImportModel");
 
@@ -64,6 +67,7 @@ void run_ui()
 	qmlRegisterType<MapLocation>("org.subsurfacedivelog.mobile", 1, 0, "MapLocation");
 
 	QQmlApplicationEngine engine;
+	LOG_STP("run_ui qml engine started");
 	KirigamiPlugin::getInstance().registerTypes();
 #if defined(__APPLE__) && !defined(Q_OS_IOS)
 	// when running the QML UI on a Mac the deployment of the QML Components seems
@@ -80,11 +84,13 @@ void run_ui()
 #endif
 	engine.addImportPath("qrc://imports");
 	DiveListModel diveListModel;
+	LOG_STP("run_ui diveListModel started");
 	DiveListSortModel *sortModel = new DiveListSortModel(0);
 	sortModel->setSourceModel(&diveListModel);
 	sortModel->setDynamicSortFilter(true);
 	sortModel->setSortRole(DiveListModel::DiveDateRole);
 	sortModel->sort(0, Qt::DescendingOrder);
+	LOG_STP("run_ui diveListModel sorted");
 	GpsListModel gpsListModel;
 	QSortFilterProxyModel *gpsSortModel = new QSortFilterProxyModel(0);
 	gpsSortModel->setSourceModel(&gpsListModel);
@@ -96,15 +102,13 @@ void run_ui()
 	ctxt->setContextProperty("gpsModel", gpsSortModel);
 	ctxt->setContextProperty("vendorList", vendorList);
 	set_non_bt_addresses();
+	LOG_STP("run_ui set_non_bt_adresses");
 
 	ctxt->setContextProperty("connectionListModel", &connectionListModel);
 	ctxt->setContextProperty("logModel", MessageHandlerModel::self());
 
-	// call again to be able to log
-	// FIXME - this is redundant - but otherwise they don't end up in the AppLog
-	fill_computer_list();
-
 	engine.load(QUrl(QStringLiteral("qrc:///qml/main.qml")));
+	LOG_STP("run_ui qml loaded");
 	qqWindowObject = engine.rootObjects().value(0);
 	if (!qqWindowObject) {
 		fprintf(stderr, "can't create window object\n");
@@ -117,6 +121,11 @@ void run_ui()
 	QScreen *screen = qml_window->screen();
 	QObject::connect(qml_window, &QQuickWindow::screenChanged, QMLManager::instance(), &QMLManager::screenChanged);
 	QMLManager *manager = QMLManager::instance();
+	LOG_STP("run_ui qmlmanager instance started");
+	// now that the log file is initialized...
+	show_computer_list();
+	LOG_STP("run_ui show_computer_list");
+
 	manager->setDevicePixelRatio(qml_window->devicePixelRatio(), qml_window->screen());
 	manager->dlSortModel = sortModel;
 	manager->screenChanged(screen);
@@ -126,6 +135,7 @@ void run_ui()
 	qml_window->setWidth(800);
 #endif
 	qml_window->show();
+	LOG_STP("run_ui running exec");
 	qApp->exec();
 }
 
