@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "testparse.h"
 #include "core/dive.h"
+#include "core/subsurface-string.h"
 #include "core/parse.h"
 #include "core/file.h"
 #include "core/divelist.h"
@@ -203,7 +204,7 @@ void TestParse::testParseHUDC()
 	params[pnr++] = intdup(-1);
 	params[pnr++] = strdup("pressureField");
 	params[pnr++] = intdup(-1);
-	params[pnr++] = strdup("setpointFiend");
+	params[pnr++] = strdup("setpointField");
 	params[pnr++] = intdup(-1);
 	params[pnr++] = strdup("separatorIndex");
 	params[pnr++] = intdup(2);
@@ -491,7 +492,7 @@ void TestParse::parseDL7()
 	params[pnr++] = intdup(-1);
 	params[pnr++] = strdup("pressureField");
 	params[pnr++] = intdup(-1);
-	params[pnr++] = strdup("setpointFiend");
+	params[pnr++] = strdup("setpointField");
 	params[pnr++] = intdup(-1);
 	params[pnr++] = strdup("separatorIndex");
 	params[pnr++] = intdup(3);
